@@ -60,6 +60,7 @@ main (gint argc, gchar *argv[])
   gchar *full_cmd_line = NULL;
   gchar *runconf_cmd_line = NULL;
   gboolean is_de = FALSE;
+  NcmRNG *rng = ncm_rng_pool_get ("darkenergy");
 
   ncm_cfg_init ();
 
@@ -187,7 +188,8 @@ main (gint argc, gchar *argv[])
   model = nc_hicosmo_new_from_name (NC_TYPE_HICOSMO, de_model.model_name);
   mset = ncm_mset_new (NCM_MODEL (model), NULL);
   dist = nc_distance_new (2.0);
-  ncm_cfg_set_named_instance (dist, "MainDist", FALSE);
+  
+  ncm_serialize_global_set (dist, "MainDist", FALSE);
 
   if (g_type_is_a (G_OBJECT_TYPE (model), NC_TYPE_HICOSMO_DE))
     is_de = TRUE;
@@ -243,11 +245,15 @@ main (gint argc, gchar *argv[])
     }
     else if (snia_id->value >= NC_DATA_SNIA_COV_START && snia_id->value <= NC_DATA_SNIA_COV_END)
     {
-      NcSNIADistCov *dcov = nc_snia_dist_cov_new (dist);
+      NcSNIADistCov *dcov;
       NcmData *data;
 
-      if (de_data_simple.snia_prop != NULL)
-        ncm_cfg_object_set_property (G_OBJECT (dcov), de_data_simple.snia_prop);
+      if (de_data_simple.snia_objser == NULL)
+        dcov = nc_snia_dist_cov_new (dist);
+      else
+        dcov = NC_SNIA_DIST_COV (ncm_serialize_global_from_string (de_data_simple.snia_objser));
+
+      g_assert (NC_IS_SNIA_DIST_COV (dcov));
 
       data = nc_data_snia_cov_new (de_data_simple.snia_use_det);
       nc_data_snia_load_cat (NC_DATA_SNIA_COV (data), snia_id->value);
@@ -340,10 +346,11 @@ main (gint argc, gchar *argv[])
                                                                      de_data_simple.cluster_id);
     if (cluster_id != NULL)
     {
-      ca_array = nc_de_data_cluster_new (dist, mset, &de_data_cluster, dset, cluster_id->value);
+      ca_array = nc_de_data_cluster_new (dist, mset, &de_data_cluster, dset, cluster_id->value, rng);
     }
     else
       g_error ("Cluster sample '%s' not found run --cluster-list to list the available options", de_data_simple.cluster_id);
+
   }
 
   if (de_data_simple.BBN)
@@ -356,14 +363,6 @@ main (gint argc, gchar *argv[])
     ncm_mset_func_free (Omega_bh2);
   }
 
-  if (de_data_simple.H0_Hst)
-  {
-    if (ncm_mset_param_get_ftype (mset, nc_hicosmo_id (), NC_HICOSMO_DE_H0) == NCM_PARAM_TYPE_FIXED)
-      g_warning ("A prior on H0 was required but H0 was kept fixed. This prior will be useless.");
-
-    ncm_prior_add_gaussian_data (lh, nc_hicosmo_id (), NC_HICOSMO_DE_H0, 73.8, 2.4);
-  }
-
   if (de_fit.qspline_cp && TRUE)
   {
     if (!NC_IS_HICOSMO_QSPLINE (model))
@@ -371,15 +370,15 @@ main (gint argc, gchar *argv[])
     else
     {
       NcHICosmoQSplineContPrior *qspline_cp = 
-        nc_hicosmo_qspline_add_continuity_priors (NC_HICOSMO_QSPLINE (model), lh, de_fit.qspline_cp_sigma);
+        nc_hicosmo_qspline_add_continuity_priors (NC_HICOSMO_QSPLINE (model), lh, 1.0e-10, de_fit.qspline_cp_sigma);
       nc_hicosmo_qspline_cont_prior_free (qspline_cp);
     }
   }
 
-  if (de_fit.save_mset != NULL)
-    ncm_mset_save (mset, de_fit.save_mset, TRUE);
   if (de_fit.fiducial != NULL)
-    fiduc = ncm_mset_load (de_fit.fiducial); 
+    fiduc = ncm_mset_load (de_fit.fiducial);
+  else
+    fiduc = ncm_mset_ref (mset);
 
   {
     const GEnumValue *fit_type_id = ncm_cfg_get_enum_by_id_name_nick (NCM_TYPE_FIT_TYPE, de_fit.fit_type);
@@ -390,6 +389,8 @@ main (gint argc, gchar *argv[])
       g_error ("Fit type '%s' not found run --fit-list to list the available options", de_fit.fit_diff);
 
     fit = ncm_fit_new (fit_type_id->value, de_fit.fit_algo, lh, mset, fit_diff_id->value);
+    ncm_fit_set_m2lnL_reltol (fit, de_fit.fit_reltol);
+    ncm_fit_set_params_reltol (fit, de_fit.fit_params_reltol);
   }
 
   if (de_fit.qspline_cp && FALSE)
@@ -404,14 +405,81 @@ main (gint argc, gchar *argv[])
     }
   }
 
+  if (de_data_simple.priors_gauss != NULL)
+  {
+    guint i;
+    guint npriors = g_strv_length (de_data_simple.priors_gauss);
+    
+    for (i = 0; i < npriors; i++)
+    {
+      GError *error = NULL;
+      gchar *priors_str = de_data_simple.priors_gauss[i];
+      GVariant *prior_hash = g_variant_parse (G_VARIANT_TYPE ("a{sv}"), priors_str, NULL, NULL, &error);
+      if (prior_hash == NULL)
+        g_error ("Invalid prior string: %s.", error->message);
+
+      {
+        NcmMSetPIndex p_i;
+        gchar *model_ns = NULL;
+        gchar *p_name = NULL;
+        gdouble mu, sigma;
+        
+        if (!g_variant_lookup (prior_hash, "model", "s", &model_ns))
+          g_error ("Prior must contain `model' key.");
+        
+        p_i.mid = ncm_mset_get_id_by_ns (model_ns);
+        if (p_i.mid < 0)
+          g_error ("Model %s not found.", model_ns);
+
+        if (ncm_mset_peek (mset, p_i.mid) == NULL)
+          g_error ("Model `%s' not present in NcmMSet.", model_ns);
+
+        if (!g_variant_lookup (prior_hash, "param", "s", &p_name))
+          g_error ("Prior must contain `param' key.");
+
+        if (!ncm_model_param_index_from_name (ncm_mset_peek (mset, p_i.mid), p_name, &p_i.pid))
+          g_error ("Parameter `%s' not found in model `%s'.", p_name, model_ns);
+        
+        if (!g_variant_lookup (prior_hash, "mean", "d", &mu))
+          g_error ("Prior must contain `mean' key.");
+        
+        if (!g_variant_lookup (prior_hash, "sigma", "d", &sigma))
+          g_error ("Prior must contain `sigma' key.");
+
+        g_assert_cmpfloat (sigma, >, 0.0);
+
+        ncm_prior_add_gaussian_data (lh, p_i.mid, p_i.pid, mu, sigma);
+        g_variant_unref (prior_hash);
+        g_free (model_ns);
+        g_free (p_name);
+      }
+    }
+  }
+
+
   
+  /* All initializations done! */
+
+  if (de_fit.resample)
+  {
+    NcmRNG *resample_rng;
+    if (de_fit.mc_seed > -1)
+      resample_rng = ncm_rng_seeded_new (NULL, de_fit.mc_seed);
+    else
+      resample_rng = ncm_rng_new (NULL);
+
+    ncm_cfg_msg_sepa ();
+    ncm_message ("# Resampling from fiducial model.\n");
+    ncm_dataset_resample (dset, fiduc, resample_rng);
+    ncm_rng_free (resample_rng);
+  }
+
   de_fit.fisher = (de_fit.fisher || (de_fit.nsigma_fisher != -1) || (de_fit.nsigma != -1) || (de_fit.onedim_cr != NULL));
   de_fit.fit = (de_fit.fit || de_fit.fisher);
   de_fit.save_best_fit = (de_fit.save_best_fit || de_fit.save_fisher);
 
   if (de_fit.fit)
   {
-    fit->params_reltol = 1e-5;
     ncm_fit_set_maxiter (fit, de_fit.max_iter);
     ncm_fit_run (fit, de_fit.msg_level);
     
@@ -456,27 +524,76 @@ main (gint argc, gchar *argv[])
 
   if (de_fit.montecarlo > 0)
   {
-    NcmMSet *resample_mset;
-    NcmFitMC *mc = ncm_fit_mc_new (fit);
-    gdouble m2lnL = fit->fstate->m2lnL;
-
-    if (de_fit.fiducial != NULL)
-      resample_mset = fiduc;
-    else
-      resample_mset = fit->mset;
-
-    ncm_fit_mc_run (mc, resample_mset, de_fit.mc_ni, de_fit.montecarlo, de_fit.msg_level);
-    ncm_fit_mc_mean_covar (mc);
-    ncm_fit_mc_gof_pdf (mc);
-    ncm_fit_log_covar (fit);
-    ncm_fit_mc_gof_pdf_print (mc);
+    if (de_fit.mcbs <= 0)
     {
-      gdouble p_value = ncm_fit_mc_gof_pdf_pvalue (mc, m2lnL, FALSE);
-      ncm_message ("#   - pvalue for fitted model [% 20.15g] %04.2f%%.\n#\n", m2lnL, 100.0 * p_value);
+      NcmFitMC *mc = ncm_fit_mc_new (fit, de_fit.mc_rtype, de_fit.msg_level);
+      gdouble m2lnL = ncm_fit_state_get_m2lnL_curval (fit->fstate);
+
+      if (de_fit.fiducial != NULL)
+        ncm_fit_mc_set_fiducial (mc, fiduc);
+
+      if (de_fit.mc_nthreads > 1)
+        ncm_fit_mc_set_nthreads (mc, de_fit.mc_nthreads);
+
+      if (de_fit.mc_seed > -1)
+      {
+        NcmRNG *mc_rng = ncm_rng_seeded_new (NULL, de_fit.mc_seed);
+        ncm_fit_mc_set_rng (mc, mc_rng);
+        ncm_rng_free (mc_rng);
+      }
+      
+      if (de_fit.mc_data != NULL)
+        ncm_fit_mc_set_data_file (mc, de_fit.mc_data);
+      
+      ncm_fit_mc_start_run (mc);
+      
+      if (de_fit.mc_ni >= 0)
+        ncm_fit_mc_set_first_sample_id (mc, de_fit.mc_ni);
+
+      ncm_fit_mc_run_lre (mc, de_fit.montecarlo, de_fit.mc_lre);
+      ncm_fit_mc_end_run (mc);
+      
+      ncm_fit_mc_mean_covar (mc);
+      ncm_fit_catalog_param_pdf (mc->fcat, 0);
+      ncm_fit_log_covar (fit);
+      {
+        gdouble p_value = ncm_fit_catalog_param_pdf_pvalue (mc->fcat, m2lnL, FALSE);
+        ncm_message ("#   - pvalue for fitted model [% 20.15g] %04.2f%%.\n#\n", m2lnL, 100.0 * p_value);
+      }
+
+      ncm_fit_mc_clear (&mc);
+    }
+    else
+    {
+      NcmMSet *resample_mset;
+      NcmFitMCBS *mcbs = ncm_fit_mcbs_new (fit);
+      gdouble m2lnL = ncm_fit_state_get_m2lnL_curval (fit->fstate);
+
+      if (de_fit.fiducial != NULL)
+        resample_mset = fiduc;
+      else
+        resample_mset = fit->mset;
+
+      if (de_fit.mc_seed > -1)
+      {
+        NcmRNG *mc_rng = ncm_rng_seeded_new (NULL, de_fit.mc_seed);
+        ncm_fit_mcbs_set_rng (mcbs, mc_rng);
+        ncm_rng_free (mc_rng);
+      }
+      
+      if (de_fit.mc_data != NULL)
+        ncm_fit_mcbs_set_filename (mcbs, de_fit.mc_data);
+
+      ncm_fit_mcbs_run (mcbs, resample_mset, de_fit.mc_ni, de_fit.montecarlo, de_fit.mcbs, de_fit.mc_rtype, de_fit.msg_level, de_fit.mc_nthreads);
+
+      ncm_fit_catalog_param_pdf (mcbs->fcat, 0);
+      ncm_fit_log_covar (fit);
+      {
+        gdouble p_value = ncm_fit_catalog_param_pdf_pvalue (mcbs->fcat, m2lnL, FALSE);
+        ncm_message ("#   - pvalue for fitted model [% 20.15g] %04.2f%%.\n#\n", m2lnL, 100.0 * p_value);
+      }
+      ncm_fit_mcbs_clear (&mcbs);
     }
-    
-    if (de_fit.mc_data)
-      ncm_fit_mc_print (mc);
   }
 
   if (de_fit.onedim_cr != NULL)
@@ -514,16 +631,6 @@ main (gint argc, gchar *argv[])
     }
   }
 
-  if (de_fit.resample)
-  {
-    ncm_message ("# Resample from bestfit values\n");
-    ncm_dataset_resample (dset, mset);
-    ncm_fit_run (fit, de_fit.msg_level);
-    ncm_fit_log_info (fit);
-    ncm_fit_numdiff_m2lnL_covar (fit);
-    ncm_fit_log_covar (fit);
-  }
-
   if (de_fit.nsigma >= 0 && (de_fit.bidim_cr[0] != -1) && (de_fit.bidim_cr[1] != -1))
   {
     NcmLHRatio2d *lhr2d;
@@ -724,6 +831,9 @@ main (gint argc, gchar *argv[])
     g_ptr_array_free (ca_array, TRUE);
   }
 
+  if (de_fit.save_mset != NULL)
+    ncm_mset_save (mset, de_fit.save_mset, TRUE);
+
   g_free (de_model_entries); de_model_entries = NULL;
   g_free (de_data_simple_entries); de_data_simple_entries = NULL;
   g_free (de_data_cluster_entries); de_data_cluster_entries = NULL;
@@ -732,12 +842,13 @@ main (gint argc, gchar *argv[])
   if (fiduc != NULL)
     ncm_mset_free (fiduc);
   
-  ncm_cfg_free_named_instances ();
+  ncm_serialize_global_reset ();
   ncm_model_free (NCM_MODEL (model));
   ncm_mset_free (mset);
   ncm_fit_free (fit);
   ncm_likelihood_free (lh);
   ncm_dataset_free (dset);
+  ncm_rng_free (rng);
   nc_distance_free (dist);
   g_free (full_cmd_line);
 
