@@ -8,11 +8,11 @@
 /*
  * numcosmo
  * Copyright (C) Mariana Penna Lima 2012 <pennalima@gmail.com>
-   * numcosmo is free software: you can redistribute it and/or modify it
+ * numcosmo is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
-   *
+ *
  * numcosmo is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
@@ -36,14 +36,14 @@
 #include "build_cfg.h"
 
 #include "lss/nc_cluster_abundance.h"
+#include "lss/nc_cluster_mass_benson.h"
 #include "math/ncm_spline_cubic_notaknot.h"
 #include "math/ncm_spline2d_bicubic.h"
 #include "math/integral.h"
 #include "math/memory_pool.h"
 #include "math/ncm_cfg.h"
-#include <gsl/gsl_histogram.h>
 
-#include "lss/nc_cluster_mass_benson.h"
+#include <gsl/gsl_histogram.h>
 
 enum
 {
@@ -60,8 +60,8 @@ G_DEFINE_TYPE (NcClusterAbundance, nc_cluster_abundance, G_TYPE_OBJECT);
 #define LNM_MIN (10.0 * M_LN10)
 #define _NC_CLUSTER_ABUNDANCE_DEFAULT_INT_KEY 6
 
-static gdouble _intp_d2N (NcClusterAbundance *cad, NcHICosmo *model, gdouble lnM, gdouble z) {g_error ("Function d2NdzdlnM_val not implemented or cad not prepared."); return 0.0;};
-static gdouble _N (NcClusterAbundance *cad, NcHICosmo *model) {g_error ("Function N_val not implemented or cad not prepared."); return 0.0;};
+static gdouble _intp_d2N (NcClusterAbundance *cad, NcHICosmo *model, gdouble lnM, gdouble z) { NCM_UNUSED (cad); NCM_UNUSED (model); NCM_UNUSED (lnM); NCM_UNUSED (z); g_error ("Function d2NdzdlnM_val not implemented or cad not prepared."); return 0.0;};
+static gdouble _N (NcClusterAbundance *cad, NcHICosmo *model) { NCM_UNUSED (cad); NCM_UNUSED (model); g_error ("Function N_val not implemented or cad not prepared."); return 0.0;};
 
 /**
  * nc_cluster_abundance_new:
@@ -709,19 +709,25 @@ _nc_cad_inv_dNdz_convergence_f_onemn (gdouble onemn, gdouble epsilon)
 void
 nc_cluster_abundance_prepare_inv_dNdz (NcClusterAbundance *cad, NcHICosmo *model)
 {
-  gint i, j;
+  guint i, j;
   gdouble z0 = cad->zi;
-  gint middle = cad->inv_z->len / 2;
+  guint middle = cad->inv_z->len / 2;
   gboolean use_spline = FALSE;
   NcMassFunctionSplineOptimize sp_optimize = NC_MASS_FUNCTION_SPLINE_Z;
   g_assert (cad->zi != 0);
 
-  cad->z_epsilon = nc_mass_function_n (cad->mfp, model, cad->lnMi, cad->lnMf, cad->zi + (cad->zf - cad->zi) / (cad->inv_z->len - 1.0) *
-                                       (cad->inv_z->len - 2.0), cad->zf, sp_optimize) / cad->norma;
+  {
+    const gdouble delta_z = (cad->zf - cad->zi) / (cad->inv_z->len - 1.0);
+    const gdouble zfm1 = cad->zf - delta_z;
+    cad->z_epsilon = nc_mass_function_n (cad->mfp, model, cad->lnMi, cad->lnMf, zfm1, cad->zf, sp_optimize) / cad->norma;  
+  }
 
-  cad->lnM_epsilon = nc_mass_function_dn_dz (cad->mfp, model, cad->lnMi + (cad->lnMf - cad->lnMi) / (cad->inv_lnM->len - 1.0) *
-                                             (cad->inv_lnM->len - 2.0), cad->lnMf, cad->zf, use_spline) /
-    nc_mass_function_dn_dz (cad->mfp, model, cad->lnMi, cad->lnMf, cad->zf, use_spline);
+  {
+    const gdouble delta_lnM = (cad->lnMf - cad->lnMi) / (cad->inv_lnM->len - 1.0); 
+    const gdouble lnMfm1 = cad->lnMf - delta_lnM;
+    cad->lnM_epsilon = nc_mass_function_dn_dz (cad->mfp, model, lnMfm1, cad->lnMf, cad->zf, use_spline) /
+      nc_mass_function_dn_dz (cad->mfp, model, cad->lnMi, cad->lnMf, cad->zf, use_spline);
+  }
 
   //printf ("Aqui!!!\n");
   //printf ("z epsilon % 20.15g lnM epsilon % 20.15g\n", cad->z_epsilon, cad->lnM_epsilon);
@@ -731,7 +737,7 @@ nc_cluster_abundance_prepare_inv_dNdz (NcClusterAbundance *cad, NcHICosmo *model
 
     ncm_vector_set (cad->inv_lnM_z->xv, 0, _nc_cad_inv_dNdz_convergence_f (0.0, cad->lnM_epsilon)); // 0.0;
     ncm_matrix_set (cad->inv_lnM_z->zm, middle, 0, cad->lnMi);
-
+    
     //printf ("%zu % 20.15g  %u % 20.15g\n", cad->inv_z->len, zm, ncm_vector_len (cad->inv_lnM_z->xv), cad->lnMi);
     //printf ("# eval[%d] u2 lnM % 20.15g % 20.15g\n", 0, ncm_vector_get (cad->inv_lnM->xv, 0), ncm_spline_eval (cad->inv_lnM, ncm_vector_get (cad->inv_lnM->xv, 0)));
     for (j = 1; j < ncm_vector_len (cad->inv_lnM_z->xv) - 1; j++)
@@ -830,12 +836,14 @@ nc_cluster_abundance_prepare_inv_dNdlnM_z (NcClusterAbundance *cad, NcHICosmo *m
   gdouble ntot = 0.0;
   gdouble f = _nc_cad_inv_dNdz_convergence_f (0.0, cad->lnM_epsilon);
   gdouble Delta;
-  gint i;
+  guint i;
   g_assert (z > 0.0);
 
   ncm_vector_set (cad->inv_lnM->xv, 0, f); //dNdz_u2 / dNdz;
   ncm_vector_set (cad->inv_lnM->yv, 0, lnM0);
 
+  //printf ("prep[%d] % 20.15g % 20.15g % 20.15g % 20.15g\n", 0, ntot, f, lnM0, z);
+  
   for (i = 1; i < cad->inv_lnM->len; i++)
   {
     gdouble lnM1 = cad->lnMi + (cad->lnMf - cad->lnMi) / (cad->inv_lnM->len - 1.0) * i;
@@ -847,7 +855,7 @@ nc_cluster_abundance_prepare_inv_dNdlnM_z (NcClusterAbundance *cad, NcHICosmo *m
       f = _nc_cad_inv_dNdz_convergence_f (ntot, cad->lnM_epsilon);
     ncm_vector_set (cad->inv_lnM->xv, i, f); //dNdz_u2 / dNdz;
     ncm_vector_set (cad->inv_lnM->yv, i, lnM1);
-    //printf ("prep[%d] % 20.15g % 20.15g % 20.15g % 20.15g\n", i, ntot, f, lnM1, z);
+    //printf ("prep[%d] % 20.15g % 20.15g % 20.15g % 20.15g\n", i, 1.0 - ntot, f, lnM1, z);
     lnM0 = lnM1;
   }
 
@@ -855,10 +863,10 @@ nc_cluster_abundance_prepare_inv_dNdlnM_z (NcClusterAbundance *cad, NcHICosmo *m
   {
     gdouble lnM1 = cad->lnMi + (cad->lnMf - cad->lnMi) / (cad->inv_lnM->len - 1.0) * i;
     gdouble onemn = nc_mass_function_dn_dz (cad->mfp, model, lnM1, cad->lnMf, z, use_spline) / dNdz;
-    f = _nc_cad_inv_dNdz_convergence_f_onemn (onemn, cad->lnM_epsilon);
+    f = _nc_cad_inv_dNdz_convergence_f_onemn (fabs (onemn), cad->lnM_epsilon);
     ncm_vector_set (cad->inv_lnM->xv, i, f); //dNdz_u2 / dNdz;
     ncm_vector_set (cad->inv_lnM->yv, i, lnM1);
-    //printf ("prep[%d] % 20.15g % 20.15g % 20.15g % 20.15g\n", i, onemn, f, lnM1, z);
+    //printf ("prep[%d] % 20.15g % 20.15g % 20.15g % 20.15g % 20.15g\n", i, onemn, f, lnM1, cad->lnMf, z);
   }
 
   ncm_spline_prepare (cad->inv_lnM);
@@ -1073,7 +1081,7 @@ nc_cluster_abundance_class_init (NcClusterAbundanceClass *klass)
 void
 nc_cluster_abundance_bin_realization (GArray *zr, gsl_histogram **h)
 {
-  int i;
+  guint i;
   for (i = 0; i < zr->len; i++)
   {
     int j;
@@ -1087,7 +1095,7 @@ void
 nc_cluster_abundance_realizations_save_to_file (GPtrArray *realizations, gchar *filename)
 {
   FILE *out;
-  gint i, j;
+  guint i, j;
   out = fopen (filename,"w");
 
   for (j = 0; j < realizations->len; j++)
@@ -1112,14 +1120,14 @@ nc_cluster_abundance_realizations_save_to_file (GPtrArray *realizations, gchar *
  * GPtrArray *realizations is an array of array with the z values of all realizations. To complete...
  *
  * Returns: (transfer full): FIXME
-   */
+ */
 GPtrArray *
-nc_cluster_abundance_realizations_read_from_file (gchar *file_realization, gint n_realizations)
+nc_cluster_abundance_realizations_read_from_file (gchar *file_realization, guint n_realizations)
 {
   FILE *frealization = fopen (file_realization, "r");
   GPtrArray *realizations = g_ptr_array_sized_new (n_realizations);
-  int i, j;
-  long int file_position, goby;
+  guint i, j;
+  glong file_position, goby;
 
   if (frealization == NULL)
     g_error ("abundance_random_generator_read_from_file: file %s, do not exist.", file_realization);
