@@ -60,12 +60,6 @@ typedef gdouble (*NcClusterAbundanceIntPd2N) (NcClusterAbundance *cad, NcHICosmo
 #define nc_cluster_abundance_dNdlnM_val(cad,cp,lnM,zl,zu) (cad)->dNdlnM_val(cad,cp,lnM,zl,zu)
 #define nc_cluster_abundance_N_val(cad,cp,lnMl,lnMu,zl,zu) (cad)->N_val(cad,cp,lnMl,lnMu,zl,zu)
 
-struct _NcClusterAbundanceClass
-{
-  /*< private >*/
-  GObjectClass parent_class;
-};
-
 struct _NcClusterAbundance
 {
   /*< private >*/
@@ -92,6 +86,12 @@ struct _NcClusterAbundance
   NcmModelCtrl *ctrl;
 };
 
+struct _NcClusterAbundanceClass
+{
+  /*< private >*/
+  GObjectClass parent_class;
+};
+
 GType nc_cluster_abundance_get_type (void) G_GNUC_CONST;
 
 NcClusterAbundance *nc_cluster_abundance_new (NcMassFunction *mfp, NcHaloBiasFunc *mbiasf, NcClusterRedshift *clusterz, NcClusterMass *clusterm);
@@ -124,7 +124,7 @@ G_INLINE_FUNC NcClusterMass *nc_cluster_abundance_peek_mass (NcClusterAbundance
 /*
 void nc_cluster_abundance_bin_realization (GArray *zr, gsl_histogram **h);
 void nc_cluster_abundance_realizations_save_to_file (GPtrArray *realizations, gchar *filename);
-GPtrArray *nc_cluster_abundance_realizations_read_from_file (gchar *file_realization, gint n_realizations);
+GPtrArray *nc_cluster_abundance_realizations_read_from_file (gchar *file_realization, guint n_realizations);
 gdouble nc_cluster_abundance_d2NdzdlnM_photoz (NcClusterAbundance *cad, NcHICosmo *model, gdouble lnM, gdouble *z_obs, gdouble *z_obs_params);
 gdouble nc_cluster_abundance_d2NdzdlnM_Mobs (NcClusterAbundance *cad, NcHICosmo *model, gdouble *lnM_obs, gdouble *lnM_obs_params, gdouble z);
 gdouble nc_cluster_abundance_d2NdzdlnM_photoz_Mobs (NcClusterAbundance *cad, NcHICosmo *model, gdouble *lnM_obs, gdouble *lnM_obs_params, gdouble *z_obs, gdouble *z_obs_params);
