@@ -8,12 +8,12 @@
 /*
  * numcosmo
  * Copyright (C) Mariana Penna Lima 2012 <pennalima@gmail.com>
-   * 
+ * 
  * numcosmo is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
-   * 
+ * 
  * numcosmo is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
@@ -235,36 +235,45 @@ _nc_cluster_mass_benson_intp (NcClusterMass *clusterm, NcHICosmo *model, gdouble
 }
 
 static gboolean
-_nc_cluster_mass_benson_resample (NcClusterMass *clusterm, NcHICosmo *model, gdouble lnM, gdouble z, gdouble *xi, gdouble *xi_params)
+_nc_cluster_mass_benson_resample (NcClusterMass *clusterm, NcHICosmo *model, gdouble lnM, gdouble z, gdouble *xi, gdouble *xi_params, NcmRNG *rng)
 {
   NcClusterMassBenson *msz = NC_CLUSTER_MASS_BENSON (clusterm);
-  gsl_rng *rng = ncm_cfg_rng_get ();
   gdouble lnzeta, lnzeta_obs, zeta_obs, xi_mean;
   const gdouble E0 = nc_hicosmo_E (model, msz->z0);
   const gdouble E = nc_hicosmo_E (model, z);
 
+  NCM_UNUSED (xi_params);
+  
   lnzeta = B_SZ * (lnM - log (msz->M0)) + C_SZ * log (E / E0) + log (A_SZ);
  
   {
-    lnzeta_obs = lnzeta + gsl_ran_gaussian (rng, D_SZ);
+    gboolean ret;
+    ncm_rng_lock (rng);
+    lnzeta_obs = lnzeta + gsl_ran_gaussian (rng->r, D_SZ);
     zeta_obs = exp (lnzeta_obs);
     if (zeta_obs > 1.0 && zeta_obs < 2.0)
-      return FALSE;
+      ret = FALSE;
     else
     {
       xi_mean = _nc_cluster_mass_benson_xi_mean (zeta_obs);
-      xi[0] = xi_mean + gsl_ran_gaussian (rng, 1.0);
+      xi[0] = xi_mean + gsl_ran_gaussian (rng->r, 1.0);
 
       //printf("M = %e z = %.5g zeta = %.5g xi = %.5g xiobs = %.5g | xiobs_min = %.5g\n", exp(lnM), z, zeta_obs, xi_mean, xi[0], msz->signif_obs_min);
 
-      return (xi[0] >= msz->signif_obs_min);
+      ret = (xi[0] >= msz->signif_obs_min);
     }
+    ncm_rng_unlock (rng);
+    return ret;
   }
 }
 
 static gdouble
 _significance_to_zeta (NcClusterMass *clusterm, NcHICosmo *model, gdouble z, gdouble xi)
 {
+  NCM_UNUSED (clusterm);
+  NCM_UNUSED (model);
+  NCM_UNUSED (z);
+  
   return sqrt (xi * xi - 3.0);
 }
 
@@ -293,6 +302,8 @@ _nc_cluster_mass_benson_p_limits (NcClusterMass *clusterm, NcHICosmo *model, gdo
   const gdouble zetau = _significance_to_zeta (clusterm, model, 0.0, xiu) + 7.0 * D_SZ;
   const gdouble lnMu = _zeta_to_mass (clusterm, model, 0.0, zetau);
 
+  NCM_UNUSED (xi_params);
+  
   *lnM_lower = lnMl;
   *lnM_upper = lnMu;
 
@@ -318,8 +329,8 @@ _nc_cluster_mass_benson_n_limits (NcClusterMass *clusterm, NcHICosmo *model, gdo
   return;  
 }
 
-guint _nc_cluster_mass_benson_obs_len (NcClusterMass *clusterm) { return 1; }
-guint _nc_cluster_mass_benson_obs_params_len (NcClusterMass *clusterm) { return 0; }
+guint _nc_cluster_mass_benson_obs_len (NcClusterMass *clusterm) { NCM_UNUSED (clusterm); return 1; }
+guint _nc_cluster_mass_benson_obs_params_len (NcClusterMass *clusterm) { NCM_UNUSED (clusterm); return 0; }
 
 static void
 _nc_cluster_mass_benson_set_property (GObject * object, guint prop_id, const GValue * value, GParamSpec * pspec)
@@ -408,8 +419,6 @@ nc_cluster_mass_benson_class_init (NcClusterMassBensonClass *klass)
   parent_class->impl = NC_CLUSTER_MASS_IMPL_ALL;
 
   object_class->finalize = _nc_cluster_mass_benson_finalize;
-  object_class->set_property = &ncm_model_class_set_property;
-  object_class->get_property = &ncm_model_class_get_property;
 
   model_class->set_property = &_nc_cluster_mass_benson_set_property;
   model_class->get_property = &_nc_cluster_mass_benson_get_property;
@@ -499,7 +508,7 @@ nc_cluster_mass_benson_class_init (NcClusterMassBensonClass *klass)
    * FIXME Set correct values (limits)
    */
   ncm_model_class_set_sparam (model_class, NC_CLUSTER_MASS_BENSON_D_SZ, "D_{SZ}", "Dsz",
-                              1e-8,  10.0, 1.0e-2,
+                              1e-2,  1.0, 1.0e-2,
                               NC_CLUSTER_MASS_BENSON_DEFAULT_PARAMS_ABSTOL, NC_CLUSTER_MASS_BENSON_DEFAULT_D_SZ,
                               NCM_PARAM_TYPE_FIXED);
 
