@@ -43,8 +43,9 @@
 
 G_DEFINE_TYPE (NcClusterMassLnnormal, nc_cluster_mass_lnnormal, NC_TYPE_CLUSTER_MASS);
 
-#define _NC_CLUSTER_MASS_LNNORMAL_BIAS 0.0
-#define _NC_CLUSTER_MASS_LNNORMAL_SIGMA 0.04
+#define VECTOR (NCM_MODEL (mlnn)->params)
+#define BIAS   (ncm_vector_get (VECTOR, NC_CLUSTER_MASS_LNNORMAL_BIAS))
+#define SIGMA  (ncm_vector_get (VECTOR, NC_CLUSTER_MASS_LNNORMAL_SIGMA))
 
 enum
 {
@@ -57,53 +58,58 @@ enum
 static gdouble
 _nc_cluster_mass_lnnormal_p (NcClusterMass *clusterm, NcHICosmo *model, gdouble lnM, gdouble z, gdouble *lnM_obs, gdouble *lnM_obs_params)
 {
+  NcClusterMassLnnormal *mlnn = NC_CLUSTER_MASS_LNNORMAL (clusterm);
   const gdouble lnMobs = lnM_obs[0];
-  const gdouble lnM_bias = lnM_obs_params[NC_CLUSTER_MASS_LNNORMAL_BIAS];
-  const gdouble sigma = lnM_obs_params[NC_CLUSTER_MASS_LNNORMAL_SIGMA];
-  const gdouble sqrt2_sigma = M_SQRT2 * sigma;
-  const gdouble x = (lnMobs - lnM - lnM_bias) / sqrt2_sigma;
+  const gdouble sqrt2_sigma = M_SQRT2 * SIGMA;
+  const gdouble x = (lnMobs - lnM - BIAS) / sqrt2_sigma;
 
-  return M_2_SQRTPI / (2.0 * M_SQRT2) * exp (- x * x) / (sigma);
+  NCM_UNUSED (model);
+  NCM_UNUSED (z);
+
+  return M_2_SQRTPI / (2.0 * M_SQRT2) * exp (- x * x) / (SIGMA);
 }
 
 static gdouble
 _nc_cluster_mass_lnnormal_intp (NcClusterMass *clusterm, NcHICosmo *model, gdouble lnM, gdouble z)
 {
   NcClusterMassLnnormal *mlnn = NC_CLUSTER_MASS_LNNORMAL (clusterm);
-  const gdouble sigma = _NC_CLUSTER_MASS_LNNORMAL_SIGMA;
-  const gdouble sqrt2_sigma = M_SQRT2 * sigma;
+  const gdouble sqrt2_sigma = M_SQRT2 * SIGMA;
   const gdouble x_min = (lnM - mlnn->lnMobs_min) / sqrt2_sigma;
   const gdouble x_max = (lnM - mlnn->lnMobs_max) / sqrt2_sigma;
 
+  NCM_UNUSED (model);
+  NCM_UNUSED (z);
+  
   if (x_max > 4.0)
-	return -(erfc (x_min) - erfc (x_max)) / 2.0;
+    return -(erfc (x_min) - erfc (x_max)) / 2.0;
   else
-	return (erf (x_min) - erf (x_max)) / 2.0;
+    return (erf (x_min) - erf (x_max)) / 2.0;
 }
 
 static gboolean
-_nc_cluster_mass_lnnormal_resample (NcClusterMass *clusterm, NcHICosmo *model, gdouble lnM, gdouble z, gdouble *lnM_obs, gdouble *lnM_obs_params)
+_nc_cluster_mass_lnnormal_resample (NcClusterMass *clusterm, NcHICosmo *model, gdouble lnM, gdouble z, gdouble *lnM_obs, gdouble *lnM_obs_params, NcmRNG *rng)
 {
   NcClusterMassLnnormal *mlnn = NC_CLUSTER_MASS_LNNORMAL (clusterm);
-  gsl_rng *rng = ncm_cfg_rng_get ();
-  const gdouble sigma = _NC_CLUSTER_MASS_LNNORMAL_SIGMA;
-  const gdouble bias = _NC_CLUSTER_MASS_LNNORMAL_BIAS;
-
-  lnM_obs_params[NC_CLUSTER_MASS_LNNORMAL_BIAS] = bias;
-  lnM_obs_params[NC_CLUSTER_MASS_LNNORMAL_SIGMA] = sigma;
-
-  lnM_obs[0] = lnM + bias + gsl_ran_gaussian (rng, sigma);
 
+  NCM_UNUSED (model);
+  NCM_UNUSED (z);
+  
+  ncm_rng_lock (rng);
+  lnM_obs[0] = lnM + BIAS + gsl_ran_gaussian (rng->r, SIGMA);
+  ncm_rng_unlock (rng);
   return (lnM_obs[0] <= mlnn->lnMobs_max) && (lnM_obs[0] >= mlnn->lnMobs_min);
 }
 
 static void
 _nc_cluster_mass_lnnormal_p_limits (NcClusterMass *clusterm, NcHICosmo *model, gdouble *lnM_obs, gdouble *lnM_obs_params, gdouble *lnM_lower, gdouble *lnM_upper)
 {
-  const gdouble mean = lnM_obs[0] - lnM_obs_params[NC_CLUSTER_MASS_LNNORMAL_BIAS];
-  const gdouble lnMl = mean - 7.0 * lnM_obs_params[NC_CLUSTER_MASS_LNNORMAL_SIGMA];
-  const gdouble lnMu = mean + 7.0 * lnM_obs_params[NC_CLUSTER_MASS_LNNORMAL_SIGMA];
+  NcClusterMassLnnormal *mlnn = NC_CLUSTER_MASS_LNNORMAL (clusterm);
+  const gdouble mean = lnM_obs[0] - BIAS;
+  const gdouble lnMl = mean - 7.0 * SIGMA;
+  const gdouble lnMu = mean + 7.0 * SIGMA;
 
+  NCM_UNUSED (model);
+  
   *lnM_lower = lnMl;
   *lnM_upper = lnMu;
 
@@ -114,17 +120,19 @@ static void
 _nc_cluster_mass_lnnormal_n_limits (NcClusterMass *clusterm, NcHICosmo *model, gdouble *lnM_lower, gdouble *lnM_upper)
 {
   NcClusterMassLnnormal *mlnn = NC_CLUSTER_MASS_LNNORMAL (clusterm);
-  const gdouble lnMl = mlnn->lnMobs_min - 7.0 * _NC_CLUSTER_MASS_LNNORMAL_SIGMA;
-  const gdouble lnMu = mlnn->lnMobs_max + 7.0 * _NC_CLUSTER_MASS_LNNORMAL_SIGMA;
+  const gdouble lnMl = mlnn->lnMobs_min - 7.0 * SIGMA;
+  const gdouble lnMu = mlnn->lnMobs_max + 7.0 * SIGMA;
 
+  NCM_UNUSED (model);
+  
   *lnM_lower = lnMl;
   *lnM_upper = lnMu;
 
   return;
 }
 
-guint _nc_cluster_mass_lnnormal_obs_len (NcClusterMass *clusterm) { return 1; }
-guint _nc_cluster_mass_lnnormal_obs_params_len (NcClusterMass *clusterm) { return 2; }
+guint _nc_cluster_mass_lnnormal_obs_len (NcClusterMass *clusterm) { NCM_UNUSED (clusterm); return 1; }
+guint _nc_cluster_mass_lnnormal_obs_params_len (NcClusterMass *clusterm) { NCM_UNUSED (clusterm); return 2; }
 
 static void
 _nc_cluster_mass_lnnormal_set_property (GObject * object, guint prop_id, const GValue * value, GParamSpec * pspec)
@@ -136,9 +144,11 @@ _nc_cluster_mass_lnnormal_set_property (GObject * object, guint prop_id, const G
   {
     case PROP_LNMOBS_MIN:
       mlnm->lnMobs_min = g_value_get_double (value);
+      g_assert (mlnm->lnMobs_min < mlnm->lnMobs_max);
       break;
-	case PROP_LNMOBS_MAX:
+    case PROP_LNMOBS_MAX:
       mlnm->lnMobs_max = g_value_get_double (value);
+      g_assert (mlnm->lnMobs_min < mlnm->lnMobs_max);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -157,7 +167,7 @@ _nc_cluster_mass_lnnormal_get_property (GObject *object, guint prop_id, GValue *
     case PROP_LNMOBS_MIN:
       g_value_set_double (value, mlnm->lnMobs_min);
       break;
-	case PROP_LNMOBS_MAX:
+    case PROP_LNMOBS_MAX:
       g_value_set_double (value, mlnm->lnMobs_max);
       break;
     default:
@@ -169,8 +179,8 @@ _nc_cluster_mass_lnnormal_get_property (GObject *object, guint prop_id, GValue *
 static void
 nc_cluster_mass_lnnormal_init (NcClusterMassLnnormal *mlnm)
 {
-  mlnm->lnMobs_min = 0.0;
-  mlnm->lnMobs_max = 0.0;
+  mlnm->lnMobs_min = GSL_NEGINF;
+  mlnm->lnMobs_max = GSL_POSINF;
 }
 
 static void
@@ -199,13 +209,11 @@ nc_cluster_mass_lnnormal_class_init (NcClusterMassLnnormalClass *klass)
   parent_class->impl = NC_CLUSTER_MASS_IMPL_ALL;
 
   object_class->finalize     = &_nc_cluster_mass_lnnormal_finalize;
-  object_class->set_property = &ncm_model_class_set_property;
-  object_class->get_property = &ncm_model_class_get_property;
 
   model_class->set_property = &_nc_cluster_mass_lnnormal_set_property;
   model_class->get_property = &_nc_cluster_mass_lnnormal_get_property;
 
-  ncm_model_class_add_params (model_class, 0, 0, PROP_SIZE);
+  ncm_model_class_add_params (model_class, 2, 0, PROP_SIZE);
   ncm_model_class_set_name_nick (model_class, "Ln-normal distribution", "Ln_Normal");
 
   /**
@@ -234,5 +242,24 @@ nc_cluster_mass_lnnormal_class_init (NcClusterMassLnnormalClass *klass)
                                                         11.0 * M_LN10, G_MAXDOUBLE, 16.0 * M_LN10,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
 
-}
+  /*
+   * Distribution's bias: bias.
+   * FIXME Set correct values (limits)
+   */
+  ncm_model_class_set_sparam (model_class, NC_CLUSTER_MASS_LNNORMAL_BIAS, "bias", "bias",
+                              0.0,  10.0, 1.0e-2,
+                              NC_CLUSTER_MASS_LNNORMAL_DEFAULT_PARAMS_ABSTOL, NC_CLUSTER_MASS_LNNORMAL_DEFAULT_BIAS,
+                              NCM_PARAM_TYPE_FIXED);
 
+  /*
+   * Standard deviation: sigma.
+   * FIXME Set correct values (limits)
+   */
+  ncm_model_class_set_sparam (model_class, NC_CLUSTER_MASS_LNNORMAL_SIGMA, "sigma", "sigma",
+                              1.0e-4,  10.0, 1.0e-2,
+                              NC_CLUSTER_MASS_LNNORMAL_DEFAULT_PARAMS_ABSTOL, NC_CLUSTER_MASS_LNNORMAL_DEFAULT_SIGMA,
+                              NCM_PARAM_TYPE_FIXED);
+
+  /* Check for errors in parameters initialization */
+  ncm_model_class_check_params_info (model_class);
+}
