@@ -600,7 +600,7 @@ nc_mass_function_set_eval_limits (NcMassFunction *mfp, NcHICosmo *model, gdouble
 void
 nc_mass_function_prepare (NcMassFunction *mfp, NcHICosmo *model)
 {
-  gint i, j;
+  guint i, j;
   if (mfp->d2NdzdlnM == NULL)
   {
     g_error ("nc_mass_function_prepare: called without a previous call of nc_mass_function_set_eval_limits");
@@ -620,6 +620,7 @@ nc_mass_function_prepare (NcMassFunction *mfp, NcHICosmo *model)
       const gdouble lnM = ncm_vector_get (D2NDZDLNM_LNM (mfp), j);
       const gdouble d2NdzdlnM_ij = (dVdz * nc_mass_function_dn_dlnm (mfp, model, lnM, z));
       ncm_matrix_set (D2NDZDLNM_VAL (mfp), i, j, d2NdzdlnM_ij);
+      //printf ("prep matrix % 20.15g % 20.15g % 20.15g\n", z, lnM, d2NdzdlnM_ij);
     }
   }
   ncm_spline2d_prepare (mfp->d2NdzdlnM);
@@ -651,6 +652,7 @@ nc_mass_function_dn_dz (NcMassFunction *mfp, NcHICosmo *model, gdouble lnMl, gdo
     dN_dz = ncm_spline2d_integ_dx_spline_val (mfp->d2NdzdlnM, lnMl, lnMu, z);
   else
     dN_dz = ncm_spline2d_integ_dx (mfp->d2NdzdlnM, lnMl, lnMu, z);
+
   return dN_dz;
 }
 
