@@ -564,7 +564,7 @@ static void _top_hat_cubic_spline_integration_rule_taylor (gdouble xa, gdouble x
  * This function returns the variance of the density contrast at redshift \f$ z = 0 \f$ computed at scale R FIXME
  *
  * Returns: a gdouble which is the variance \f$ \sigma^2 (R, z = 0) \f$.
-   */
+ */
 gdouble
 nc_matter_var_var0 (NcMatterVar *vp, NcHICosmo *model, gdouble lnR)
 {
@@ -812,7 +812,8 @@ _nc_matter_var_prepare_splineint (NcMatterVar *vp, NcHICosmo *model)
       ncm_vector_set (vp->integrand_overw2_spline->xv, i, k);
       ncm_vector_set (vp->integrand_overw2_spline->yv, i, nc_matter_var_integrand_over_window2 (vp, model, k));
     }
-    last_i = i;
+
+    /* last_i = i; */
     ncm_spline_prepare (vp->integrand_overw2_spline);
   }
   else if (FALSE)
@@ -905,6 +906,7 @@ _nc_matter_var_prepare_fft (NcMatterVar *vp, NcHICosmo *model)
     p_Rsigma2 = fftw_plan_dft_1d (N, out, in, FFTW_FORWARD, FFTW_PATIENT | FFTW_DESTROY_INPUT);
     p_Rdsigma2_dr = fftw_plan_dft_1d (N, fftRdsigma2_dr, Rdsigma2_dr, FFTW_FORWARD, FFTW_PATIENT | FFTW_DESTROY_INPUT);
     ncm_cfg_save_fftw_wisdom ("nc_matter_var_wisdown.fftw3");
+    
     planned = TRUE;
   }
 
@@ -929,7 +931,7 @@ _nc_matter_var_prepare_fft (NcMatterVar *vp, NcHICosmo *model)
       const gdouble a = 2.0 * M_PI / L * i;
       const gdouble abs_a = fabs (a);
       const gdouble sign_a = a < 0 ? -1.0 : 1.0;
-      complex U = 36.0 * (a + I) / (cexp (M_LN2 * I * a) * (a * I - 5.0));
+      complex double U = 36.0 * (a + I) / (cexp (M_LN2 * I * a) * (a * I - 5.0));
       gsl_sf_result lnr;
       gsl_sf_result arg;
       gsl_sf_lngamma_complex_e (-3.0, a, &lnr, &arg);
@@ -958,9 +960,9 @@ _nc_matter_var_prepare_fft (NcMatterVar *vp, NcHICosmo *model)
   {
     const gint ii = (i < 0) ? i + N : i;
     const gdouble r = r0 + i * dr;
-    const complex Rsigma2 = in[ii] / N;
-    const complex lnsigma2 = log (Rsigma2) - r;
-    const complex dlnsigma2_dr = Rdsigma2_dr[ii] / (Rsigma2 * N);
+    const complex double Rsigma2 = in[ii] / N;
+    const complex double lnsigma2 = log (Rsigma2) - r;
+    const complex double dlnsigma2_dr = Rdsigma2_dr[ii] / (Rsigma2 * N);
     /*
      const gdouble R = exp (r);
      const gdouble Rsigma2_old = R * nc_matter_var_over_growth2_tophat_old (vp, model, r);
@@ -1196,7 +1198,7 @@ _top_hat_cubic_spline_integration_rule_taylor (gdouble xa, gdouble xb, gdouble R
   rules[3] = delta4 * (dnj12_x2[0] / 4.0 + dnj12_x2[1] * Rdelta / 5.0 + dnj12_x2[2] * Rdelta2 / 12.0 + dnj12_x2[3] * Rdelta3 / 42.0 + dnj12_x2[4] * Rdelta4 / 192.0);
 }
 
-/**
+/*
  * nc_matter_var_gaussian_cubic_spline_integration_rule:
  * @x: x = k * R FIXME
  * @rules: FIXME
@@ -1220,7 +1222,7 @@ _top_hat_cubic_spline_integration_rule_taylor (gdouble xa, gdouble xb, gdouble R
    }
    */
 
-/**
+/*
  * nc_matter_var_derivative_gaussian_cubic_spline_integration_rule:
  * @x: x = k * R FIXME
  * @rules: FIXME
@@ -1260,10 +1262,11 @@ typedef struct
  * @rules_b: are the rules computed at the point k*R = b.
  * @R: is the radius of the cluster which is related to its mass.
  * @a: is the lower point (k*R) of the spline.
-   * @rule: is the vector with the four components whose sum gives the integration of the
+ * @rule: is the vector with the four components whose sum gives the integration of the
  * spline with nodes a (lower) and b (upper).
-   *
+ *
  * FIXME
+ * 
  */
 static void
 nc_matter_var_shift_scale_rule (gdouble *rules_a, gdouble *rules_b, gdouble R, gdouble a, gdouble *rule)
@@ -1357,10 +1360,12 @@ nc_matter_var_integrate_spline (NcMatterVar *vp, NcmSpline *s, void (*calc_rule)
   gdouble *rb = rules_b;
   gdouble *temp;
   gboolean taylor = FALSE;
-  int i;
+  guint i;
   gdouble res = 0.0;
   gdouble partial;
 
+  NCM_UNUSED (vp);
+
   calc_rule(ncm_vector_get (s->xv, 0) * R, ra);
   for(i = 0; i < s->len - 1; i++)
   {
