@@ -101,71 +101,73 @@ _nc_multiplicity_func_tinker_mean_eval (NcMultiplicityFunc *mulf, NcHICosmo *mod
   const gdouble log10_3200 = log10 (3200.0);
 
   const gdouble coef_A[8][6] = {
-	{log10_200, log10_300, 0.186, 0.0, 0.2, 0.5},
-	{log10_300, log10_400, 0.2, 0.5, 0.212, -1.56},
-	{log10_400, log10_600, 0.212, -1.56, 0.218, 3.05},
-	{log10_600, log10_800, 0.218, 3.05, 0.248, -2.95},
-	{log10_800, log10_1200, 0.248, -2.95, 0.255, 1.07},
-	{log10_1200, log10_1600, 0.255, 1.07, 0.26, -0.71},
-	{log10_1600, log10_2400, 0.26, -0.71, 0.26, 0.21},
-	{log10_2400, log10_3200, 0.26, 0.21, 0.26, 0.0}};
+    {log10_200, log10_300, 0.186, 0.0, 0.2, 0.5},
+    {log10_300, log10_400, 0.2, 0.5, 0.212, -1.56},
+    {log10_400, log10_600, 0.212, -1.56, 0.218, 3.05},
+    {log10_600, log10_800, 0.218, 3.05, 0.248, -2.95},
+    {log10_800, log10_1200, 0.248, -2.95, 0.255, 1.07},
+    {log10_1200, log10_1600, 0.255, 1.07, 0.26, -0.71},
+    {log10_1600, log10_2400, 0.26, -0.71, 0.26, 0.21},
+    {log10_2400, log10_3200, 0.26, 0.21, 0.26, 0.0}};
   const gdouble coef_a[8][6] = {
-	{log10_200, log10_300, 1.47, 0.0, 1.52, 1.19},
-	{log10_300, log10_400, 1.52, 1.19, 1.46, -6.34},
-	{log10_400, log10_600, 1.46, -6.34, 1.61, 21.36},
-	{log10_600, log10_800, 1.61, 21.36, 1.87, -10.95},
-	{log10_800, log10_1200, 1.87, -10.95, 2.13, 2.59},
-	{log10_1200, log10_1600, 2.13, 2.59, 2.3, -0.85},
-	{log10_1600, log10_2400, 2.3, -0.85, 2.53, -2.07},
-	{log10_2400, log10_3200, 2.53, -2.07, 2.66, 0.0}};
+    {log10_200, log10_300, 1.47, 0.0, 1.52, 1.19},
+    {log10_300, log10_400, 1.52, 1.19, 1.46, -6.34},
+    {log10_400, log10_600, 1.46, -6.34, 1.61, 21.36},
+    {log10_600, log10_800, 1.61, 21.36, 1.87, -10.95},
+    {log10_800, log10_1200, 1.87, -10.95, 2.13, 2.59},
+    {log10_1200, log10_1600, 2.13, 2.59, 2.3, -0.85},
+    {log10_1600, log10_2400, 2.3, -0.85, 2.53, -2.07},
+    {log10_2400, log10_3200, 2.53, -2.07, 2.66, 0.0}};
   const gdouble coef_b[8][6] = {
-	{log10_200, log10_300, 2.57, 0.0, 2.25, -1.08},
-	{log10_300, log10_400, 2.25, -1.08, 2.05, 12.61},
-	{log10_400, log10_600, 2.05, 12.61, 1.87, -20.96},
-	{log10_600, log10_800, 1.87, -20.96, 1.59, 24.08},
-	{log10_800, log10_1200, 1.59, 24.08, 1.51, -6.64},
-	{log10_1200, log10_1600, 1.51, -6.64, 1.46, 3.84},
-	{log10_1600, log10_2400, 1.46, 3.84, 1.44, -2.09},
-	{log10_2400, log10_3200, 1.44, -2.09, 1.41, 0.0}};
+    {log10_200, log10_300, 2.57, 0.0, 2.25, -1.08},
+    {log10_300, log10_400, 2.25, -1.08, 2.05, 12.61},
+    {log10_400, log10_600, 2.05, 12.61, 1.87, -20.96},
+    {log10_600, log10_800, 1.87, -20.96, 1.59, 24.08},
+    {log10_800, log10_1200, 1.59, 24.08, 1.51, -6.64},
+    {log10_1200, log10_1600, 1.51, -6.64, 1.46, 3.84},
+    {log10_1600, log10_2400, 1.46, 3.84, 1.44, -2.09},
+    {log10_2400, log10_3200, 1.44, -2.09, 1.41, 0.0}};
   const gdouble coef_c[8][6] = {
-	{log10_200, log10_300, 1.19, 0.0, 1.27, 0.94},
-	{log10_300, log10_400, 1.27, 0.94, 1.34, -0.43},
-	{log10_400, log10_600, 1.34, -0.43, 1.45, 4.61},
-	{log10_600, log10_800, 1.45, 4.61, 1.58, 0.01},
-	{log10_800, log10_1200, 1.58, 0.01, 1.8, 1.21},
-	{log10_1200, log10_1600, 1.8, 1.21, 1.97, 1.43},
-	{log10_1600, log10_2400, 1.97, 1.43, 2.24, 0.33},
-	{log10_2400, log10_3200, 2.24, 0.33, 2.44, 0.0}};
+    {log10_200, log10_300, 1.19, 0.0, 1.27, 0.94},
+    {log10_300, log10_400, 1.27, 0.94, 1.34, -0.43},
+    {log10_400, log10_600, 1.34, -0.43, 1.45, 4.61},
+    {log10_600, log10_800, 1.45, 4.61, 1.58, 0.01},
+    {log10_800, log10_1200, 1.58, 0.01, 1.8, 1.21},
+    {log10_1200, log10_1600, 1.8, 1.21, 1.97, 1.43},
+    {log10_1600, log10_2400, 1.97, 1.43, 2.24, 0.33},
+    {log10_2400, log10_3200, 2.24, 0.33, 2.44, 0.0}};
   gint i;
   gdouble A0 = 0.0, a0 = 0.0, b0 = 0.0, c = 0.0;
 
   g_assert (Delta <= 3200.0);
 
+  NCM_UNUSED (model);
+
   for (i = 0; i < 8; i++)
   {
-	if (log10_Delta >= coef_A[i][1])
-	  continue;
-	else 
-	{
-	  A0 = calc_polynomial (coef_A[i], log10_Delta);
-	  a0 = calc_polynomial (coef_a[i], log10_Delta);
-	  b0 = calc_polynomial (coef_b[i], log10_Delta);
-	  c = calc_polynomial (coef_c[i], log10_Delta);
-	  break;
-	}
+    if (log10_Delta >= coef_A[i][1])
+      continue;
+    else 
+    {
+      A0 = calc_polynomial (coef_A[i], log10_Delta);
+      a0 = calc_polynomial (coef_a[i], log10_Delta);
+      b0 = calc_polynomial (coef_b[i], log10_Delta);
+      c = calc_polynomial (coef_c[i], log10_Delta);
+      break;
+    }
   }
-  
+
   {
-	const gdouble A = A0 * pow(1.0 + z, -0.14);
-	const gdouble a = a0 * pow(1.0 + z, -0.06);
-	const gdouble log10alpha = -pow(0.75 / log10 (Delta / 75.0), 1.2);
-	const gdouble alpha = pow(10.0, log10alpha);
-	const gdouble b = b0 * pow(1.0 + z, -alpha);
-	const gdouble f_Tinker_mean = A * (pow(sigma / b, -a) + 1.0) * exp(-c / (sigma * sigma));
+    const gdouble A = A0 * pow(1.0 + z, -0.14);
+    const gdouble a = a0 * pow(1.0 + z, -0.06);
+    const gdouble log10alpha = -pow(0.75 / log10 (Delta / 75.0), 1.2);
+    const gdouble alpha = pow(10.0, log10alpha);
+    const gdouble b = b0 * pow(1.0 + z, -alpha);
+    const gdouble f_Tinker_mean = A * (pow(sigma / b, -a) + 1.0) * exp(-c / (sigma * sigma));
 
-	//printf ("NEW % 5.5g % 5.5g % 20.15g % 20.15g % 20.15g % 20.15g\n", z, mulf_tinker_mean->Delta, Delta_z, E2, Omega_m, gsl_pow_3 (1.0 + z));
+    //printf ("NEW % 5.5g % 5.5g % 20.15g % 20.15g % 20.15g % 20.15g\n", z, mulf_tinker_mean->Delta, Delta_z, E2, Omega_m, gsl_pow_3 (1.0 + z));
 
-	return f_Tinker_mean;
+    return f_Tinker_mean;
   }
 }
 
@@ -219,9 +221,9 @@ _nc_multiplicity_func_tinker_mean_set_property (GObject * object, guint prop_id,
 
   switch (prop_id)
   {
-	case PROP_DELTA:
-	  mulf_tinker_mean->Delta = g_value_get_double (value);
-	  break;
+    case PROP_DELTA:
+      mulf_tinker_mean->Delta = g_value_get_double (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -252,7 +254,7 @@ nc_multiplicity_func_tinker_mean_class_init (NcMultiplicityFuncTinkerMeanClass *
   NcMultiplicityFuncClass* parent_class = NC_MULTIPLICITY_FUNC_CLASS (klass);
 
   parent_class->eval = &_nc_multiplicity_func_tinker_mean_eval;
-  
+
   object_class->finalize = _nc_multiplicity_func_tinker_mean_finalize;
   object_class->set_property = _nc_multiplicity_func_tinker_mean_set_property;
   object_class->get_property = _nc_multiplicity_func_tinker_mean_get_property;
