@@ -36,6 +36,7 @@
 #include "build_cfg.h"
 
 #include "math/function_cache.h"
+#include "math/ncm_util.h"
 
 #include <gsl/gsl_math.h>
 
@@ -98,9 +99,7 @@ ncm_function_cache_free (NcmFunctionCache *cache)
 void
 ncm_function_cache_clear (NcmFunctionCache **cache)
 {
-  ncm_function_cache_free (*cache);
-  *cache = NULL;
-  return;
+  g_clear_pointer (cache, &ncm_function_cache_free);
 }
 
 /**
@@ -130,7 +129,7 @@ ncm_function_cache_insert (NcmFunctionCache *cache, gdouble x, ...)
 {
   gdouble *x_ptr;
   gsl_vector *v;
-  gint i;
+  guint i;
   va_list ap;
   _NCM_MUTEX_LOCK (&cache->lock);
   cache_clean (cache);
@@ -258,6 +257,7 @@ static gint
 gdouble_compare (gconstpointer a, gconstpointer b, gpointer user_data)
 {
 //  printf ("BLOB %g %g\n", *((gdouble *)a), *((gdouble *)b));
+  NCM_UNUSED (user_data);
   return gsl_fcmp ( *((gdouble *)a), *((gdouble *)b), 1e-15);
 }
 
