@@ -50,11 +50,8 @@ enum
   PROP_0,
   PROP_NPOINTS,
   PROP_USE_DET,
-/*  
-  PROP_USE_OFIT,
-  PROP_OFIT_MEAN,
-  PROP_OFIT_SIGMA,
- */ 
+  PROP_MEAN,
+  PROP_COV,
   PROP_SIZE,
 };
 
@@ -63,18 +60,13 @@ G_DEFINE_ABSTRACT_TYPE (NcmDataGaussCov, ncm_data_gauss_cov, NCM_TYPE_DATA);
 static void
 ncm_data_gauss_cov_init (NcmDataGaussCov *gauss)
 {
-  gauss->np           = 0;
-  gauss->y            = NULL;
-  gauss->v            = NULL;
-  gauss->cov          = NULL;
-  gauss->LLT          = NULL;
-  gauss->prepared_LLT = FALSE;
-  gauss->use_det      = FALSE;
-/*
-  gauss->use_ofit     = FALSE;
-  gauss->ofit_mean    = 0.0;
-  gauss->ofit_sigma   = 0.1;
- */ 
+  gauss->np               = 0;
+  gauss->y                = NULL;
+  gauss->v                = NULL;
+  gauss->cov              = NULL;
+  gauss->LLT              = NULL;
+  gauss->prepared_LLT     = FALSE;
+  gauss->use_det          = FALSE;
 }
 
 static void
@@ -87,7 +79,7 @@ _ncm_data_gauss_cov_constructed (GObject *object)
 }
 
 static void
-ncm_data_gauss_cov_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
+_ncm_data_gauss_cov_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
   NcmDataGaussCov *gauss = NCM_DATA_GAUSS_COV (object);
   g_return_if_fail (NCM_IS_DATA_GAUSS_COV (object));
@@ -100,17 +92,12 @@ ncm_data_gauss_cov_set_property (GObject *object, guint prop_id, const GValue *v
     case PROP_USE_DET:
       gauss->use_det = g_value_get_boolean (value);
       break;
-/*
-    case PROP_USE_OFIT:
-      gauss->use_ofit = g_value_get_boolean (value);
+    case PROP_MEAN:
+      ncm_vector_set_from_variant (gauss->y, g_value_get_variant (value));
       break;
-    case PROP_OFIT_MEAN:
-      gauss->ofit_mean = g_value_get_double (value);
+    case PROP_COV:
+      ncm_matrix_set_from_variant (gauss->cov, g_value_get_variant (value));
       break;
-    case PROP_OFIT_SIGMA:
-      gauss->ofit_sigma = g_value_get_double (value);
-      break;
- */
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -118,7 +105,7 @@ ncm_data_gauss_cov_set_property (GObject *object, guint prop_id, const GValue *v
 }
 
 static void
-ncm_data_gauss_cov_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
+_ncm_data_gauss_cov_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
   NcmDataGaussCov *gauss = NCM_DATA_GAUSS_COV (object);
   g_return_if_fail (NCM_IS_DATA_GAUSS_COV (object));
@@ -131,17 +118,12 @@ ncm_data_gauss_cov_get_property (GObject *object, guint prop_id, GValue *value,
     case PROP_USE_DET:
       g_value_set_boolean (value, gauss->use_det);
       break;
-/*
-    case PROP_USE_OFIT:
-      g_value_set_boolean (value, gauss->use_ofit);
+    case PROP_MEAN:
+      g_value_take_variant (value, ncm_vector_get_variant (gauss->y));
       break;
-    case PROP_OFIT_MEAN:
-      g_value_set_double (value, gauss->ofit_mean);
+    case PROP_COV:
+      g_value_take_variant (value, ncm_matrix_get_variant (gauss->cov));
       break;
-    case PROP_OFIT_SIGMA:
-      g_value_set_double (value, gauss->ofit_sigma);
-      break;
- */
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -149,7 +131,7 @@ ncm_data_gauss_cov_get_property (GObject *object, guint prop_id, GValue *value,
 }
 
 static void
-ncm_data_gauss_cov_dispose (GObject *object)
+_ncm_data_gauss_cov_dispose (GObject *object)
 {
   NcmDataGaussCov *gauss = NCM_DATA_GAUSS_COV (object);
 
@@ -163,7 +145,7 @@ ncm_data_gauss_cov_dispose (GObject *object)
 }
 
 static void
-ncm_data_gauss_cov_finalize (GObject *object)
+_ncm_data_gauss_cov_finalize (GObject *object)
 {
   
   /* Chain up : end */
@@ -171,11 +153,12 @@ ncm_data_gauss_cov_finalize (GObject *object)
 }
 
 static guint _ncm_data_gauss_cov_get_length (NcmData *data); 
-static void _ncm_data_gauss_cov_copyto (NcmData *data, NcmData *data_dest);
 /* static void _ncm_data_gauss_cov_begin (NcmData *data); */
-static void _ncm_data_gauss_cov_resample (NcmData *data, NcmMSet *mset);
+static void _ncm_data_gauss_cov_resample (NcmData *data, NcmMSet *mset, NcmRNG *rng);
 static void _ncm_data_gauss_cov_m2lnL_val (NcmData *data, NcmMSet *mset, gdouble *m2lnL);
 static void _ncm_data_gauss_cov_leastsquares_f (NcmData *data, NcmMSet *mset, NcmVector *v);
+static void _ncm_data_gauss_cov_set_size (NcmDataGaussCov *gauss, guint np);
+static guint _ncm_data_gauss_cov_get_size (NcmDataGaussCov *gauss);
 
 static void
 ncm_data_gauss_cov_class_init (NcmDataGaussCovClass *klass)
@@ -185,10 +168,10 @@ ncm_data_gauss_cov_class_init (NcmDataGaussCovClass *klass)
   NcmDataGaussCovClass *gauss_cov_class = NCM_DATA_GAUSS_COV_CLASS (klass);
 
   object_class->constructed  = &_ncm_data_gauss_cov_constructed;
-  object_class->set_property = &ncm_data_gauss_cov_set_property;
-  object_class->get_property = &ncm_data_gauss_cov_get_property;
-  object_class->dispose      = &ncm_data_gauss_cov_dispose;
-  object_class->finalize     = &ncm_data_gauss_cov_finalize;
+  object_class->set_property = &_ncm_data_gauss_cov_set_property;
+  object_class->get_property = &_ncm_data_gauss_cov_get_property;
+  object_class->dispose      = &_ncm_data_gauss_cov_dispose;
+  object_class->finalize     = &_ncm_data_gauss_cov_finalize;
 
   g_object_class_install_property (object_class,
                                    PROP_NPOINTS,
@@ -196,7 +179,7 @@ ncm_data_gauss_cov_class_init (NcmDataGaussCovClass *klass)
                                                       NULL,
                                                       "Data sample size",
                                                       0, G_MAXUINT, 0,
-                                                      G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
 
   g_object_class_install_property (object_class,
                                    PROP_USE_DET,
@@ -205,41 +188,36 @@ ncm_data_gauss_cov_class_init (NcmDataGaussCovClass *klass)
                                                          "Use determinant to calculate -2lnL",
                                                          FALSE,
                                                          G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
-/*  
+
   g_object_class_install_property (object_class,
-                                   PROP_USE_OFIT,
-                                   g_param_spec_boolean ("use-ofit",
+                                   PROP_MEAN,
+                                   g_param_spec_variant ("mean",
                                                          NULL,
-                                                         "Use overfitting penalty function",
-                                                         FALSE,
+                                                         "Data mean",
+                                                         G_VARIANT_TYPE ("ad"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  
+  g_object_class_install_property (object_class,
+                                   PROP_COV,
+                                   g_param_spec_variant ("cov",
+                                                         NULL,
+                                                         "Data covariance",
+                                                         G_VARIANT_TYPE ("aad"), NULL,
                                                          G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
 
-  g_object_class_install_property (object_class,
-                                   PROP_OFIT_MEAN,
-                                   g_param_spec_double ("ofit-mean",
-                                                        NULL,
-                                                        "Overfitting penalty mean",
-                                                        -1e300, 1e300, 0.0,
-                                                        G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  data_class->bootstrap          = TRUE;
+  
+  data_class->get_length         = &_ncm_data_gauss_cov_get_length;
+  data_class->begin              = NULL;
 
-  g_object_class_install_property (object_class,
-                                   PROP_OFIT_SIGMA,
-                                   g_param_spec_double ("ofit-sigma",
-                                                        NULL,
-                                                        "Overfitting penalty standard deviation",
-                                                        1e-300, 1e300, 1e-1,
-                                                        G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
-*/
-  data_class->get_length     = &_ncm_data_gauss_cov_get_length;
-  data_class->copyto         = &_ncm_data_gauss_cov_copyto;
-  data_class->begin          = NULL;
-
-  data_class->resample       = &_ncm_data_gauss_cov_resample;
-  data_class->m2lnL_val      = &_ncm_data_gauss_cov_m2lnL_val;
-  data_class->leastsquares_f = &_ncm_data_gauss_cov_leastsquares_f;
+  data_class->resample           = &_ncm_data_gauss_cov_resample;
+  data_class->m2lnL_val          = &_ncm_data_gauss_cov_m2lnL_val;
+  data_class->leastsquares_f     = &_ncm_data_gauss_cov_leastsquares_f;
 
   gauss_cov_class->mean_func = NULL;
   gauss_cov_class->cov_func  = NULL;
+  gauss_cov_class->set_size  = &_ncm_data_gauss_cov_set_size;
+  gauss_cov_class->get_size  = &_ncm_data_gauss_cov_get_size;
 }
 
 static guint 
@@ -248,23 +226,6 @@ _ncm_data_gauss_cov_get_length (NcmData *data)
   return NCM_DATA_GAUSS_COV (data)->np; 
 }
 
-static void
-_ncm_data_gauss_cov_copyto (NcmData *data, NcmData *data_dest)
-{
-  NcmDataGaussCov *src  = NCM_DATA_GAUSS_COV (data);
-  NcmDataGaussCov *dest = NCM_DATA_GAUSS_COV (data_dest);
-  
-  ncm_vector_memcpy (dest->y, src->y);
-  ncm_matrix_memcpy (dest->cov, src->cov);
-
-  g_assert (dest->LLT == NULL);
-  
-  if (src->LLT != NULL)
-    dest->LLT = ncm_matrix_dup (src->LLT);
-
-  ncm_data_set_init (data_dest);
-}
-
 static void
 _ncm_data_gauss_cov_prepare_LLT (NcmData *data)
 {
@@ -281,29 +242,30 @@ _ncm_data_gauss_cov_prepare_LLT (NcmData *data)
 }
 
 static void
-_ncm_data_gauss_cov_resample (NcmData *data, NcmMSet *mset)
+_ncm_data_gauss_cov_resample (NcmData *data, NcmMSet *mset, NcmRNG *rng)
 {
   NcmDataGaussCov *gauss = NCM_DATA_GAUSS_COV (data);
   NcmDataGaussCovClass *gauss_cov_class = NCM_DATA_GAUSS_COV_GET_CLASS (gauss);
   gboolean cov_update = FALSE;
-  gsl_rng *rng = ncm_cfg_rng_get ();
   gint ret;
-  gint i;
+  guint i;
 
   if (gauss_cov_class->cov_func != NULL)
     cov_update = gauss_cov_class->cov_func (gauss, mset, gauss->cov);
 
   if (cov_update || !gauss->prepared_LLT)
     _ncm_data_gauss_cov_prepare_LLT (data);
-  
+
+  ncm_rng_lock (rng);
   for (i = 0; i < gauss->np; i++)
   {
-    const gdouble u_i = gsl_ran_ugaussian (rng);
+    const gdouble u_i = gsl_ran_ugaussian (rng->r);
     ncm_vector_set (gauss->v, i, u_i);
   }
-
+  ncm_rng_unlock (rng);
+  
   ret = gsl_blas_dtrmv (CblasLower, CblasNoTrans, CblasNonUnit, 
-                        NCM_MATRIX_GSL (gauss->LLT), ncm_vector_gsl (gauss->v));
+                        ncm_matrix_gsl (gauss->LLT), ncm_vector_gsl (gauss->v));
   NCM_TEST_GSL_RESULT ("_ncm_data_gauss_cov_resample", ret);
 
   gauss_cov_class->mean_func (gauss, mset, gauss->y);
@@ -331,32 +293,58 @@ _ncm_data_gauss_cov_m2lnL_val (NcmData *data, NcmMSet *mset, gdouble *m2lnL)
     _ncm_data_gauss_cov_prepare_LLT (data);
 
   ret = gsl_blas_dtrsv (CblasLower, CblasNoTrans, CblasNonUnit, 
-                        NCM_MATRIX_GSL (gauss->LLT), ncm_vector_gsl (gauss->v));
+                        ncm_matrix_gsl (gauss->LLT), ncm_vector_gsl (gauss->v));
   NCM_TEST_GSL_RESULT ("_ncm_data_gauss_cov_m2lnL_val", ret);
 
-  ret = gsl_blas_ddot (ncm_vector_gsl (gauss->v), 
-                       ncm_vector_gsl (gauss->v), 
-                       m2lnL);
-  NCM_TEST_GSL_RESULT ("_ncm_data_gauss_cov_m2lnL_val", ret);
-
-  if (gauss->use_det)
+  if (!ncm_data_bootstrap_enabled (data))
   {
-    gint i;
-    gdouble lndetL = 0.0;
-    *m2lnL += gauss->np * ncm_c_ln2pi ();
-    
-    for (i = 0; i < gauss->np; i++)
+    ret = gsl_blas_ddot (ncm_vector_gsl (gauss->v),
+                         ncm_vector_gsl (gauss->v),
+                         m2lnL);
+    NCM_TEST_GSL_RESULT ("_ncm_data_gauss_cov_m2lnL_val", ret);
+
+    if (gauss->use_det)
     {
-      lndetL += log (ncm_matrix_get (gauss->LLT, i, i));
+      guint i;
+      gdouble lndetL = 0.0;
+      *m2lnL += gauss->np * ncm_c_ln2pi ();
+
+      for (i = 0; i < gauss->np; i++)
+      {
+        lndetL += log (ncm_matrix_get (gauss->LLT, i, i));
+      }
+      *m2lnL += 2.0 * lndetL;
     }
-    *m2lnL += 2.0 * lndetL;
   }
-/*
-  if (gauss->use_ofit)
+  else
   {
-    *m2lnL += gsl_pow_2 ((*m2lnL - gauss->ofit_mean) / gauss->ofit_sigma);
+    const guint bsize = ncm_bootstrap_get_bsize (data->bstrap);
+    guint i;
+    g_assert (ncm_bootstrap_is_init (data->bstrap));
+    
+    if (gauss->use_det)
+    {
+      gdouble lndetL = 0.0;
+      *m2lnL += bsize * ncm_c_ln2pi ();
+      for (i = 0; i < bsize; i++)
+      {
+        guint k = ncm_bootstrap_get (data->bstrap, i);
+        const gdouble u_i = ncm_vector_get (gauss->v, k); 
+        lndetL += log (ncm_matrix_get (gauss->LLT, k, k));
+        *m2lnL += u_i * u_i;
+      }
+      *m2lnL += 2.0 * lndetL;
+    }
+    else
+    {
+      for (i = 0; i < bsize; i++)
+      {
+        guint k = ncm_bootstrap_get (data->bstrap, i);
+        const gdouble u_i = ncm_vector_get (gauss->v, k); 
+        *m2lnL += u_i * u_i;
+      }
+    }
   }
-*/
 }
 
 static void
@@ -367,6 +355,9 @@ _ncm_data_gauss_cov_leastsquares_f (NcmData *data, NcmMSet *mset, NcmVector *v)
   gboolean cov_update = FALSE;
   gint ret;
 
+  if (ncm_data_bootstrap_enabled (data))
+    g_error ("NcmDataGaussCov: does not support bootstrap with least squares");
+  
   gauss_cov_class->mean_func (gauss, mset, v);
   ncm_vector_sub (v, gauss->y);
   
@@ -377,21 +368,14 @@ _ncm_data_gauss_cov_leastsquares_f (NcmData *data, NcmMSet *mset, NcmVector *v)
     _ncm_data_gauss_cov_prepare_LLT (data);
 
   ret = gsl_blas_dtrsv (CblasLower, CblasNoTrans, CblasNonUnit, 
-                        NCM_MATRIX_GSL (gauss->LLT), ncm_vector_gsl (v));
+                        ncm_matrix_gsl (gauss->LLT), ncm_vector_gsl (v));
   NCM_TEST_GSL_RESULT ("_ncm_data_gauss_cov_leastsquares_f", ret);
 }
 
-/**
- * ncm_data_gauss_cov_set_size:
- * @gauss: a #NcmDataGauss
- * @np: FIXME
- *
- * FIXME
- * 
- */
-void 
-ncm_data_gauss_cov_set_size (NcmDataGaussCov *gauss, guint np)
+static void 
+_ncm_data_gauss_cov_set_size (NcmDataGaussCov *gauss, guint np)
 {
+  NcmData *data = NCM_DATA (gauss);
   if ((np == 0) || (np != gauss->np))
   {
     gauss->np = 0;
@@ -399,26 +383,55 @@ ncm_data_gauss_cov_set_size (NcmDataGaussCov *gauss, guint np)
     ncm_vector_clear (&gauss->v);
     ncm_matrix_clear (&gauss->cov);
     ncm_matrix_clear (&gauss->LLT);
+    data->init = FALSE;
   }
   if ((np != 0) && (np != gauss->np))
   {
+    NcmData *data = NCM_DATA (gauss);
     gauss->np  = np;
     gauss->y   = ncm_vector_new (gauss->np);
     gauss->v   = ncm_vector_new (gauss->np);
     gauss->cov = ncm_matrix_new (gauss->np, gauss->np);
+    if (ncm_data_bootstrap_enabled (data))
+    {
+      ncm_bootstrap_set_fsize (data->bstrap, np);
+      ncm_bootstrap_set_bsize (data->bstrap, np);
+    }
+    data->init = FALSE;
   }
 }
 
+static guint 
+_ncm_data_gauss_cov_get_size (NcmDataGaussCov *gauss)
+{
+  return gauss->np;
+}
+
 /**
- * ncm_data_gauss_cov_get_size:
- * @gauss: a #NcmDataGauss
+ * ncm_data_gauss_cov_set_size: (virtual set_size)
+ * @gauss: a #NcmDataGaussCov
+ * @np: data size.
  *
- * FIXME
+ * Sets the data size to @np.
+ * 
+ */
+void 
+ncm_data_gauss_cov_set_size (NcmDataGaussCov *gauss, guint np)
+{
+  NCM_DATA_GAUSS_COV_GET_CLASS (gauss)->set_size (gauss, np);
+}
+
+/**
+ * ncm_data_gauss_cov_get_size: (virtual get_size)
+ * @gauss: a #NcmDataGaussCov
+ *
+ * Gets the data size.
+ * 
+ * Returns: Data size.
  * 
- * Returns: FIXME
  */
 guint 
 ncm_data_gauss_cov_get_size (NcmDataGaussCov *gauss)
 {
-  return gauss->np;
+  return NCM_DATA_GAUSS_COV_GET_CLASS (gauss)->get_size (gauss);
 }
