@@ -39,7 +39,7 @@
 
 #include "math/ncm_fit.h"
 #include "math/ncm_cfg.h"
-#include "math/util.h"
+#include "math/ncm_util.h"
 #include "math/integral.h"
 #include "math/memory_pool.h"
 #include "math/ncm_fit_gsl_ls.h"
@@ -59,11 +59,16 @@
 enum
 {
   PROP_0,
-  PROP_MAXITER,
   PROP_LIKELIHOOD,
   PROP_MSET,
   PROP_STATE,
   PROP_GRAD_TYPE,
+  PROP_MAXITER,
+  PROP_M2LNL_RELTOL,
+  PROP_M2LNL_ABSTOL,
+  PROP_PARAMS_RELTOL,
+  PROP_EQC,
+  PROP_INEQC,
   PROP_SIZE,
 };
 
@@ -122,11 +127,12 @@ ncm_fit_constraint_free (NcmFitConstraint *fitc)
 static void
 ncm_fit_init (NcmFit *fit)
 {
-  fit->maxiter       = NCM_FIT_MAXITER;
-  fit->m2lnL_reltol  = NCM_FIT_DEFAULT_M2LNL_RELTOL * 0.0 + 1e-8;
-  fit->params_reltol = NC_HICOSMO_DEFAULT_PARAMS_RELTOL * 0.0 + 1e-8;
-  fit->timer = g_timer_new ();
-  fit->mtype = NCM_FIT_RUN_MSGS_NONE;
+  fit->maxiter       = 0;
+  fit->m2lnL_reltol  = 0.0;
+  fit->m2lnL_abstol  = 0.0;
+  fit->params_reltol = 0.0;
+  fit->timer         = g_timer_new ();
+  fit->mtype         = NCM_FIT_RUN_MSGS_NONE;
 
   fit->equality_constraints = g_ptr_array_sized_new (10);
   g_ptr_array_set_free_func (fit->equality_constraints, (GDestroyNotify) &ncm_fit_constraint_free);
@@ -143,7 +149,8 @@ _ncm_fit_constructed (GObject *object)
   {
     NcmFit *fit = NCM_FIT (object);
     gint n = ncm_dataset_get_n (fit->lh->dset);
-    gint n_priors = ncm_likelihood_priors_length (fit->lh);
+    gint n_priors = ncm_likelihood_priors_length_f (fit->lh) + 
+      ncm_likelihood_priors_length_m2lnL (fit->lh);
     gint data_dof = ncm_dataset_get_dof (fit->lh->dset);
 
     g_assert (ncm_dataset_all_init (fit->lh->dset));
@@ -158,14 +165,19 @@ _ncm_fit_constructed (GObject *object)
      * g_warning ("ncm_fit_new: mset object has 0 free parameters");
      * 
      */
+    
     {
       guint data_len   = n + n_priors;
       guint fparam_len = ncm_mset_fparam_len (fit->mset);
       gint dof         = data_dof + n_priors - fparam_len;
 
-      fit->fstate = ncm_fit_state_new (data_len, fparam_len, dof,
-                                       NCM_FIT_GET_CLASS (fit)->is_least_squares);
-
+      if (fit->fstate == NULL)
+        fit->fstate = ncm_fit_state_new (data_len, fparam_len, dof,
+                                         NCM_FIT_GET_CLASS (fit)->is_least_squares);
+      else
+        ncm_fit_state_set_all (fit->fstate, data_len, fparam_len, dof, 
+                               NCM_FIT_GET_CLASS (fit)->is_least_squares);
+      
       g_assert (data_len > 0);
     }
   }
@@ -179,9 +191,6 @@ _ncm_fit_set_property (GObject *object, guint prop_id, const GValue *value, GPar
 
   switch (prop_id)
   {
-    case PROP_MAXITER:
-      ncm_fit_set_maxiter (fit, g_value_get_uint (value));
-      break;
     case PROP_LIKELIHOOD:
       ncm_likelihood_clear (&fit->lh);
       fit->lh = g_value_dup_object (value);
@@ -190,9 +199,47 @@ _ncm_fit_set_property (GObject *object, guint prop_id, const GValue *value, GPar
       ncm_mset_clear (&fit->mset);
       fit->mset = g_value_dup_object (value);
       break;
+    case PROP_STATE:
+      ncm_fit_state_clear (&fit->fstate);
+      fit->fstate = g_value_dup_object (value);
+      break;
     case PROP_GRAD_TYPE:
       ncm_fit_set_grad_type (fit, g_value_get_enum (value));
       break;
+    case PROP_MAXITER:
+      ncm_fit_set_maxiter (fit, g_value_get_uint (value));
+      break;
+    case PROP_M2LNL_RELTOL:
+      ncm_fit_set_m2lnL_reltol (fit, g_value_get_double (value));
+      break;
+    case PROP_M2LNL_ABSTOL:
+      ncm_fit_set_m2lnL_abstol (fit, g_value_get_double (value));
+      break;
+    case PROP_PARAMS_RELTOL:
+      ncm_fit_set_params_reltol (fit, g_value_get_double (value));
+      break;
+    case PROP_EQC:
+    {
+      guint p = g_value_get_int (value);
+      GPtrArray *eqc = GINT_TO_POINTER (p);
+      if (eqc != fit->equality_constraints)
+      {
+        g_ptr_array_unref (fit->equality_constraints);
+        fit->equality_constraints = g_ptr_array_ref (eqc);
+      }
+      break;
+    }
+    case PROP_INEQC:
+    {
+      guint p = g_value_get_int (value);
+      GPtrArray *ineqc = GINT_TO_POINTER (p);
+      if (ineqc != fit->inequality_constraints)
+      {
+        g_ptr_array_unref (fit->inequality_constraints);
+        fit->inequality_constraints = g_ptr_array_ref (ineqc);
+      }
+      break;
+    }
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -207,18 +254,36 @@ _ncm_fit_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec
 
   switch (prop_id)
   {
-    case PROP_MAXITER:
-      g_value_set_uint (value, ncm_fit_get_maxiter (fit));
-      break;
     case PROP_LIKELIHOOD:
       g_value_set_object (value, fit->lh);
       break;
     case PROP_MSET:
       g_value_set_object (value, fit->mset);
       break;
+    case PROP_STATE:
+      g_value_set_object (value, fit->fstate);
+      break;
     case PROP_GRAD_TYPE:
       g_value_set_enum (value, fit->grad.gtype);
       break;
+    case PROP_MAXITER:
+      g_value_set_uint (value, ncm_fit_get_maxiter (fit));
+      break;
+    case PROP_M2LNL_RELTOL:
+      g_value_set_double (value, ncm_fit_get_m2lnL_reltol (fit));
+      break;
+    case PROP_M2LNL_ABSTOL:
+      g_value_set_double (value, ncm_fit_get_m2lnL_abstol (fit));
+      break;
+    case PROP_PARAMS_RELTOL:
+      g_value_set_double (value, ncm_fit_get_params_reltol (fit));
+      break;
+    case PROP_EQC:
+      g_value_set_int (value, GPOINTER_TO_INT (fit->equality_constraints));
+      break;
+    case PROP_INEQC:
+      g_value_set_int (value, GPOINTER_TO_INT (fit->inequality_constraints));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -277,27 +342,27 @@ ncm_fit_class_init (NcmFitClass *klass)
   klass->is_least_squares = FALSE;
   klass->reset            = &_ncm_fit_reset;
 
-  g_object_class_install_property (object_class,
-                                   PROP_MAXITER,
-                                   g_param_spec_uint ("maxiter",
-                                                      NULL,
-                                                      "Maximum number of interations",
-                                                      0, G_MAXUINT32, NCM_FIT_MAXITER,
-                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
   g_object_class_install_property (object_class,
                                    PROP_LIKELIHOOD,
                                    g_param_spec_object ("likelihood",
                                                         NULL,
                                                         "Likelihood object",
                                                         NCM_TYPE_LIKELIHOOD,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
   g_object_class_install_property (object_class,
                                    PROP_MSET,
                                    g_param_spec_object ("mset",
                                                         NULL,
                                                         "Model set object",
                                                         NCM_TYPE_MSET,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_STATE,
+                                   g_param_spec_object ("state",
+                                                        NULL,
+                                                        "Fit state object",
+                                                        NCM_TYPE_FIT_STATE,
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
   g_object_class_install_property (object_class,
                                    PROP_GRAD_TYPE,
                                    g_param_spec_enum ("grad-type",
@@ -306,6 +371,50 @@ ncm_fit_class_init (NcmFitClass *klass)
                                                       NCM_TYPE_FIT_GRAD_TYPE, NCM_FIT_GRAD_NUMDIFF_FORWARD,
                                                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));  
 
+  g_object_class_install_property (object_class,
+                                   PROP_MAXITER,
+                                   g_param_spec_uint ("maxiter",
+                                                      NULL,
+                                                      "Maximum number of interations",
+                                                      0, G_MAXUINT32, NCM_FIT_DEFAULT_MAXITER,
+                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+
+  g_object_class_install_property (object_class,
+                                   PROP_M2LNL_RELTOL,
+                                   g_param_spec_double ("m2lnL-reltol",
+                                                        NULL,
+                                                        "Relative tolarence in m2lnL",
+                                                        0.0, G_MAXDOUBLE, NCM_FIT_DEFAULT_M2LNL_RELTOL,
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_M2LNL_ABSTOL,
+                                   g_param_spec_double ("m2lnL-abstol",
+                                                        NULL,
+                                                        "Absolute tolarence in m2lnL",
+                                                        0.0, G_MAXDOUBLE, NCM_FIT_DEFAULT_M2LNL_ABSTOL,
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_PARAMS_RELTOL,
+                                   g_param_spec_double ("params-reltol",
+                                                        NULL,
+                                                        "Relative tolarence in fitted parameters",
+                                                        0.0, G_MAXDOUBLE, NCM_FIT_DEFAULT_PARAMS_RELTOL,
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_EQC,
+                                   g_param_spec_int ("equality-constraints",
+                                                     NULL,
+                                                     "Equality constraints pointer",
+                                                     G_MININT, G_MAXINT, 0,
+                                                     G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_INEQC,
+                                   g_param_spec_int ("inequality-constraints",
+                                                     NULL,
+                                                     "Inequality constraints pointer",
+                                                     G_MININT, G_MAXINT, 0,
+                                                     G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+
 }
 
 static void
@@ -314,7 +423,8 @@ _ncm_fit_reset (NcmFit *fit)
   ncm_mset_prepare_fparam_map (fit->mset);
   {
     guint n          = ncm_dataset_get_n (fit->lh->dset);
-    gint n_priors    = ncm_likelihood_priors_length (fit->lh);
+    gint n_priors    = ncm_likelihood_priors_length_f (fit->lh) + 
+      ncm_likelihood_priors_length_m2lnL (fit->lh);
     guint data_len   = n + n_priors;
     guint fparam_len = ncm_mset_fparam_len (fit->mset);
     gint data_dof    = ncm_dataset_get_dof (fit->lh->dset);
@@ -402,6 +512,21 @@ ncm_fit_copy_new (NcmFit *fit, NcmLikelihood *lh, NcmMSet *mset, NcmFitGradType
   return NCM_FIT_GET_CLASS (fit)->copy_new (fit, lh, mset, gtype);
 }
 
+/**
+ * ncm_fit_dup:
+ * @fit: a #NcmFit
+ * @ser: a #NcmSerialize
+ *
+ * Duplicates the #NcmFit object duplicating all its contents.
+ *
+ * Returns: (transfer full): FIXME
+ */
+NcmFit *
+ncm_fit_dup (NcmFit *fit, NcmSerialize *ser)
+{
+  return NCM_FIT (ncm_serialize_dup_obj (ser, G_OBJECT (fit)));
+}
+
 /**
  * ncm_fit_free:
  * @fit: a #NcmFit.
@@ -500,34 +625,6 @@ ncm_fit_set_grad_type (NcmFit *fit, NcmFitGradType gtype)
   }
 }
 
-/**
- * ncm_fit_ls_set_state:
- * @fit: a #NcmFit.
- * @prec: FIXME
- * @x: a #NcmVector.
- * @f: a #NcmVector.
- * @J: a #NcmMatrix.
- *
- * FIXME
- */
-void
-ncm_fit_ls_set_state (NcmFit *fit, gdouble prec, NcmVector *x, NcmVector *f, NcmMatrix *J)
-{
-  ncm_mset_fparams_set_vector (fit->mset, x);
-  g_assert (fit->fstate->is_least_squares);
-
-  fit->fstate->m2lnL_prec = prec;
-  fit->fstate->m2lnL = gsl_blas_dnrm2 (ncm_vector_gsl (f));
-
-  ncm_vector_memcpy (fit->fstate->ls_f, f);
-  
-  gsl_blas_dgemv (CblasTrans, 2.0, NCM_MATRIX_GSL(fit->fstate->ls_J), 
-                  ncm_vector_gsl (fit->fstate->ls_f), 0.0, 
-                  ncm_vector_gsl (fit->fstate->dm2lnL));
-
-  ncm_matrix_memcpy (fit->fstate->ls_J, J);
-}
-
 /**
  * ncm_fit_set_maxiter:
  * @fit: a #NcmFit.
@@ -555,6 +652,101 @@ ncm_fit_get_maxiter (NcmFit *fit)
   return fit->maxiter;
 }
 
+/**
+ * ncm_fit_set_m2lnL_reltol:
+ * @fit: a #NcmFit.
+ * @tol: FIXME.
+ *
+ * FIXME
+ */
+void 
+ncm_fit_set_m2lnL_reltol (NcmFit *fit, gdouble tol)
+{
+  fit->m2lnL_reltol = tol;
+}
+
+/**
+ * ncm_fit_get_m2lnL_reltol:
+ * @fit: a #NcmFit.
+ *
+ * FIXME
+ * 
+ * Returns: FIXME
+ */
+gdouble 
+ncm_fit_get_m2lnL_reltol (NcmFit *fit)
+{
+  return fit->m2lnL_reltol;
+}
+
+/**
+ * ncm_fit_set_m2lnL_abstol:
+ * @fit: a #NcmFit.
+ * @tol: FIXME.
+ *
+ * FIXME
+ */
+void 
+ncm_fit_set_m2lnL_abstol (NcmFit *fit, gdouble tol)
+{
+  fit->m2lnL_abstol = tol;
+}
+
+/**
+ * ncm_fit_get_m2lnL_abstol:
+ * @fit: a #NcmFit.
+ *
+ * FIXME
+ * 
+ * Returns: FIXME
+ */
+gdouble 
+ncm_fit_get_m2lnL_abstol (NcmFit *fit)
+{
+  return fit->m2lnL_abstol;
+}
+
+/**
+ * ncm_fit_set_params_reltol:
+ * @fit: a #NcmFit.
+ * @tol: FIXME.
+ *
+ * FIXME
+ */
+void 
+ncm_fit_set_params_reltol (NcmFit *fit, gdouble tol)
+{
+  fit->params_reltol = tol;
+}
+
+/**
+ * ncm_fit_get_params_reltol:
+ * @fit: a #NcmFit.
+ *
+ * FIXME
+ * 
+ * Returns: FIXME
+ */
+gdouble 
+ncm_fit_get_params_reltol (NcmFit *fit)
+{
+  return fit->params_reltol;
+}
+
+/**
+ * ncm_fit_set_params:
+ * @fit: a #NcmFit.
+ * @params: FIXME
+ *
+ * FIXME
+ * 
+ */
+void 
+ncm_fit_set_params (NcmFit *fit, NcmVector *params)
+{
+  ncm_mset_fparams_set_vector (fit->mset, params);
+}
+
 /**
  * ncm_fit_add_equality_constraint:
  * @fit: a #NcmFit.
@@ -653,8 +845,8 @@ ncm_fit_ls_covar (NcmFit *fit)
 {
   g_assert (fit->fstate->is_least_squares);
   ncm_fit_ls_J (fit, fit->fstate->ls_J);
-  gsl_multifit_covar (NCM_MATRIX_GSL (fit->fstate->ls_J), 0.0, 
-                      NCM_MATRIX_GSL (fit->fstate->covar));
+  gsl_multifit_covar (ncm_matrix_gsl (fit->fstate->ls_J), 0.0, 
+                      ncm_matrix_gsl (fit->fstate->covar));
   fit->fstate->has_covar = TRUE;
 }
 
@@ -810,16 +1002,18 @@ _ncm_fit_run_empty (NcmFit *fit, NcmFitRunMsgs mtype)
 {
   fit->mtype = mtype;
 
-  ncm_fit_m2lnL_val (fit, &fit->fstate->m2lnL);
+  ncm_fit_m2lnL_val (fit, &fit->fstate->m2lnL_curval);
   ncm_fit_log_step (fit);
 
-  fit->fstate->m2lnL_prec   = 0.0;
-  fit->fstate->has_covar    = FALSE;
-  fit->fstate->is_best_fit  = TRUE;
-  fit->fstate->elapsed_time = 0.0;
-  fit->fstate->niter        = 0;
-  fit->fstate->func_eval    = 0;
-  fit->fstate->grad_eval    = 0;
+  fit->fstate->m2lnL_prec  = 0.0;
+  fit->fstate->params_prec = 0.0;
+  
+  fit->fstate->has_covar     = FALSE;
+  fit->fstate->is_best_fit   = TRUE;
+  fit->fstate->elapsed_time  = 0.0;
+  fit->fstate->niter         = 0;
+  fit->fstate->func_eval     = 0;
+  fit->fstate->grad_eval     = 0;
   
   return TRUE;
 }
@@ -963,7 +1157,9 @@ ncm_fit_log_end (NcmFit *fit)
   {
     if (fit->mtype == NCM_FIT_RUN_MSGS_SIMPLE)
       g_message ("\n");
-    g_message ("#  Minimum found with precision: % 8.5e (|df|/f or |dx|)\n", fit->fstate->m2lnL_prec);
+    g_message ("#  Minimum found with precision: |df|/f = % 8.5e and |dx| = % 8.5e\n", 
+               ncm_fit_state_get_m2lnL_prec (fit->fstate), 
+               ncm_fit_state_get_params_prec (fit->fstate));
   }
   ncm_fit_log_state (fit);
   return;
@@ -979,7 +1175,7 @@ ncm_fit_log_end (NcmFit *fit)
 void
 ncm_fit_log_state (NcmFit *fit)
 {
-  gint i;
+  guint i;
   if (fit->mtype > NCM_FIT_RUN_MSGS_NONE)
   {
     gdouble elap_sec = g_timer_elapsed (fit->timer, NULL);
@@ -995,10 +1191,10 @@ ncm_fit_log_state (NcmFit *fit)
     g_message ("#  function evaluations [%06d]\n", fit->fstate->func_eval);
     g_message ("#  gradient evaluations [%06d]\n", fit->fstate->grad_eval);
     g_message ("#  degrees of freedom   [%06d]\n", fit->fstate->dof);
-    g_message ("#  m2lnL     = %20.15g\n", fit->fstate->m2lnL);
+    g_message ("#  m2lnL     = %20.15g\n", ncm_fit_state_get_m2lnL_curval (fit->fstate));
     g_message ("#  Fit parameters:\n#    ");
     for (i = 0; i < ncm_mset_fparam_len (fit->mset); i++)
-      g_message ("[% -20.15g] ", ncm_mset_fparam_get (fit->mset, i));
+      g_message ("% -20.15g ", ncm_mset_fparam_get (fit->mset, i));
     g_message ("\n");
   }
   return;
@@ -1034,12 +1230,12 @@ ncm_fit_log_finish (NcmFit *fit)
 {
   if (fit->mtype > NCM_FIT_RUN_MSGS_NONE)
   {
-    const gdouble m2lnL = fit->fstate->m2lnL;
+    const gdouble m2lnL = ncm_fit_state_get_m2lnL_curval (fit->fstate);
     const gint dof = fit->fstate->dof;
     const gdouble m2lnL_dof = m2lnL / dof;
     
     g_message ("#  m2lnL/dof = %20.15g\n", m2lnL_dof);
-    g_message ("#  |m2lnL-dof|/sqrt(2*dof) = %20.15g,\n", fabs(m2lnL_dof) / sqrt(2.0 * dof));
+    g_message ("#  |m2lnL-dof|/sqrt(2*dof) = %20.15g,\n", fabs (m2lnL_dof) / sqrt(2.0 * dof));
     g_message ("#  GoF_tt = %4.2f%% = (%4.2f + %4.2f)%%; GoF = %4.2f%%\n",
                gsl_cdf_chisq_Q (dof + fabs(dof - m2lnL), dof) * 100.0 +
                gsl_cdf_chisq_P (dof - fabs(dof - m2lnL), dof) * 100.0,
@@ -1073,7 +1269,7 @@ ncm_fit_log_info (NcmFit *fit)
 void
 ncm_fit_log_covar (NcmFit *fit)
 {
-  gint i, j;
+  guint i, j;
   guint name_size = ncm_mset_max_fparam_name (fit->mset);
   guint free_params_len = ncm_mset_fparam_len (fit->mset);
   gchar *box = "---------------";
@@ -1126,7 +1322,7 @@ ncm_fit_log_covar (NcmFit *fit)
 void
 ncm_fit_fishermatrix_print (NcmFit *fit, FILE *out, gchar *header)
 {
-  gint i, j;
+  guint i, j;
   guint name_size = ncm_mset_max_param_name (fit->mset);
   guint free_params_len = ncm_mset_fparam_len (fit->mset);
 
@@ -1268,7 +1464,7 @@ ncm_fit_m2lnL_grad_nd_fo (NcmFit *fit, NcmVector *grad)
 void
 ncm_fit_m2lnL_grad_nd_ce (NcmFit *fit, NcmVector *grad)
 {
-  gint i;
+  guint i;
   guint fparam_len = ncm_mset_fparam_len (fit->mset);
 
   for (i = 0; i < fparam_len; i++)
@@ -1325,7 +1521,7 @@ ncm_fit_m2lnL_grad_nd_ac (NcmFit *fit, NcmVector *grad)
   gsl_function F;
   __ncm_fit_numdiff_1 nd;
   guint fparam_len = ncm_mset_fparam_len (fit->mset);
-  gint i;
+  guint i;
 
   nd.fit = fit;
   F.params = &nd;
@@ -1387,7 +1583,7 @@ ncm_fit_m2lnL_val_grad_an (NcmFit *fit, gdouble *result, NcmVector *df)
 void
 ncm_fit_m2lnL_val_grad_nd_fo (NcmFit *fit, gdouble *m2lnL, NcmVector *grad)
 {
-  gint i;
+  guint i;
   guint fparam_len = ncm_mset_fparam_len (fit->mset);
 
   ncm_fit_m2lnL_val (fit, m2lnL);
@@ -1480,7 +1676,7 @@ ncm_fit_ls_J_an (NcmFit *fit, NcmMatrix *J)
 void
 ncm_fit_ls_J_nd_fo (NcmFit *fit, NcmMatrix *J)
 {
-  gint i;
+  guint i;
   guint fparam_len = ncm_mset_fparam_len (fit->mset);
 
   ncm_fit_ls_f (fit, fit->fstate->ls_f);
@@ -1494,7 +1690,8 @@ ncm_fit_ls_J_nd_fo (NcmFit *fit, NcmMatrix *J)
     const gdouble pph = p + htilde;
     const gdouble h = pph - p;
     const gdouble one_h = 1.0 / h;
-    gint k;
+    guint k;
+    
     ncm_mset_fparam_set (fit->mset, i, pph);
     ncm_fit_ls_f (fit, J_col_i);
     for (k = 0; k < ncm_vector_len (fit->fstate->ls_f); k++)
@@ -1518,7 +1715,7 @@ ncm_fit_ls_J_nd_fo (NcmFit *fit, NcmMatrix *J)
 void
 ncm_fit_ls_J_nd_ce (NcmFit *fit, NcmMatrix *J)
 {
-  gint i;
+  guint i;
   guint fparam_len = ncm_mset_fparam_len (fit->mset);
 
   for (i = 0; i < fparam_len; i++)
@@ -1531,7 +1728,7 @@ ncm_fit_ls_J_nd_ce (NcmFit *fit, NcmMatrix *J)
     const gdouble pmh = p - h;
     const gdouble twoh = pph - pmh;
     const gdouble one_2h = 1.0 / twoh;
-    gint k;
+    guint k;
 
     ncm_mset_fparam_set (fit->mset, i, pph);
     ncm_fit_ls_f (fit, J_col_i);
@@ -1652,7 +1849,7 @@ ncm_fit_numdiff_m2lnL_hessian (NcmFit *fit, NcmMatrix *H)
 {
   gsl_function F;
   _ncm_fit_numdiff_2 nd;
-  gint i, j;
+  guint i, j;
   gdouble fx;
   const gdouble target_err = 1e-5;
   guint free_params_len = ncm_mset_fparams_len (fit->mset);
@@ -1730,10 +1927,10 @@ ncm_fit_numdiff_m2lnL_covar (NcmFit *fit)
   ncm_matrix_memcpy (fit->fstate->covar, fit->fstate->hessian);
   ncm_matrix_scale (fit->fstate->covar, 0.5);
 
-  ret = gsl_linalg_cholesky_decomp (NCM_MATRIX_GSL (fit->fstate->covar));
+  ret = gsl_linalg_cholesky_decomp (ncm_matrix_gsl (fit->fstate->covar));
   if (ret == GSL_SUCCESS)
   {
-    ret = gsl_linalg_cholesky_invert (NCM_MATRIX_GSL (fit->fstate->covar));
+    ret = gsl_linalg_cholesky_invert (ncm_matrix_gsl (fit->fstate->covar));
     NCM_TEST_GSL_RESULT ("ncm_fit_numdiff_m2lnL_covar[gsl_linalg_cholesky_invert]", ret);
   }
   else if (ret == GSL_EDOM)
@@ -1747,10 +1944,10 @@ ncm_fit_numdiff_m2lnL_covar (NcmFit *fit)
     
     g_warning ("ncm_fit_numdiff_m2lnL_covar: covariance matrix not positive definite, errors are not trustworthy.");
     
-    ret1 = gsl_linalg_LU_decomp (NCM_MATRIX_GSL (LU), p, &signum);
+    ret1 = gsl_linalg_LU_decomp (ncm_matrix_gsl (LU), p, &signum);
     NCM_TEST_GSL_RESULT ("ncm_fit_numdiff_m2lnL_covar[gsl_linalg_LU_decomp]", ret1);
     
-    ret1 = gsl_linalg_LU_invert (NCM_MATRIX_GSL (LU), p, NCM_MATRIX_GSL (fit->fstate->covar));
+    ret1 = gsl_linalg_LU_invert (ncm_matrix_gsl (LU), p, ncm_matrix_gsl (fit->fstate->covar));
     NCM_TEST_GSL_RESULT ("ncm_fit_numdiff_m2lnL_covar[gsl_linalg_LU_invert]", ret1);
     
     gsl_permutation_free (p);
@@ -1776,7 +1973,11 @@ fit_dprob(gdouble val, gpointer p)
   ncm_fit_run (dprob_arg->fit_val, NCM_FIT_RUN_MSGS_NONE);
   if (dprob_arg->fit->mtype > NCM_FIT_RUN_MSGS_NONE)
     g_message (".");
-  return exp (-(dprob_arg->fit_val->fstate->m2lnL - dprob_arg->fit->fstate->m2lnL) / 2.0);
+  {
+    const gdouble m2lnL_fv = ncm_fit_state_get_m2lnL_curval (dprob_arg->fit_val->fstate);
+    const gdouble m2lnL = ncm_fit_state_get_m2lnL_curval (dprob_arg->fit->fstate);
+    return exp (-(m2lnL_fv - m2lnL) / 2.0);
+  }
 }
 
 /**
@@ -1795,13 +1996,15 @@ gdouble
 ncm_fit_prob (NcmFit *fit, NcmModelID mid, guint pid, gdouble a, gdouble b)
 {
   NcmFit *fit_val;
-  NcmMSet *mset_val = ncm_mset_dup (fit->mset);
+  NcmSerialize *ser = ncm_serialize_global ();
+  NcmMSet *mset_val = ncm_mset_dup (fit->mset, ser);
   gsl_integration_workspace **w;
   FitDProb dprob_arg;
   gdouble result, error;
   gint error_code;
   gsl_function F;
 
+  ncm_serialize_free (ser);
   ncm_mset_param_set_ftype (mset_val, mid, pid, NCM_PARAM_TYPE_FIXED);
   fit_val = ncm_fit_copy_new (fit, fit->lh, mset_val, fit->grad.gtype);
 
@@ -1845,10 +2048,12 @@ void
 ncm_fit_dprob (NcmFit *fit, NcmModelID mid, guint pid, gdouble a, gdouble b, gdouble step, gdouble norm)
 {
   NcmFit *fit_val;
-  NcmMSet *mset_val = ncm_mset_dup (fit->mset);
+  NcmSerialize *ser = ncm_serialize_global ();
+  NcmMSet *mset_val = ncm_mset_dup (fit->mset, ser);
   FitDProb dprob_arg;
   gdouble point;
 
+  ncm_serialize_free (ser);
   ncm_mset_param_set_ftype (mset_val, mid, pid, NCM_PARAM_TYPE_FIXED);
   fit_val = ncm_fit_copy_new (fit, fit->lh, mset_val, fit->grad.gtype);
 
@@ -1883,9 +2088,11 @@ void
 ncm_fit_lr_test_range (NcmFit *fit, NcmModelID mid, guint pid, gdouble start, gdouble stop, gdouble step)
 {
   NcmFit *fit_val;
-  NcmMSet *mset_val = ncm_mset_dup (fit->mset);
+  NcmSerialize *ser = ncm_serialize_global ();
+  NcmMSet *mset_val = ncm_mset_dup (fit->mset, ser);
   gdouble walk;
 
+  ncm_serialize_free (ser);
   ncm_mset_param_set_ftype (mset_val, mid, pid, NCM_PARAM_TYPE_FIXED);
   fit_val = ncm_fit_copy_new (fit, fit->lh, mset_val, fit->grad.gtype);
 
@@ -1893,13 +2100,17 @@ ncm_fit_lr_test_range (NcmFit *fit, NcmModelID mid, guint pid, gdouble start, gd
   {
     ncm_mset_param_set (fit_val->mset, mid, pid, walk);
     ncm_fit_run (fit_val, NCM_FIT_RUN_MSGS_NONE);
-    //g_message ("%g %g %g\n", walk, (fit_val->fstate->m2lnL - fit->fstate->m2lnL), gsl_cdf_chisq_Q (fit_val->fstate->m2lnL - fit->fstate->m2lnL, 1));
-    g_message ("%g %g %g %g %g\n", walk,
-               (fit_val->fstate->m2lnL - fit->fstate->m2lnL),
-               gsl_ran_chisq_pdf (fit_val->fstate->m2lnL - fit->fstate->m2lnL, 1),
-               gsl_cdf_chisq_Q (fit_val->fstate->m2lnL - fit->fstate->m2lnL, 1),
-               gsl_cdf_ugaussian_Q (sqrt(fit_val->fstate->m2lnL - fit->fstate->m2lnL))
-               );
+
+    {
+      const gdouble m2lnL_fv = ncm_fit_state_get_m2lnL_curval (fit_val->fstate);
+      const gdouble m2lnL = ncm_fit_state_get_m2lnL_curval (fit->fstate);
+      g_message ("%g %g %g %g %g\n", walk,
+                 (m2lnL_fv - m2lnL),
+                 gsl_ran_chisq_pdf (m2lnL_fv - m2lnL, 1),
+                 gsl_cdf_chisq_Q (m2lnL_fv - m2lnL, 1),
+                 gsl_cdf_ugaussian_Q (sqrt(m2lnL_fv - m2lnL))
+                 );
+    }
   }
 
   ncm_fit_free (fit_val);
@@ -1923,16 +2134,21 @@ gdouble
 ncm_fit_lr_test (NcmFit *fit, NcmModelID mid, guint pid, gdouble val, gint dof)
 {
   NcmFit *fit_val;
-  NcmMSet *mset_val = ncm_mset_dup (fit->mset);
+  NcmSerialize *ser = ncm_serialize_global ();
+  NcmMSet *mset_val = ncm_mset_dup (fit->mset, ser);
   gdouble result;
 
+  ncm_serialize_free (ser);
   ncm_mset_param_set_ftype (mset_val, mid, pid, NCM_PARAM_TYPE_FIXED);
   fit_val = ncm_fit_copy_new (fit, fit->lh, mset_val, fit->grad.gtype);
 
   ncm_mset_param_set (fit_val->mset, mid, pid, val);
   ncm_fit_run (fit_val, NCM_FIT_RUN_MSGS_NONE);
-  result = gsl_cdf_chisq_Q (fit_val->fstate->m2lnL - fit->fstate->m2lnL, dof);
-
+  {
+    const gdouble m2lnL_fv = ncm_fit_state_get_m2lnL_curval (fit_val->fstate);
+    const gdouble m2lnL = ncm_fit_state_get_m2lnL_curval (fit->fstate);
+    result = gsl_cdf_chisq_Q (m2lnL_fv - m2lnL, dof);
+  } 
   ncm_fit_free (fit_val);
   ncm_mset_free (mset_val);
   return result;
@@ -2010,7 +2226,7 @@ ncm_fit_function_error (NcmFit *fit, NcmMSetFunc *func, gdouble *x, gboolean pre
   gdouble result;
   gint ret;
 
-  ret = gsl_blas_dgemv (CblasNoTrans, 1.0, NCM_MATRIX_GSL (fit->fstate->covar), ncm_vector_gsl (v), 0.0, ncm_vector_gsl (tmp1));
+  ret = gsl_blas_dgemv (CblasNoTrans, 1.0, ncm_matrix_gsl (fit->fstate->covar), ncm_vector_gsl (v), 0.0, ncm_vector_gsl (tmp1));
   NCM_TEST_GSL_RESULT("ncm_fit_function_error[covar.v]", ret);
   ret = gsl_blas_ddot (ncm_vector_gsl (v), ncm_vector_gsl (tmp1), &result);
   NCM_TEST_GSL_RESULT("ncm_fit_function_error[v.covar.v]", ret);
@@ -2042,6 +2258,12 @@ ncm_fit_function_error (NcmFit *fit, NcmMSetFunc *func, gdouble *x, gboolean pre
 gdouble
 ncm_fit_function_cov (NcmFit *fit, NcmMSetFunc *func1, gdouble z1, NcmMSetFunc *func2, gdouble z2, gboolean pretty_print)
 {
+  NCM_UNUSED (fit);
+  NCM_UNUSED (func1);
+  NCM_UNUSED (z1);
+  NCM_UNUSED (func2);
+  NCM_UNUSED (z2);
+  NCM_UNUSED (pretty_print);
   g_assert_not_reached ();
   /*
    gdouble result, cor, s1, s2;
@@ -2052,19 +2274,19 @@ ncm_fit_function_cov (NcmFit *fit, NcmMSetFunc *func1, gdouble z1, NcmMSetFunc *
    NCM_FUNC_DF (func1, fit->mset, fit->pt, z1, tmp1);
    NCM_FUNC_DF (func2, fit->mset, fit->pt, z2, tmp2);
 
-   ret = gsl_blas_dgemv (CblasNoTrans, 1.0, NCM_MATRIX_GSL (fit->fstate->covar), ncm_vector_gsl (tmp1), 0.0, ncm_vector_gsl (fit->df));
+   ret = gsl_blas_dgemv (CblasNoTrans, 1.0, ncm_matrix_gsl (fit->fstate->covar), ncm_vector_gsl (tmp1), 0.0, ncm_vector_gsl (fit->df));
    NCM_TEST_GSL_RESULT("ncm_fit_function_error[covar.v]", ret);
    ret = gsl_blas_ddot (ncm_vector_gsl (fit->df), ncm_vector_gsl (tmp1), &result);
    NCM_TEST_GSL_RESULT("ncm_fit_function_error[v.covar.v]", ret);
    s1 = sqrt(result);
 
-   ret = gsl_blas_dgemv (CblasNoTrans, 1.0, NCM_MATRIX_GSL (fit->fstate->covar), ncm_vector_gsl (tmp2), 0.0, ncm_vector_gsl (fit->df));
+   ret = gsl_blas_dgemv (CblasNoTrans, 1.0, ncm_matrix_gsl (fit->fstate->covar), ncm_vector_gsl (tmp2), 0.0, ncm_vector_gsl (fit->df));
    NCM_TEST_GSL_RESULT("ncm_fit_function_error[covar.v]", ret);
    ret = gsl_blas_ddot (ncm_vector_gsl (fit->df), ncm_vector_gsl (tmp2), &result);
    NCM_TEST_GSL_RESULT("ncm_fit_function_error[v.covar.v]", ret);
    s2 = sqrt(result);
 
-   ret = gsl_blas_dgemv (CblasNoTrans, 1.0, NCM_MATRIX_GSL (fit->fstate->covar), ncm_vector_gsl (tmp1), 0.0, ncm_vector_gsl (fit->df));
+   ret = gsl_blas_dgemv (CblasNoTrans, 1.0, ncm_matrix_gsl (fit->fstate->covar), ncm_vector_gsl (tmp1), 0.0, ncm_vector_gsl (fit->df));
    NCM_TEST_GSL_RESULT("ncm_fit_function_error[covar.v]", ret);
    ret = gsl_blas_ddot (ncm_vector_gsl (fit->df), ncm_vector_gsl (tmp2), &result);
    NCM_TEST_GSL_RESULT("ncm_fit_function_error[v.covar.v]", ret);
