@@ -37,6 +37,7 @@
 
 #include "math/ncm_func_eval.h"
 #include "math/ncm_cfg.h"
+#include "math/ncm_util.h"
 #include <stdio.h>
 
 typedef struct _NcmFunctionEvalCtrl
@@ -66,7 +67,7 @@ func (gpointer data, gpointer empty)
 {
   NcmLoopFuncEval *arg = (NcmLoopFuncEval *)data;
   NcmFunctionEvalCtrl *ctrl = arg->ctrl;
-
+  NCM_UNUSED (empty);
   arg->lfunc (arg->i, arg->f, arg->data);
   g_slice_free (NcmLoopFuncEval, arg);
 
@@ -74,7 +75,7 @@ func (gpointer data, gpointer empty)
 
   ctrl->active_threads--;
   if (ctrl->active_threads == 0)
-	g_cond_signal (ctrl->finish);
+    g_cond_signal (ctrl->finish);
 
   g_mutex_unlock (ctrl->update);
 
@@ -98,8 +99,8 @@ ncm_func_eval_get_pool ()
   _NCM_MUTEX_LOCK (&create_lock);
   if (_function_thread_pool == NULL)
   {
-	_function_thread_pool = g_thread_pool_new (func, NULL, NCM_THREAD_POOL_MAX, TRUE, &err);
-	g_clear_error (&err);
+    _function_thread_pool = g_thread_pool_new (func, NULL, NCM_THREAD_POOL_MAX, TRUE, &err);
+    g_clear_error (&err);
   }
   _NCM_MUTEX_UNLOCK (&create_lock);
 
@@ -110,7 +111,9 @@ ncm_func_eval_get_pool ()
  * ncm_func_eval_set_max_threads:
  * @mt: new max threads to be used in the pool, -1 means unlimited
  *
- * Set the new maximun number of threads to be used by the pool
+ * Set the new maximun number of threads to be used by the pool. Note that this
+ * function is global changing this will affect every place which uses these
+ * functions.
  *
  */
 void
@@ -122,21 +125,27 @@ ncm_func_eval_set_max_threads (gint mt)
 }
 
 /**
- * ncm_func_eval_threaded_loop:
+ * ncm_func_eval_threaded_loop_nw:
  * @lfunc: (scope notified): #NcmLoopFunc to be evaluated in threads
  * @i: initial index
  * @f: final index
  * @data: pointer to be passed to @fl
+ * @nworkers: number of workers.
  *
- * Using the thread pool, evaluate @fl in each value of (@f-@i)/nthreads
+ * Using the thread pool, evaluate @fl in each value of (@f-@i)/@nwork.
  *
  */
 #if NCM_THREAD_POOL_MAX > 1
 void
-ncm_func_eval_threaded_loop (NcmLoopFunc lfunc, glong i, glong f, gpointer data)
+ncm_func_eval_threaded_loop_nw (NcmLoopFunc lfunc, glong i, glong f, gpointer data, guint nworkers)
 {
-  NcmFunctionEvalCtrl ctrl = {0, NULL, NULL};
-  gint nthreads, delta, res;
+  NcmFunctionEvalCtrl ctrl = {0, NULL, NULL, 
+#if !((GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 32))
+    {NULL},
+    {NULL},
+#endif
+  };
+  guint delta, res;
 
   ncm_func_eval_get_pool ();
 #if (GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 32)
@@ -149,47 +158,49 @@ ncm_func_eval_threaded_loop (NcmLoopFunc lfunc, glong i, glong f, gpointer data)
   ctrl.finish = &ctrl.finish_c;
 #endif
 
-  g_assert (f >= i);
-  nthreads = g_thread_pool_get_max_threads (_function_thread_pool);
-  delta = (f-i) / nthreads;
-  res = (f-i) % nthreads;
+  g_assert_cmpuint (f, >=, i);
+  g_assert_cmpuint (f - i, >, nworkers);
+  g_assert_cmpuint (nworkers, >, 0);
+
+  delta = (f-i) / nworkers;
+  res = (f-i) % nworkers;
 
   if (delta == 0)
   {
-	lfunc (i, f, data);
+    lfunc (i, f, data);
   }
   else
   {
-	GError *err = NULL;
-	glong li = i;
-	glong lf = delta + res;
-	ctrl.active_threads = nthreads;
+    GError *err = NULL;
+    glong li = i;
+    glong lf = delta + res;
+    ctrl.active_threads = nworkers;
 
-	do {
-	  NcmLoopFuncEval *arg = g_slice_new (NcmLoopFuncEval);
-	  arg->lfunc = lfunc;
-	  arg->i = li;
-	  arg->f = lf;
-	  arg->data = data;
-	  arg->ctrl = &ctrl;
-	  g_thread_pool_push (_function_thread_pool, arg, &err);
-	  li = lf;
-	  lf += delta;
-	} while (--nthreads);
+    do {
+      NcmLoopFuncEval *arg = g_slice_new (NcmLoopFuncEval);
+      arg->lfunc = lfunc;
+      arg->i = li;
+      arg->f = lf;
+      arg->data = data;
+      arg->ctrl = &ctrl;
+      g_thread_pool_push (_function_thread_pool, arg, &err);
+      li = lf;
+      lf += delta;
+    } while (--nworkers);
   }
 
   g_mutex_lock (ctrl.update);
   while (ctrl.active_threads != 0)
-	g_cond_wait (ctrl.finish, ctrl.update);
+    g_cond_wait (ctrl.finish, ctrl.update);
   g_mutex_unlock (ctrl.update);
 
   if (FALSE)
   {
-	printf  ("Unused:      %d\n", g_thread_pool_get_num_unused_threads ());fflush (stdout);
-	printf  ("Max Unused:  %d\n", g_thread_pool_get_max_unused_threads ());fflush (stdout);
-	printf  ("Running:     %d\n", g_thread_pool_get_num_threads (_function_thread_pool));fflush (stdout);
-	printf  ("Unprocessed: %d\n", g_thread_pool_unprocessed (_function_thread_pool));fflush (stdout);
-	printf  ("Unused:      %d\n", g_thread_pool_get_max_threads (_function_thread_pool));fflush (stdout);
+    printf  ("Unused:      %d\n", g_thread_pool_get_num_unused_threads ());fflush (stdout);
+    printf  ("Max Unused:  %d\n", g_thread_pool_get_max_unused_threads ());fflush (stdout);
+    printf  ("Running:     %d\n", g_thread_pool_get_num_threads (_function_thread_pool));fflush (stdout);
+    printf  ("Unprocessed: %d\n", g_thread_pool_unprocessed (_function_thread_pool));fflush (stdout);
+    printf  ("Unused:      %d\n", g_thread_pool_get_max_threads (_function_thread_pool));fflush (stdout);
   }
 
 #if (GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 32)
@@ -202,8 +213,28 @@ ncm_func_eval_threaded_loop (NcmLoopFunc lfunc, glong i, glong f, gpointer data)
 }
 #else
 void
-ncm_func_eval_threaded_loop (NcmLoopFunc lfunc, glong i, glong f, gpointer data)
+ncm_func_eval_threaded_loop_nw (NcmLoopFunc lfunc, glong i, glong f, gpointer data, guint nworkers)
 {
   lfunc (i, f, data);
 }
 #endif
+
+/**
+ * ncm_func_eval_threaded_loop:
+ * @lfunc: (scope notified): #NcmLoopFunc to be evaluated in threads
+ * @i: initial index
+ * @f: final index
+ * @data: pointer to be passed to @fl
+ *
+ * Using the thread pool, evaluate @fl in each value of (@f-@i)/nthreads
+ *
+ */
+void
+ncm_func_eval_threaded_loop (NcmLoopFunc lfunc, glong i, glong f, gpointer data)
+{
+  ncm_func_eval_get_pool ();
+  {
+    guint nthreads = g_thread_pool_get_max_threads (_function_thread_pool);
+    ncm_func_eval_threaded_loop_nw (lfunc, i, f, data, nthreads);
+  }
+}
