@@ -42,7 +42,7 @@
 #include "math/ncm_c.h"
 #include "math/ncm_cfg.h"
 #include "math/ncm_matrix.h"
-#include "math/util.h"
+#include "math/ncm_util.h"
 
 #include <gsl/gsl_cdf.h>
 #include <gsl/gsl_roots.h>
@@ -66,6 +66,7 @@ ncm_lh_ratio2d_init (NcmLHRatio2d *lhr2d)
 {
   lhr2d->fit         = NULL;
   lhr2d->constrained = NULL;
+  lhr2d->rng         = ncm_rng_new (NULL);
   lhr2d->pi[0].mid   = -1;
   lhr2d->pi[0].pid   = 0;
   lhr2d->pi[1].mid   = -1;
@@ -91,8 +92,11 @@ ncm_lh_ratio2d_constructed (GObject *object)
   G_OBJECT_CLASS (ncm_lh_ratio2d_parent_class)->constructed (object);
   {
     NcmLHRatio2d *lhr2d = NCM_LH_RATIO2D (object);
-    NcmMSet *mset = ncm_mset_dup (lhr2d->fit->mset);
+    NcmSerialize *ser = ncm_serialize_global ();
+    NcmMSet *mset = ncm_mset_dup (lhr2d->fit->mset, ser);
     gint i;
+
+    ncm_serialize_free (ser);
     g_assert (lhr2d->fit->fstate->is_best_fit);
 
     for (i = 0; i < 2; i++)
@@ -188,6 +192,7 @@ ncm_lh_ratio2d_dispose (GObject *object)
   ncm_fit_clear (&lhr2d->constrained);
   ncm_matrix_clear (&lhr2d->e_vec);
   ncm_vector_clear (&lhr2d->e_val);
+  ncm_rng_clear (&lhr2d->rng);
 
   /* Chain up : end */
   G_OBJECT_CLASS (ncm_lh_ratio2d_parent_class)->dispose (object);
@@ -302,9 +307,9 @@ _ncm_lh_ratio2d_prepare_coords (NcmLHRatio2d *lhr2d)
   ncm_matrix_set (cov, 1, 0, ncm_fit_covar_fparam_cov (lhr2d->fit, fpi2, fpi1));
   ncm_matrix_set (cov, 1, 1, ncm_fit_covar_fparam_cov (lhr2d->fit, fpi2, fpi2));
 
-  gsl_eigen_symmv (NCM_MATRIX_GSL (cov), 
+  gsl_eigen_symmv (ncm_matrix_gsl (cov), 
                    ncm_vector_gsl (lhr2d->e_val), 
-                   NCM_MATRIX_GSL (lhr2d->e_vec), 
+                   ncm_matrix_gsl (lhr2d->e_vec), 
                    vw);
   gsl_eigen_symmv_free (vw);
 
@@ -473,6 +478,7 @@ ncm_lh_ratio2d_log_border_found (NcmLHRatio2d *lhr2d, gdouble r)
 static gboolean 
 _ncm_lh_ratio2d_inside_interval (gdouble *p, const gdouble lb, const gdouble ub, const gdouble prec)
 {
+  NCM_UNUSED (prec);
   if (p[0] < lb)
   {
     if (ncm_cmp (p[0], lb, 1e-4) == 0)
@@ -527,8 +533,12 @@ ncm_lh_ratio2d_f (gdouble x, gpointer ptr)
   lhr2d->niter     += lhr2d->constrained->fstate->niter;
   lhr2d->func_eval += lhr2d->constrained->fstate->func_eval;
   lhr2d->grad_eval += lhr2d->constrained->fstate->grad_eval;
-  
-  return lhr2d->constrained->fstate->m2lnL - (lhr2d->fit->fstate->m2lnL + lhr2d->chisquare);
+
+  {
+    const gdouble m2lnL_const = ncm_fit_state_get_m2lnL_curval (lhr2d->constrained->fstate);
+    const gdouble m2lnL = ncm_fit_state_get_m2lnL_curval (lhr2d->fit->fstate);
+    return m2lnL_const - (m2lnL + lhr2d->chisquare);
+  }
 }
 
 static gdouble
@@ -814,6 +824,8 @@ _ncm_lh_ratio2d_set_angular_interval (NcmLHRatio2d *lhr2d, const gdouble da, gdo
 {
   gdouble val0 = ncm_lh_ratio2d_f (*theta0, lhr2d);
   gdouble val1 = ncm_lh_ratio2d_f (*theta1, lhr2d);
+  gdouble ada = da;
+  guint n = 1;
   g_assert (*theta1 > *theta0);
   g_assert (da > 0.0);
 
@@ -823,7 +835,10 @@ _ncm_lh_ratio2d_set_angular_interval (NcmLHRatio2d *lhr2d, const gdouble da, gdo
     {
       ncm_lh_ratio2d_log_param_val (lhr2d, ncm_c_radian_to_degree (ncm_c_radian_0_2pi (*theta0)), val0);
       *theta1 = *theta0;
-      *theta0 -= da;
+      *theta0 -= ada;
+      if (n % 10 == 0)
+        ada *= 0.95;
+      n++;
     } while ((val0 = ncm_lh_ratio2d_f (*theta0, lhr2d)) < 0);
   }
   else if (val1 > 0.0)
@@ -832,7 +847,10 @@ _ncm_lh_ratio2d_set_angular_interval (NcmLHRatio2d *lhr2d, const gdouble da, gdo
     {
       ncm_lh_ratio2d_log_param_val (lhr2d, ncm_c_radian_to_degree (ncm_c_radian_0_2pi (*theta1)), val1);
       *theta0 = *theta1;
-      *theta1 += da;
+      *theta1 += ada;
+      if (n % 10 == 0)
+        ada *= 0.95;
+      n++;
     } while ((val1 = ncm_lh_ratio2d_f (*theta1, lhr2d)) > 0);
   }
 }
@@ -854,7 +872,6 @@ ncm_lh_ratio2d_conf_region (NcmLHRatio2d *lhr2d, gdouble clevel, gdouble expecte
   GTimer *iter_timer = g_timer_new ();
   gdouble total_time = 0.0;
   gdouble init_x, init_y;
-  gsl_rng *rand = ncm_cfg_rng_get ();
   gint i, counter = -1;
   GList *points = NULL, *final_points = NULL;
   gboolean completed = FALSE;
@@ -870,7 +887,7 @@ ncm_lh_ratio2d_conf_region (NcmLHRatio2d *lhr2d, gdouble clevel, gdouble expecte
   lhr2d->chisquare = gsl_cdf_chisq_Qinv (1.0 - clevel, 2);
   lhr2d->shift[0] = 0.0;
   lhr2d->shift[1] = 0.0;
-  lhr2d->theta = gsl_rng_uniform (rand) * 2.0 * M_PI;
+  lhr2d->theta = gsl_rng_uniform (lhr2d->rng->r) * 2.0 * M_PI;
   
   switch (lhr2d->rtype)
   {
@@ -980,9 +997,9 @@ ncm_lh_ratio2d_conf_region (NcmLHRatio2d *lhr2d, gdouble clevel, gdouble expecte
     }
   }
 
-  total_time += g_timer_elapsed (iter_timer, NULL);
+  /* total_time += g_timer_elapsed (iter_timer, NULL); */
   g_timer_destroy (iter_timer);
-
+  
   {
     NcmLHRatio2dRegion *rg = ncm_lh_ratio2d_points_to_region (final_points, clevel);
     ncm_lh_ratio2d_points_free (final_points);
@@ -1007,6 +1024,9 @@ ncm_lh_ratio2d_fisher_border (NcmLHRatio2d *lhr2d, gdouble clevel, gdouble expec
   GList *points = NULL;
   gdouble theta;
   gdouble step;
+
+  NCM_UNUSED (mtype);
+  
   if (expected_np <= 1.0)
     expected_np = 600.0;
 
@@ -1076,9 +1096,7 @@ ncm_lh_ratio2d_region_free (NcmLHRatio2dRegion *rg)
 void 
 ncm_lh_ratio2d_region_clear (NcmLHRatio2dRegion **rg)
 {
-  if (*rg != NULL)
-    ncm_lh_ratio2d_region_free (*rg);
-  *rg = NULL;
+  g_clear_pointer (rg, &ncm_lh_ratio2d_region_free);
 }
 
 /**
