@@ -37,8 +37,16 @@
 #include "build_cfg.h"
 
 #include "math/ncm_mset.h"
+#include "math/ncm_serialize.h"
 #include "math/ncm_cfg.h"
 
+enum
+{
+  PROP_0,
+  PROP_VALID_MAP,
+  PROP_SIZE,
+};
+
 G_DEFINE_TYPE (NcmMSet, ncm_mset, G_TYPE_OBJECT);
 G_DEFINE_BOXED_TYPE (NcmMSetPIndex, ncm_mset_pindex, ncm_mset_pindex_dup, ncm_mset_pindex_free);
 
@@ -220,69 +228,16 @@ ncm_mset_ref (NcmMSet *mset)
 /**
  * ncm_mset_dup:
  * @mset: a #NcmMSet.
+ * @ser: a #NcmSerialize.
  *
  * FIXME
  *
  * Returns: (transfer full): a new #NcmMSet.
  */
 NcmMSet *
-ncm_mset_dup (NcmMSet *mset)
+ncm_mset_dup (NcmMSet *mset, NcmSerialize *ser)
 {
-  NcmMSet *mset_copy = ncm_mset_empty_new ();
-  NcmModelID mid;
-  for (mid = 0; mid < NCM_MODEL_MAX_ID; mid++)
-  {
-    if (mset->model[mid] != NULL)
-    {
-      NcmModel *m_copy = ncm_model_copy (ncm_mset_peek (mset, mid));
-      ncm_mset_set (mset_copy, m_copy);
-      ncm_model_free (m_copy);
-    }
-  }
-  return mset_copy;
-}
-
-/**
- * ncm_mset_copy:
- * @mset: a #NcmMSet.
- *
- * FIXME
- *
- * Returns: (transfer full): a new #NcmMSet.
- */
-NcmMSet *
-ncm_mset_copy (NcmMSet *mset)
-{
-  NcmMSet *mset_copy = ncm_mset_empty_new ();
-  NcmModelID mid;
-  for (mid = 0; mid < NCM_MODEL_MAX_ID; mid++)
-  {
-    if (mset->model[mid] != NULL)
-      ncm_mset_set (mset_copy, ncm_mset_peek (mset, mid));
-  }
-  return mset_copy;
-}
-
-/**
- * ncm_mset_copyto:
- * @mset_src: a #NcmMSet.
- * @mset_dest: a #NcmMSet.
- *
- * FIXME
- *
- */
-void
-ncm_mset_copyto (NcmMSet *mset_src, NcmMSet *mset_dest)
-{
-  guint i;
-  for (i = 0; i < NCM_MODEL_MAX_ID; i++)
-  {
-    if (mset_src->model[i] != NULL)
-    {
-      g_assert (mset_dest->model[i] != NULL);
-      ncm_model_copyto (mset_src->model[i], mset_dest->model[i]);
-    }
-  }
+  return NCM_MSET (ncm_serialize_dup_obj (ser, G_OBJECT (mset)));
 }
 
 /**
@@ -373,22 +328,26 @@ ncm_mset_remove (NcmMSet *mset, NcmModelID mid)
 void
 ncm_mset_set (NcmMSet *mset, NcmModel *model)
 {
-  NcmModelID mid = ncm_model_id (model);
-  guint model_len = ncm_model_len (model);
-  guint i;
+  g_assert (model != NULL);
+  {
+    NcmModelID mid = ncm_model_id (model);
+    guint model_len = ncm_model_len (model);
+    guint i;
 
-  if (mset->model[mid] != NULL)
-    ncm_mset_remove (mset, mid);
+    if (mset->model[mid] != NULL)
+      ncm_mset_remove (mset, mid);
 
-  mset->model[mid] = ncm_model_ref (model);
+    mset->model[mid] = ncm_model_ref (model);
 
-  g_array_set_size (mset->fpi_array[mid], model_len);
-  for (i = 0; i < model_len; i++)
-    g_array_index (mset->fpi_array[mid], gint, i) = -1;
+    g_array_set_size (mset->fpi_array[mid], model_len);
+    for (i = 0; i < model_len; i++)
+      g_array_index (mset->fpi_array[mid], gint, i) = -1;
 
-  mset->total_len += model_len;
+    mset->total_len += model_len;
 
-  mset->valid_map = FALSE;
+    if (mset->valid_map)
+      ncm_mset_prepare_fparam_map (mset);
+  }
 }
 
 /**
@@ -411,6 +370,28 @@ ncm_mset_exists (NcmMSet *mset, NcmModel *model)
     return FALSE;
 }
 
+/**
+ * ncm_mset_get_id_by_ns:
+ * @ns: FIXME
+ *
+ * FIXME
+ * 
+ * Returns: FIXME
+ */
+gint 
+ncm_mset_get_id_by_ns (const gchar *ns)
+{
+  NcmMSetClass *mset_class = g_type_class_ref (NCM_TYPE_MSET);
+  gpointer model_id;
+  gboolean has = g_hash_table_lookup_extended (mset_class->ns_table, ns, NULL, &model_id);
+
+  g_type_class_unref (mset_class);
+  if (has)
+    return GPOINTER_TO_INT (model_id);
+  else
+    return -1;
+}
+
 /**
  * ncm_mset_prepare_fparam_map:
  * @mset: FIXME
@@ -426,6 +407,7 @@ ncm_mset_prepare_fparam_map (NcmMSet *mset)
 
   mset->fparam_len = 0;
   g_array_set_size (mset->pi_array, 0);
+  g_ptr_array_set_size (mset->fullname_parray, 0);
 
   for (mid = 0; mid < NCM_MODEL_MAX_ID; mid++)
   {
@@ -434,7 +416,7 @@ ncm_mset_prepare_fparam_map (NcmMSet *mset)
       continue;
     else
     {
-      gint n_params = ncm_model_len (model);
+      guint n_params = ncm_model_len (model);
       for (pid = 0; pid < n_params; pid++)
       {
         if (ncm_model_param_get_ftype (model, pid) == NCM_PARAM_TYPE_FREE)
@@ -449,6 +431,8 @@ ncm_mset_prepare_fparam_map (NcmMSet *mset)
       }
     }
   }
+
+  g_ptr_array_set_size (mset->fullname_parray, mset->fparam_len);
   mset->valid_map = TRUE;
 }
 
@@ -770,6 +754,7 @@ ncm_mset_cmp (NcmMSet *mset0, NcmMSet *mset1, gboolean cmp_model)
   {
     NcmModel *model0 = ncm_mset_peek (mset0, mid);
     NcmModel *model1 = ncm_mset_peek (mset1, mid);
+
     if (model0 == NULL && model1 == NULL)
       continue;
     else if (model0 == NULL && model1 != NULL)
@@ -1098,6 +1083,27 @@ ncm_mset_fparams_get_vector (NcmMSet *mset, NcmVector *x)
   }
 }
 
+/**
+ * ncm_mset_fparams_get_vector_offset:
+ * @mset: a #NcmMSet
+ * @x: FIXME
+ * @offset: FIXME
+ *
+ * FIXME
+ *
+ */
+void
+ncm_mset_fparams_get_vector_offset (NcmMSet *mset, NcmVector *x, guint offset)
+{
+  guint fpi;
+
+  for (fpi = 0; fpi < mset->fparam_len; fpi++)
+  {
+    const NcmMSetPIndex pi = g_array_index (mset->pi_array, NcmMSetPIndex, fpi);
+    ncm_vector_set (x, fpi + offset, ncm_mset_param_get (mset, pi.mid, pi.pid));
+  }
+}
+
 /**
  * ncm_mset_fparams_set_vector:
  * @mset: a #NcmMSet
@@ -1178,7 +1184,7 @@ ncm_mset_fparams_len (NcmMSet *mset)
  *
  * FIXME
  *
- * Returns: FIXME
+ * Returns: (transfer none): FIXME
  */
 const gchar *
 ncm_mset_fparam_name (NcmMSet *mset, guint n)
@@ -1190,6 +1196,33 @@ ncm_mset_fparam_name (NcmMSet *mset, guint n)
   }
 }
 
+/**
+ * ncm_mset_fparam_full_name:
+ * @mset: a #NcmMSet
+ * @n: FIXME
+ *
+ * FIXME
+ *
+ * Returns: (transfer none): FIXME
+ */
+const gchar *
+ncm_mset_fparam_full_name (NcmMSet *mset, guint n)
+{
+  gchar *fullname;
+  g_assert (mset->valid_map && n < mset->fparam_len);
+  if ((fullname = g_ptr_array_index (mset->fullname_parray, n)) != NULL)
+    return fullname;
+  else
+  {
+    const NcmMSetPIndex pi = g_array_index (mset->pi_array, NcmMSetPIndex, n);
+    const gchar *model_nick = ncm_model_nick (ncm_mset_peek (mset, pi.mid));
+    const gchar *pname = ncm_mset_param_name (mset, pi.mid, pi.pid);
+    fullname = g_strdup_printf ("%s:%s", model_nick, pname);
+    g_ptr_array_index (mset->fullname_parray, n) = fullname;
+  }
+  return fullname;
+}
+
 /**
  * ncm_mset_fparam_get_scale:
  * @mset: a #NcmMSet
@@ -1366,7 +1399,7 @@ ncm_mset_save (NcmMSet *mset, gchar *filename, gboolean save_comment)
       GVariant *params = NULL;
       gchar *obj_name = NULL;
       gchar *group = g_strdup_printf ("%s", mset_class->model_desc[i].ns);
-      GVariant *model_var = ncm_cfg_serialize_to_variant (G_OBJECT (model));
+      GVariant *model_var = ncm_serialize_global_to_variant (G_OBJECT (model));
       guint nparams;
 
       g_variant_get (model_var, "{s@a{sv}}", &obj_name, &params);
@@ -1493,7 +1526,7 @@ ncm_mset_load (gchar *filename)
     }
 
     {
-      GObject *obj = ncm_cfg_create_from_string (obj_ser->str);
+      GObject *obj = ncm_serialize_global_from_string (obj_ser->str);
       g_assert (NCM_IS_MODEL (obj));
       if (ncm_mset_exists (mset, NCM_MODEL (obj)))
         g_error ("ncm_mset_load: a model ``%s'' already exists in NcmMSet.", obj_ser->str);
@@ -1515,12 +1548,80 @@ ncm_mset_init (NcmMSet *mset)
   mset->pi_array = g_array_sized_new (FALSE, TRUE, sizeof (NcmMSetPIndex), 20);
   for (i = 0; i < NCM_MODEL_MAX_ID; i++)
     mset->fpi_array[i] = g_array_sized_new (FALSE, TRUE, sizeof (gint), 10);
+
+  mset->fullname_parray = g_ptr_array_new ();
+  g_ptr_array_set_free_func (mset->fullname_parray, g_free);
+
   mset->valid_map = FALSE;
   mset->total_len = 0;
 }
 
 static void
-ncm_mset_dispose (GObject *object)
+_ncm_mset_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
+{
+  NcmMSet *mset = NCM_MSET (object);
+  g_return_if_fail (NCM_IS_MSET (object));
+
+  switch (prop_id)
+  {
+    case PROP_VALID_MAP:
+      g_value_set_boolean (value, mset->valid_map);
+      break;
+    default:
+    {
+      g_assert (prop_id >= PROP_SIZE);
+      if (prop_id >= PROP_SIZE + NCM_MODEL_MAX_ID)
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      else
+      {
+        guint mid = prop_id - PROP_SIZE;
+        g_value_set_object (value, ncm_mset_peek (mset, mid));
+      }
+      break;
+    }
+  }
+}
+
+static void
+_ncm_mset_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
+{
+  NcmMSet *mset = NCM_MSET (object);
+  g_return_if_fail (NCM_IS_MSET (object));
+
+  switch (prop_id)
+  {
+    case PROP_VALID_MAP:
+    {
+      if (g_value_get_boolean (value))
+      {
+        if (!mset->valid_map)
+          ncm_mset_prepare_fparam_map (mset);
+      }
+      else
+        mset->valid_map = FALSE;
+      break;
+    }
+    default:
+    {
+      g_assert (prop_id >= PROP_SIZE);
+      if (prop_id >= PROP_SIZE + NCM_MODEL_MAX_ID)
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      else
+      {
+        guint mid = prop_id - PROP_SIZE;
+        NcmModel *model = g_value_get_object (value);
+        if (model != NULL)
+          ncm_mset_set (mset, model);
+        else
+          ncm_mset_remove (mset, mid);
+      }
+      break;
+    }
+  }
+}
+
+static void
+_ncm_mset_dispose (GObject *object)
 {
   NcmMSet *mset = NCM_MSET (object);
   NcmModelID mid;
@@ -1536,7 +1637,7 @@ ncm_mset_dispose (GObject *object)
 }
 
 static void
-ncm_mset_finalize (GObject *object)
+_ncm_mset_finalize (GObject *object)
 {
   NcmMSet *mset = NCM_MSET (object);
   gint i;
@@ -1546,6 +1647,8 @@ ncm_mset_finalize (GObject *object)
 
   g_array_unref (mset->pi_array);
 
+  g_ptr_array_unref (mset->fullname_parray);
+
   /* Chain up : end */
   G_OBJECT_CLASS (ncm_mset_parent_class)->finalize (object);
 }
@@ -1555,12 +1658,27 @@ ncm_mset_class_init (NcmMSetClass *klass)
 {
   GObjectClass* object_class = G_OBJECT_CLASS (klass);
   guint i;
-  object_class->dispose = ncm_mset_dispose;
-  object_class->finalize = ncm_mset_finalize;
+
+  object_class->set_property = &_ncm_mset_set_property;
+  object_class->get_property = &_ncm_mset_get_property;
+  object_class->dispose      = &_ncm_mset_dispose;
+  object_class->finalize     = &_ncm_mset_finalize;
+
   klass->ns_table = g_hash_table_new (g_str_hash, g_str_equal);
+
+  g_object_class_install_property (object_class, PROP_VALID_MAP,
+                                   g_param_spec_boolean ("valid-map", NULL, "Valid properties map",
+                                                        FALSE, 
+                                                        G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+
   for (i = 0; i < NCM_MODEL_MAX_ID; i++)
   {
+    gchar *name = g_strdup_printf ("model-type-%u", i);
+    gchar *desc = g_strdup_printf ("Model type %u", i);
     klass->model_desc[i].init = FALSE;
+    g_object_class_install_property (object_class, PROP_SIZE + i,
+                                     g_param_spec_object (name, NULL, desc,
+                                                          NCM_TYPE_MODEL,
+                                                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT));    
   }
 }
-
