@@ -145,6 +145,7 @@ enum {
 static void
 nc_hicosmo_qlinear_init (NcHICosmoQLinear *qlinear)
 {
+  NCM_UNUSED (qlinear);
 }
 
 static void
@@ -162,8 +163,6 @@ nc_hicosmo_qlinear_class_init (NcHICosmoQLinearClass *klass)
   NcHICosmoClass* parent_class = NC_HICOSMO_CLASS (klass);
   NcmModelClass *model_class = NCM_MODEL_CLASS (klass);
 
-  object_class->set_property = &ncm_model_class_set_property;
-  object_class->get_property = &ncm_model_class_get_property;
   object_class->finalize     = &nc_hicosmo_qlinear_finalize;
 
   ncm_model_class_add_params (model_class, 7, 0, PROP_SIZE);
