@@ -163,34 +163,31 @@ continuity_prior_f (NcmMSet *mset, gpointer obj, const gdouble *x, gdouble *f)
   NcHICosmoQSpline *qspline = NC_HICOSMO_QSPLINE (ncm_mset_peek (mset, nc_hicosmo_id ()));
   NcHICosmoQSplineContPriorKnot *acp = (NcHICosmoQSplineContPriorKnot *) obj;
   const gdouble sigma = exp (nc_hicosmo_qspline_cont_prior_get_lnsigma (acp->qspline_cp, acp->knot));
-/*
-  const gdouble x_i = ncm_vector_get (qspline->q_z->xv, acp->knot);
-  const gdouble x_ip1 = ncm_vector_get (qspline->q_z->xv, acp->knot + 1);
-  const gdouble x_ip2 = ncm_vector_get (qspline->q_z->xv, acp->knot + 2);
-  const gdouble qz_i = ncm_spline_eval (qspline->q_z, x_i);
-  const gdouble qz_ip1 = ncm_spline_eval (qspline->q_z, x_ip1);
-  const gdouble qz_ip2 = ncm_spline_eval (qspline->q_z, x_ip2);
-  const gdouble mu = (qz_i + qz_ip2 - 2.0 * qz_ip1) * 0.5;
-*/
-  gdouble *x_ptr = ncm_vector_ptr (qspline->q_z->xv, acp->knot);
-  gdouble *y_ptr = ncm_vector_ptr (qspline->q_z->yv, acp->knot);
+  const gdouble abstol = nc_hicosmo_qspline_cont_prior_get_abstol (acp->qspline_cp);
+  const guint n = 5;
+  const gdouble zi = ncm_vector_get (qspline->q_z->xv, acp->knot);
+  const gdouble zip2 = ncm_vector_get (qspline->q_z->xv, acp->knot + 2);
+  gdouble x_ptr[n];
+  gdouble y_ptr[n];
+  gdouble w_ptr[n];
   gdouble c0, c1, cov00, cov01, cov11, chisq;
-  gdouble var = sigma * sigma;
-  gdouble w[] = {
-    1.0 / ((y_ptr[0] * y_ptr[0] + 1.0) * var), 
-    1.0 / ((y_ptr[1] * y_ptr[1] + 1.0) * var),
-    1.0 / ((y_ptr[2] * y_ptr[2] + 1.0) * var)
-  };
+  guint i;
 
-  gsl_fit_wlinear (x_ptr, 1, w, 1, y_ptr, 1, 3, &c0, &c1, &cov00, &cov01, &cov11, &chisq);
-  f[0] = sqrt (chisq);
-}
+  NCM_UNUSED (x);
+  
+  for (i = 0; i < n; i++)
+  {
+    const gdouble z = zi + (zip2 - zi) / (n - 1.0) * i;
+    const gdouble qz = ncm_spline_eval (qspline->q_z, z);
+    const gdouble reltol = qz * sigma;
+    const gdouble qz_weight = 1.0 / GSL_MAX (reltol * reltol, abstol * abstol);
+    x_ptr[i] = z;
+    y_ptr[i] = qz;
+    w_ptr[i] = qz_weight;
+  }
 
-static void
-continuity_constraint_f (NcmMSet *mset, gpointer obj, const gdouble *x, gdouble *f)
-{
-  continuity_prior_f (mset, obj, x, f);
-  f[0] = f[0] * f[0]; 
+  gsl_fit_wlinear (x_ptr, 1, w_ptr, 1, y_ptr, 1, n, &c0, &c1, &cov00, &cov01, &cov11, &chisq);
+  f[0] = chisq / n;
 }
 
 static void
@@ -212,14 +209,14 @@ _nc_hicosmo_spline_continuity_prior_free (gpointer obj)
  *
  */
 void
-nc_hicosmo_qspline_add_continuity_prior (NcHICosmoQSpline *qspline, NcmLikelihood *lh, gint knot, NcHICosmoQSplineContPrior *qspline_cp)
+nc_hicosmo_qspline_add_continuity_prior (NcHICosmoQSpline *qspline, NcmLikelihood *lh, guint knot, NcHICosmoQSplineContPrior *qspline_cp)
 {
   NcHICosmoQSplineContPriorKnot *cp = g_slice_new (NcHICosmoQSplineContPriorKnot);
   NcmMSetFunc *func = ncm_mset_func_new (continuity_prior_f, 0, 1, cp, _nc_hicosmo_spline_continuity_prior_free);
   g_assert (knot < qspline->nknots - 2);
   cp->knot = knot;
   cp->qspline_cp = nc_hicosmo_qspline_cont_prior_ref (qspline_cp);
-  ncm_likelihood_priors_add (lh, func);
+  ncm_likelihood_priors_add (lh, func, TRUE);
   return;
 }
 
@@ -228,23 +225,28 @@ nc_hicosmo_qspline_add_continuity_prior (NcHICosmoQSpline *qspline, NcmLikelihoo
  * @qspline: FIXME
  * @lh: FIXME
  * @sigma: FIXME
+ * @abstol: FIXME
  *
  * FIXME
  * 
  * Returns: (transfer full): FIXME
  */
 NcHICosmoQSplineContPrior *
-nc_hicosmo_qspline_add_continuity_priors (NcHICosmoQSpline *qspline, NcmLikelihood *lh, gdouble sigma)
+nc_hicosmo_qspline_add_continuity_priors (NcHICosmoQSpline *qspline, NcmLikelihood *lh, gdouble sigma, gdouble abstol)
 {
-  guint i;
-  NcHICosmoQSplineContPrior *qspline_cp = nc_hicosmo_qspline_cont_prior_new (qspline->nknots);
-  g_assert (sigma > 0);
-  
-  nc_hicosmo_qspline_cont_prior_set_all_lnsigma (qspline_cp, log (sigma));
-  for (i = 0; i < qspline->nknots - 2; i++)
-    nc_hicosmo_qspline_add_continuity_prior (qspline, lh, i, qspline_cp);
-  
-  return qspline_cp;
+  g_assert (sigma > 0 && qspline->nknots > 2);
+  {
+    NcHICosmoQSplineContPrior *qspline_cp = nc_hicosmo_qspline_cont_prior_new (qspline->nknots - 2);
+    guint i;
+
+    nc_hicosmo_qspline_cont_prior_set_all_lnsigma (qspline_cp, log (sigma));
+    nc_hicosmo_qspline_cont_prior_set_abstol (qspline_cp, abstol);
+    
+    for (i = 0; i < qspline->nknots - 2; i++)
+      nc_hicosmo_qspline_add_continuity_prior (qspline, lh, i, qspline_cp);
+
+    return qspline_cp;
+  }
 }
 
 /**
@@ -258,10 +260,10 @@ nc_hicosmo_qspline_add_continuity_priors (NcHICosmoQSpline *qspline, NcmLikeliho
  *
  */
 void
-nc_hicosmo_qspline_add_continuity_constraint (NcHICosmoQSpline *qspline, NcmFit *fit, gint knot, NcHICosmoQSplineContPrior *qspline_cp)
+nc_hicosmo_qspline_add_continuity_constraint (NcHICosmoQSpline *qspline, NcmFit *fit, guint knot, NcHICosmoQSplineContPrior *qspline_cp)
 {
   NcHICosmoQSplineContPriorKnot *cp = g_slice_new (NcHICosmoQSplineContPriorKnot);
-  NcmMSetFunc *func = ncm_mset_func_new (continuity_constraint_f, 0, 1, cp, _nc_hicosmo_spline_continuity_prior_free);
+  NcmMSetFunc *func = ncm_mset_func_new (continuity_prior_f, 0, 1, cp, _nc_hicosmo_spline_continuity_prior_free);
   g_assert (knot < qspline->nknots - 2);
   cp->knot = knot;
   cp->qspline_cp = nc_hicosmo_qspline_cont_prior_ref (qspline_cp);
@@ -442,8 +444,6 @@ nc_hicosmo_qspline_class_init (NcHICosmoQSplineClass *klass)
   NcHICosmoClass* parent_class = NC_HICOSMO_CLASS (klass);
   NcmModelClass *model_class = NCM_MODEL_CLASS (klass);
 
-  object_class->set_property = &ncm_model_class_set_property;
-  object_class->get_property = &ncm_model_class_get_property;
   object_class->constructed  = &_nc_hicosmo_qspline_constructed;
   object_class->dispose      = &nc_hicosmo_qspline_dispose;
   object_class->finalize     = &nc_hicosmo_qspline_finalize;
@@ -527,7 +527,6 @@ nc_hicosmo_qspline_class_init (NcHICosmoQSplineClass *klass)
                                                         "Spline object",
                                                         NCM_TYPE_SPLINE,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
-  
   g_object_class_install_property (object_class,
                                    PROP_Z_F,
                                    g_param_spec_double ("zf",
@@ -535,7 +534,7 @@ nc_hicosmo_qspline_class_init (NcHICosmoQSplineClass *klass)
                                                         "final redshift",
                                                         0.0, 100.0, 1.0,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
-
+  
   nc_hicosmo_set_H0_impl       (parent_class, &_nc_hicosmo_qspline_H0);
   nc_hicosmo_set_E2_impl       (parent_class, &_nc_hicosmo_qspline_E2);
   nc_hicosmo_set_dE2_dz_impl   (parent_class, &_nc_hicosmo_qspline_dE2_dz);
@@ -549,16 +548,15 @@ nc_hicosmo_qspline_class_init (NcHICosmoQSplineClass *klass)
 enum
 {
   PROP_CP_0,
-  PROP_CP_NKNOTS,
+  PROP_CP_SIZE,
 };
 
-G_DEFINE_TYPE (NcHICosmoQSplineContPrior, nc_hicosmo_qspline_cont_prior, G_TYPE_OBJECT);
+G_DEFINE_TYPE (NcHICosmoQSplineContPrior, nc_hicosmo_qspline_cont_prior, NCM_TYPE_MODEL);
 
 static void
 nc_hicosmo_qspline_cont_prior_init (NcHICosmoQSplineContPrior *qspline_cp)
 {
-  qspline_cp->nknots      = 0;
-  qspline_cp->ln_sigma    = NULL;
+  NCM_UNUSED (qspline_cp);
 }
 
 static void
@@ -582,8 +580,7 @@ nc_hicosmo_qspline_cont_prior_dispose (GObject *object)
 static void
 nc_hicosmo_qspline_cont_prior_finalize (GObject *object)
 {
-  NcHICosmoQSplineContPrior *qspline_cp = NC_HICOSMO_QSPLINE_CONT_PRIOR (object);
-  ncm_vector_free (qspline_cp->ln_sigma);
+
   /* Chain up : end */
   G_OBJECT_CLASS (nc_hicosmo_qspline_cont_prior_parent_class)->finalize (object);
 }
@@ -591,14 +588,12 @@ nc_hicosmo_qspline_cont_prior_finalize (GObject *object)
 static void
 nc_hicosmo_qspline_cont_prior_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
-  NcHICosmoQSplineContPrior *qspline_cp = NC_HICOSMO_QSPLINE_CONT_PRIOR (object); 
+/*  NcHICosmoQSplineContPrior *qspline_cp = NC_HICOSMO_QSPLINE_CONT_PRIOR (object); */ 
   g_return_if_fail (NC_IS_HICOSMO_QSPLINE_CONT_PRIOR (object));
+  NCM_UNUSED (value);
 
   switch (prop_id)
   {
-    case PROP_CP_NKNOTS:
-      nc_hicosmo_qspline_cont_prior_set_nknots (qspline_cp, g_value_get_uint (value));
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -608,54 +603,109 @@ nc_hicosmo_qspline_cont_prior_set_property (GObject *object, guint prop_id, cons
 static void
 nc_hicosmo_qspline_cont_prior_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
-  NcHICosmoQSplineContPrior *qspline_cp = NC_HICOSMO_QSPLINE_CONT_PRIOR (object);
+/*  NcHICosmoQSplineContPrior *qspline_cp = NC_HICOSMO_QSPLINE_CONT_PRIOR (object); */
   g_return_if_fail (NC_IS_HICOSMO_QSPLINE_CONT_PRIOR (object));
 
+  NCM_UNUSED (value);
   switch (prop_id)
   {
-    case PROP_CP_NKNOTS:
-      g_value_set_uint (value, nc_hicosmo_qspline_cont_prior_get_nknots (qspline_cp));
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
 
+static gboolean _nc_hicosmo_qspline_cont_prior_valid (NcmModel *model);
+NCM_MSET_MODEL_REGISTER_ID (nc_hicosmo_qspline_cont_prior, NC_TYPE_HICOSMO_QSPLINE_CONT_PRIOR);
+
 static void
 nc_hicosmo_qspline_cont_prior_class_init (NcHICosmoQSplineContPriorClass *klass)
 {
   GObjectClass* object_class = G_OBJECT_CLASS (klass);
+  NcmModelClass *model_class = NCM_MODEL_CLASS (klass);
 
-  object_class->constructed = nc_hicosmo_qspline_cont_prior_constructed;
-  object_class->dispose     = nc_hicosmo_qspline_cont_prior_dispose;
-  object_class->finalize    = nc_hicosmo_qspline_cont_prior_finalize;
-  
-  object_class->set_property = nc_hicosmo_qspline_cont_prior_set_property;
-  object_class->get_property = nc_hicosmo_qspline_cont_prior_get_property;
+  object_class->constructed  = nc_hicosmo_qspline_cont_prior_constructed;
+  object_class->dispose      = nc_hicosmo_qspline_cont_prior_dispose;
+  object_class->finalize     = nc_hicosmo_qspline_cont_prior_finalize;
 
-  g_object_class_install_property (object_class,
-                                   PROP_CP_NKNOTS,
-                                   g_param_spec_uint ("nknots",
-                                                      NULL,
-                                                      "Number of knots",
-                                                      3, G_MAXUINT, 3,
-                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  model_class->set_property  = nc_hicosmo_qspline_cont_prior_set_property;
+  model_class->get_property  = nc_hicosmo_qspline_cont_prior_get_property;
+
+  ncm_model_class_add_params (model_class, 1, 1, PROP_CP_SIZE);
+  ncm_model_class_set_name_nick (model_class, "Q Spline Cont Prior", "qspline_cp");
+
+  /**
+   * NcHICosmoQSplineContPrior:abstol:
+   *
+   * FIXME
+   */  
+  /**
+   * NcHICosmoQSplineContPrior:abstol-fit:
+   *
+   * FIXME
+   */
+  /**
+   * NcHICosmoQSplineContPrior:abstol-length:
+   *
+   * FIXME
+   */
+  ncm_model_class_set_sparam (model_class, NC_HICOSMO_QSPLINE_CONT_PRIOR_ABSTOL, "abstol", "abstol",
+                              0.0, G_MAXDOUBLE, 1.0e-1, 0.0, 1.0e-1,
+                              NCM_PARAM_TYPE_FREE);
+
+  /**
+   * NcHICosmoQSplineContPrior:lnsigma:
+   *
+   * FIXME
+   */  
+  /**
+   * NcHICosmoQSplineContPrior:lnsigma-fit:
+   *
+   * FIXME
+   */
+  /**
+   * NcHICosmoQSplineContPrior:lnsigma-length:
+   *
+   * FIXME
+   */
+  ncm_model_class_set_vparam (model_class, NC_HICOSMO_QSPLINE_CONT_PRIOR_LNSIGMA, 3, "lnsigma", "lnsigma",
+                              log (1e-200), log (1.0e200), fabs (log (1.0e-1)), 0.0, log (1.0e-1),
+                              NCM_PARAM_TYPE_FREE);
+
+  ncm_model_class_check_params_info (model_class);
+
+  ncm_mset_model_register_id (model_class, 
+                              "NcHICosmoQSplineContPrior",
+                              "NcHICosmoQSplineContPrior.",
+                              NULL);
+
+  model_class->valid = &_nc_hicosmo_qspline_cont_prior_valid;
+
+}
+
+static gboolean 
+_nc_hicosmo_qspline_cont_prior_valid (NcmModel *model)
+{
+  if (!NCM_MODEL_CLASS (nc_hicosmo_qspline_cont_prior_parent_class)->valid (model))
+    return FALSE;
+  /* Chain up : start */
+
+  return TRUE;
 }
 
 /**
  * nc_hicosmo_qspline_cont_prior_new: 
- * @nknots: FIXME
+ * @npriors: FIXME
  * 
  * FIXME
  * 
  * Returns: (transfer full): FIXME
  */
 NcHICosmoQSplineContPrior *
-nc_hicosmo_qspline_cont_prior_new (guint nknots)
+nc_hicosmo_qspline_cont_prior_new (guint npriors)
 {
   NcHICosmoQSplineContPrior *qspline_cp = g_object_new (NC_TYPE_HICOSMO_QSPLINE_CONT_PRIOR,
-                                                        "nknots", nknots,
+                                                        "lnsigma-length", npriors,
                                                         NULL);
   return qspline_cp;
 }
@@ -687,45 +737,6 @@ nc_hicosmo_qspline_cont_prior_free (NcHICosmoQSplineContPrior *qspline_cp)
   g_object_unref (qspline_cp);
 }
 
-/**
- * nc_hicosmo_qspline_cont_prior_set_nknots: 
- * @qspline_cp: FIXME
- * @nknots: FIXME
- * 
- * FIXME
- * 
- */
-void
-nc_hicosmo_qspline_cont_prior_set_nknots (NcHICosmoQSplineContPrior *qspline_cp, guint nknots)
-{
-  g_assert (nknots > 2);
-  if (nknots == 0 || (qspline_cp->nknots != nknots))
-  {
-    ncm_vector_clear (&qspline_cp->ln_sigma);
-    qspline_cp->nknots = 0;
-  }
-  
-  if (qspline_cp->nknots == 0 && (nknots > 0))
-  {
-    qspline_cp->nknots   = nknots;
-    qspline_cp->ln_sigma = ncm_vector_new (nknots - 2);
-  }
-}
-
-/**
- * nc_hicosmo_qspline_cont_prior_get_nknots: 
- * @qspline_cp: FIXME
- * 
- * FIXME
- * 
- * Returns: FIXME
- */
-guint
-nc_hicosmo_qspline_cont_prior_get_nknots (NcHICosmoQSplineContPrior *qspline_cp)
-{
-  return qspline_cp->nknots;
-}
-
 /**
  * nc_hicosmo_qspline_cont_prior_set_lnsigma: 
  * @qspline_cp: FIXME
@@ -738,8 +749,10 @@ nc_hicosmo_qspline_cont_prior_get_nknots (NcHICosmoQSplineContPrior *qspline_cp)
 void 
 nc_hicosmo_qspline_cont_prior_set_lnsigma (NcHICosmoQSplineContPrior *qspline_cp, guint i, gdouble ln_sigma)
 {
-  g_assert_cmpint (i, < ,qspline_cp->nknots - 2);
-  ncm_vector_set (qspline_cp->ln_sigma, i, ln_sigma);
+  NcmModel *model = NCM_MODEL (qspline_cp);
+  guint npriors = ncm_model_vparam_len (model, 0);
+  g_assert_cmpint (i, <, npriors);
+  ncm_model_orig_vparam_set (model, 0, i, ln_sigma);
 }
 
 /**
@@ -753,12 +766,12 @@ nc_hicosmo_qspline_cont_prior_set_lnsigma (NcHICosmoQSplineContPrior *qspline_cp
 void 
 nc_hicosmo_qspline_cont_prior_set_all_lnsigma (NcHICosmoQSplineContPrior *qspline_cp, gdouble ln_sigma)
 {
+  NcmModel *model = NCM_MODEL (qspline_cp);
+  guint npriors = ncm_model_vparam_len (model, 0);
   guint i;
 
-  for (i = 0; i < qspline_cp->nknots - 2; i++)
-  {
-    ncm_vector_set (qspline_cp->ln_sigma, i, ln_sigma);
-  }
+  for (i = 0; i < npriors; i++)
+    ncm_model_orig_vparam_set (model, 0, i, ln_sigma);
 }
 
 /**
@@ -773,6 +786,39 @@ nc_hicosmo_qspline_cont_prior_set_all_lnsigma (NcHICosmoQSplineContPrior *qsplin
 gdouble 
 nc_hicosmo_qspline_cont_prior_get_lnsigma (NcHICosmoQSplineContPrior *qspline_cp, guint i)
 {
-  g_assert_cmpint (i, < ,qspline_cp->nknots - 2);
-  return ncm_vector_get (qspline_cp->ln_sigma, i);  
+  NcmModel *model = NCM_MODEL (qspline_cp);
+  guint npriors = ncm_model_vparam_len (model, 0);
+  g_assert_cmpint (i, <, npriors);
+  return ncm_model_orig_vparam_get (model, 0, i); 
+}
+
+/**
+ * nc_hicosmo_qspline_cont_prior_set_abstol: 
+ * @qspline_cp: FIXME
+ * @abstol: FIXME
+ * 
+ * FIXME
+ * 
+ */
+void 
+nc_hicosmo_qspline_cont_prior_set_abstol (NcHICosmoQSplineContPrior *qspline_cp, gdouble abstol)
+{
+  NcmModel *model = NCM_MODEL (qspline_cp);
+  g_assert_cmpfloat (abstol, >, 0.0);
+  ncm_model_orig_param_set (model, 0, abstol);
+}
+
+/**
+ * nc_hicosmo_qspline_cont_prior_get_abstol: 
+ * @qspline_cp: FIXME
+ * 
+ * FIXME
+ * 
+ * Returns: FIXME
+ */
+gdouble 
+nc_hicosmo_qspline_cont_prior_get_abstol (NcHICosmoQSplineContPrior *qspline_cp)
+{
+  NcmModel *model = NCM_MODEL (qspline_cp);
+  return ncm_model_orig_param_get (model, 0);
 }
