@@ -45,30 +45,25 @@ G_BEGIN_DECLS
 typedef struct _NcDataBaoRDVClass NcDataBaoRDVClass;
 typedef struct _NcDataBaoRDV NcDataBaoRDV;
 
-struct _NcDataBaoRDVClass
-{
-  /*< private >*/
-  NcmDataGaussClass parent_class;
-};
-
 struct _NcDataBaoRDV
 {
   /*< private >*/
   NcmDataGauss parent_instance;
   NcDistance *dist;
   NcmVector *x;
-  NcDataBaoId id;
+};
+
+struct _NcDataBaoRDVClass
+{
+  /*< private >*/
+  NcmDataGaussClass parent_class;
 };
 
 GType nc_data_bao_rdv_get_type (void) G_GNUC_CONST;
 
 NcmData *nc_data_bao_rdv_new (NcDistance *dist, NcDataBaoId id);
 
-void nc_data_bao_rdv_set_size (NcDataBaoRDV *bao_rdv, guint np);
-guint nc_data_bao_rdv_get_size (NcDataBaoRDV *bao_rdv);
-
 void nc_data_bao_rdv_set_sample (NcDataBaoRDV *bao_rdv, NcDataBaoId id);
-NcDataBaoId nc_data_bao_rdv_get_sample (NcDataBaoRDV *bao_rdv);
 
 G_END_DECLS
 
