@@ -50,7 +50,6 @@ G_BEGIN_DECLS
 typedef struct _NcDataClusterNCountClass NcDataClusterNCountClass;
 typedef struct _NcDataClusterNCount NcDataClusterNCount;
 
-
 /**
  * NcDataClusterAbundanceId:
  * @NC_DATA_CLUSTER_ABUNDANCE_FIT: FIXME
@@ -65,12 +64,6 @@ typedef enum _NcDataClusterAbundanceId
   NC_DATA_CLUSTER_ABUNDANCE_NSAMPLES, /*< skip >*/
 } NcDataClusterAbundanceId;
 
-struct _NcDataClusterNCountClass
-{
-  /*< private >*/
-  NcmDataClass parent_class;
-};
-
 struct _NcDataClusterNCount
 {
   /*< private >*/
@@ -85,7 +78,7 @@ struct _NcDataClusterNCount
   NcmMatrix *lnM_obs;
   NcmMatrix *lnM_obs_params;
   gdouble area_survey;
-  glong np;
+  guint np;
   gdouble log_np_fac;
   gboolean use_true_data;
   gsl_histogram2d *completeness;
@@ -96,22 +89,38 @@ struct _NcDataClusterNCount
   gchar *rnd_name;
 };
 
+struct _NcDataClusterNCountClass
+{
+  /*< private >*/
+  NcmDataClass parent_class;
+};
+
 GType nc_data_cluster_ncount_get_type (void) G_GNUC_CONST;
 
 NcmData *nc_data_cluster_ncount_new (NcClusterAbundance *cad);
 
 NcDataClusterNCount *nc_data_cluster_ncount_ref (NcDataClusterNCount *ncount);
 void nc_data_cluster_ncount_free (NcDataClusterNCount *ncount);
+void nc_data_cluster_ncount_clear (NcDataClusterNCount **ncount);
+
+void nc_data_cluster_ncount_set_mass (NcDataClusterNCount *ncount, NcClusterMass *m);
+void nc_data_cluster_ncount_set_redshift (NcDataClusterNCount *ncount, NcClusterRedshift *z);
+void nc_data_cluster_ncount_set_lnM_true (NcDataClusterNCount *ncount, const NcmVector *v);
+void nc_data_cluster_ncount_set_z_true (NcDataClusterNCount *ncount, const NcmVector *v);
+void nc_data_cluster_ncount_set_lnM_obs (NcDataClusterNCount *ncount, const NcmMatrix *m);
+void nc_data_cluster_ncount_set_lnM_obs_params (NcDataClusterNCount *ncount, const NcmMatrix *m);
+void nc_data_cluster_ncount_set_z_obs (NcDataClusterNCount *ncount, const NcmMatrix *m);
+void nc_data_cluster_ncount_set_z_obs_params (NcDataClusterNCount *ncount, const NcmMatrix *m);
 
 NcmData *nc_data_cluster_ncount_binned_new (NcClusterAbundance *cad);
 void nc_data_cluster_ncount_binned_init_from_text_file_gkey (NcmData *data, gboolean obs, gchar *filename);
-void nc_data_cluster_ncount_binned_init_from_sampling (NcmData *data, NcmMSet *mset, NcmVector *nodes, gboolean obs, gdouble area_survey, gdouble lnMi, gdouble lnMf, gdouble photoz_sigma0, gdouble photoz_bias, gdouble lnM_sigma0, gdouble lnM_bias);
+void nc_data_cluster_ncount_binned_init_from_sampling (NcmData *data, NcmMSet *mset, NcmVector *nodes, gboolean obs, gdouble area_survey, gdouble lnMi, gdouble lnMf, gdouble photoz_sigma0, gdouble photoz_bias, gdouble lnM_sigma0, gdouble lnM_bias, NcmRNG *rng);
 void nc_data_cluster_ncount_binned_save (NcmData *data, gchar *filename);
 NcmMSetFunc *nc_data_cluster_ncount_binned_create_func (NcClusterAbundance *cad);
 
 NcmData *nc_data_cluster_ncount_binned_lnM_z_new (NcClusterAbundance *cad);
 void nc_data_cluster_ncount_true_data (NcmData *data, gboolean use_true_data);
-void nc_data_cluster_ncount_init_from_sampling (NcmData *data, NcmMSet *mset, NcClusterRedshift *clusterz, NcClusterMass *clusterm, gdouble area_survey);
+void nc_data_cluster_ncount_init_from_sampling (NcmData *data, NcmMSet *mset, NcClusterRedshift *clusterz, NcClusterMass *clusterm, gdouble area_survey, NcmRNG *rng);
 NcmData *nc_data_cluster_ncount_bin_data (NcmData *data, gsl_vector *nodes);
 gsl_histogram2d *nc_data_cluster_ncount_hist_lnM_z (NcmData *data, gsl_vector *lnM_nodes, gsl_vector *z_nodes);
 
