@@ -51,6 +51,7 @@ typedef struct _NcDataHubble NcDataHubble;
  * @NC_DATA_HUBBLE_MORESCO2012_BC03: FIXME
  * @NC_DATA_HUBBLE_MORESCO2012_MASTRO: FIXME
  * @NC_DATA_HUBBLE_BUSCA2013_BAO_WMAP: FIXME
+ * @NC_DATA_HUBBLE_RIESS2008_HST: FIXME
  *
  * FIXME
  */
@@ -61,33 +62,28 @@ typedef enum _NcDataHubbleId
   NC_DATA_HUBBLE_STERN2009, 
   NC_DATA_HUBBLE_MORESCO2012_BC03, 
   NC_DATA_HUBBLE_MORESCO2012_MASTRO,
-  NC_DATA_HUBBLE_BUSCA2013_BAO_WMAP, /*< private >*/
-  NC_DATA_HUBBLE_NSAMPLES,  /*< skip >*/
+  NC_DATA_HUBBLE_BUSCA2013_BAO_WMAP, 
+  NC_DATA_HUBBLE_RIESS2008_HST, /*< private >*/
+  NC_DATA_HUBBLE_NSAMPLES,      /*< skip >*/
 } NcDataHubbleId;
 
+struct _NcDataHubble
+{
+  /*< private >*/
+  NcmDataGaussDiag parent_instance;
+  NcmVector *x;
+};
+
 struct _NcDataHubbleClass
 {
   /*< private >*/
   NcmDataGaussDiagClass parent_class;
 };
 
-struct _NcDataHubble
-{
-  /*< private >*/
-  NcmDataGaussDiag parent_instance;
-  NcmVector *x;
-  NcDataHubbleId id;
-};
-
 GType nc_data_hubble_get_type (void) G_GNUC_CONST;
 
 NcmData *nc_data_hubble_new (NcDataHubbleId id);
-
-void nc_data_hubble_set_size (NcDataHubble *hubble, guint np);
-guint nc_data_hubble_get_size (NcDataHubble *hubble);
-
 void nc_data_hubble_set_sample (NcDataHubble *hubble, NcDataHubbleId id);
-NcDataHubbleId nc_data_hubble_get_sample (NcDataHubble *hubble);
 
 G_END_DECLS
 
