@@ -40,6 +40,7 @@
 #include "nc_data_snia_cov.h"
 #include "nc_snia_dist_cov.h"
 #include "math/ncm_model_ctrl.h"
+#include "math/ncm_cfg.h"
 
 #include <glib/gstdio.h>
 #ifdef NUMCOSMO_HAVE_CFITSIO
@@ -50,7 +51,27 @@ enum
 {
   PROP_0,
   PROP_SIGMA_PECZ,
-  PROP_DATA_FILENAME,
+  PROP_ZCMB,
+  PROP_ZHE,
+  PROP_SIGMA_Z,
+  PROP_MAG,
+  PROP_SIGMA_MAG,
+  PROP_WIDTH,
+  PROP_SIGMA_WIDTH,
+  PROP_COLOUR,
+  PROP_SIGMA_COLOUR,
+  PROP_THIRDPAR,
+  PROP_SIGMA_THIRDPAR,
+  PROP_DIAG_MAG_WIDTH,
+  PROP_DIAG_MAG_COLOUR,
+  PROP_DIAG_WIDTH_COLOUR,
+  PROP_ABSMAG_SET, 
+  PROP_VAR_MAG,
+  PROP_VAR_WIDTH,
+  PROP_VAR_COLOUR,
+  PROP_VAR_MAG_WIDTH,
+  PROP_VAR_MAG_COLOUR,
+  PROP_VAR_WIDTH_COLOUR,
   PROP_SIZE,
 };
 
@@ -89,9 +110,6 @@ nc_data_snia_cov_init (NcDataSNIACov *snia_cov)
   snia_cov->sigma_pecz        = 0.0;
   snia_cov->dataset           = NULL;
   snia_cov->dataset_len       = 0;
-  snia_cov->sigma_int         = NULL;
-
-  snia_cov->filename          = NULL;
 }
 
 static void
@@ -105,13 +123,78 @@ nc_data_snia_cov_set_property (GObject *object, guint prop_id, const GValue *val
     case PROP_SIGMA_PECZ:
       snia_cov->sigma_pecz = g_value_get_double (value);
       break;
-    case PROP_DATA_FILENAME:
+    case PROP_ZCMB:
+      ncm_vector_set_from_variant (snia_cov->z_cmb, g_value_get_variant (value));
+      break;
+    case PROP_ZHE:
+      ncm_vector_set_from_variant (snia_cov->z_he, g_value_get_variant (value));
+      break;
+    case PROP_SIGMA_Z:
+      ncm_vector_set_from_variant (snia_cov->sigma_z, g_value_get_variant (value));
+      break;
+    case PROP_MAG:
+      ncm_vector_set_from_variant (snia_cov->mag, g_value_get_variant (value));
+      break;
+    case PROP_SIGMA_MAG:
+      ncm_vector_set_from_variant (snia_cov->sigma_mag, g_value_get_variant (value));
+      break;
+    case PROP_WIDTH:
+      ncm_vector_set_from_variant (snia_cov->width, g_value_get_variant (value));
+      break;
+    case PROP_SIGMA_WIDTH:
+      ncm_vector_set_from_variant (snia_cov->sigma_width, g_value_get_variant (value));
+      break;
+    case PROP_COLOUR:
+      ncm_vector_set_from_variant (snia_cov->colour, g_value_get_variant (value));
+      break;
+    case PROP_SIGMA_COLOUR:
+      ncm_vector_set_from_variant (snia_cov->sigma_colour, g_value_get_variant (value));
+      break;
+    case PROP_THIRDPAR:
+      ncm_vector_set_from_variant (snia_cov->thirdpar, g_value_get_variant (value));
+      break;
+    case PROP_SIGMA_THIRDPAR:
+      ncm_vector_set_from_variant (snia_cov->sigma_thirdpar, g_value_get_variant (value));
+      break;
+    case PROP_DIAG_MAG_WIDTH:
+      ncm_vector_set_from_variant (snia_cov->diag_mag_width, g_value_get_variant (value));
+      break;
+    case PROP_DIAG_MAG_COLOUR:
+      ncm_vector_set_from_variant (snia_cov->diag_mag_colour, g_value_get_variant (value));
+      break;
+    case PROP_DIAG_WIDTH_COLOUR:
+      ncm_vector_set_from_variant (snia_cov->diag_width_colour, g_value_get_variant (value));
+      break;
+    case PROP_ABSMAG_SET:
     {
-      const gchar *filename = g_value_get_string (value);  
-      if (filename != NULL)
-        nc_data_snia_cov_load (snia_cov, filename);
+      guint lv = 0, i;
+      ncm_cfg_array_set_variant (snia_cov->dataset, g_value_get_variant (value));
+      for (i = 0; i < snia_cov->dataset->len; i++)
+      {
+        guint v = g_array_index (snia_cov->dataset, guint, i);
+        lv = GSL_MAX (lv, v);
+      }
+      snia_cov->dataset_len = lv + 1;
       break;
     }
+    case PROP_VAR_MAG:
+      ncm_matrix_set_from_variant (snia_cov->var_mag, g_value_get_variant (value));
+      break;
+    case PROP_VAR_WIDTH:
+      ncm_matrix_set_from_variant (snia_cov->var_width, g_value_get_variant (value));
+      break;
+    case PROP_VAR_COLOUR:
+      ncm_matrix_set_from_variant (snia_cov->var_colour, g_value_get_variant (value));
+      break;
+    case PROP_VAR_MAG_WIDTH:
+      ncm_matrix_set_from_variant (snia_cov->var_mag_width, g_value_get_variant (value));
+      break;
+    case PROP_VAR_MAG_COLOUR:
+      ncm_matrix_set_from_variant (snia_cov->var_mag_colour, g_value_get_variant (value));
+      break;
+    case PROP_VAR_WIDTH_COLOUR:
+      ncm_matrix_set_from_variant (snia_cov->var_width_colour, g_value_get_variant (value));
+      break;      
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -129,9 +212,69 @@ nc_data_snia_cov_get_property (GObject *object, guint prop_id, GValue *value, GP
     case PROP_SIGMA_PECZ:
       g_value_set_double (value, snia_cov->sigma_pecz);
       break;
-    case PROP_DATA_FILENAME:
-      g_value_set_string (value, snia_cov->filename);
+    case PROP_ZCMB:
+      g_value_take_variant (value, ncm_vector_peek_variant (snia_cov->z_cmb));
       break;
+    case PROP_ZHE:
+      g_value_take_variant (value, ncm_vector_peek_variant (snia_cov->z_he));
+      break;
+    case PROP_SIGMA_Z:
+      g_value_take_variant (value, ncm_vector_peek_variant (snia_cov->sigma_z));
+      break;
+    case PROP_MAG:
+      g_value_take_variant (value, ncm_vector_peek_variant (snia_cov->mag));
+      break;
+    case PROP_SIGMA_MAG:
+      g_value_take_variant (value, ncm_vector_peek_variant (snia_cov->sigma_mag));
+      break;
+    case PROP_WIDTH:
+      g_value_take_variant (value, ncm_vector_peek_variant (snia_cov->width));
+      break;
+    case PROP_SIGMA_WIDTH:
+      g_value_take_variant (value, ncm_vector_peek_variant (snia_cov->sigma_width));
+      break;
+    case PROP_COLOUR:
+      g_value_take_variant (value, ncm_vector_peek_variant (snia_cov->colour));
+      break;
+    case PROP_SIGMA_COLOUR:
+      g_value_take_variant (value, ncm_vector_peek_variant (snia_cov->sigma_colour));
+      break;
+    case PROP_THIRDPAR:
+      g_value_take_variant (value, ncm_vector_peek_variant (snia_cov->thirdpar));
+      break;
+    case PROP_SIGMA_THIRDPAR:
+      g_value_take_variant (value, ncm_vector_peek_variant (snia_cov->sigma_thirdpar));
+      break;
+    case PROP_DIAG_MAG_WIDTH:
+      g_value_take_variant (value, ncm_vector_peek_variant (snia_cov->diag_mag_width));
+      break;
+    case PROP_DIAG_MAG_COLOUR:
+      g_value_take_variant (value, ncm_vector_peek_variant (snia_cov->diag_mag_colour));
+      break;
+    case PROP_DIAG_WIDTH_COLOUR:
+      g_value_take_variant (value, ncm_vector_peek_variant (snia_cov->diag_width_colour));
+      break;
+    case PROP_ABSMAG_SET:
+      g_value_take_variant (value, ncm_cfg_array_to_variant (snia_cov->dataset, G_VARIANT_TYPE ("u")));
+      break;
+    case PROP_VAR_MAG:
+      g_value_take_variant (value, ncm_matrix_peek_variant (snia_cov->var_mag));
+      break;
+    case PROP_VAR_WIDTH:
+      g_value_take_variant (value, ncm_matrix_peek_variant (snia_cov->var_width));
+      break;
+    case PROP_VAR_COLOUR:
+      g_value_take_variant (value, ncm_matrix_peek_variant (snia_cov->var_colour));
+      break;
+    case PROP_VAR_MAG_WIDTH:
+      g_value_take_variant (value, ncm_matrix_peek_variant (snia_cov->var_mag_width));
+      break;
+    case PROP_VAR_MAG_COLOUR:
+      g_value_take_variant (value, ncm_matrix_peek_variant (snia_cov->var_mag_colour));
+      break;
+    case PROP_VAR_WIDTH_COLOUR:
+      g_value_take_variant (value, ncm_matrix_peek_variant (snia_cov->var_width_colour));
+      break;      
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -145,16 +288,17 @@ nc_data_snia_cov_constructed (GObject *object)
   G_OBJECT_CLASS (nc_data_snia_cov_parent_class)->constructed (object);
   {
 
-  
-  ncm_data_set_init (NCM_DATA (object));
+
+    ncm_data_set_init (NCM_DATA (object), TRUE);
   }
 }
 
 static void
 nc_data_snia_cov_dispose (GObject *object)
 {
-  NcDataSNIACov *snia_cov = NC_DATA_SNIA_COV (object);
-  nc_data_snia_cov_set_size (snia_cov, 0);
+/*  NcDataSNIACov *snia_cov = NC_DATA_SNIA_COV (object); */
+  NcmDataGaussCov *gauss = NCM_DATA_GAUSS_COV (object);
+  ncm_data_gauss_cov_set_size (gauss, 0);
 
   /* Chain up : end */
   G_OBJECT_CLASS (nc_data_snia_cov_parent_class)->dispose (object);
@@ -163,8 +307,7 @@ nc_data_snia_cov_dispose (GObject *object)
 static void
 nc_data_snia_cov_finalize (GObject *object)
 {
-  NcDataSNIACov *snia_cov = NC_DATA_SNIA_COV (object);
-  g_free (snia_cov->filename);
+  /* NcDataSNIACov *snia_cov = NC_DATA_SNIA_COV (object); */
   
   /* Chain up : end */
   G_OBJECT_CLASS (nc_data_snia_cov_parent_class)->finalize (object);
@@ -173,6 +316,7 @@ nc_data_snia_cov_finalize (GObject *object)
 static void _nc_data_snia_cov_prepare (NcmData *data, NcmMSet *mset);
 static void _nc_data_snia_cov_mean_func (NcmDataGaussCov *gauss, NcmMSet *mset, NcmVector *vp);
 static gboolean _nc_data_snia_cov_func (NcmDataGaussCov *gauss, NcmMSet *mset, NcmMatrix *cov);
+static void _nc_data_snia_cov_set_size (NcmDataGaussCov *gauss, guint np);
 
 static void
 nc_data_snia_cov_class_init (NcDataSNIACovClass *klass)
@@ -194,25 +338,165 @@ nc_data_snia_cov_class_init (NcDataSNIACovClass *klass)
                                                         "Error from SN Ia peculiar velocity",
                                                         0.0, 1.0e1, 5.0e-4,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
-
   g_object_class_install_property (object_class,
-                                   PROP_DATA_FILENAME,
-                                   g_param_spec_string ("data-filename",
-                                                        NULL,
-                                                        "Data filename",
-                                                        NULL,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+                                   PROP_ZCMB,
+                                   g_param_spec_variant ("z-cmb",
+                                                         NULL,
+                                                         "Data cmb redshifts",
+                                                         G_VARIANT_TYPE ("ad"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_ZHE,
+                                   g_param_spec_variant ("z-He",
+                                                         NULL,
+                                                         "Data He redshifts",
+                                                         G_VARIANT_TYPE ("ad"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_SIGMA_Z,
+                                   g_param_spec_variant ("sigma-z",
+                                                         NULL,
+                                                         "Redshifts standard deviation",
+                                                         G_VARIANT_TYPE ("ad"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_MAG,
+                                   g_param_spec_variant ("magnitudes",
+                                                         NULL,
+                                                         "Magnitudes",
+                                                         G_VARIANT_TYPE ("ad"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_SIGMA_MAG,
+                                   g_param_spec_variant ("sigma-mag",
+                                                         NULL,
+                                                         "Magnitudes standard deviation",
+                                                         G_VARIANT_TYPE ("ad"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_WIDTH,
+                                   g_param_spec_variant ("width",
+                                                         NULL,
+                                                         "Width",
+                                                         G_VARIANT_TYPE ("ad"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_SIGMA_WIDTH,
+                                   g_param_spec_variant ("sigma-width",
+                                                         NULL,
+                                                         "Width standard deviation",
+                                                         G_VARIANT_TYPE ("ad"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_COLOUR,
+                                   g_param_spec_variant ("colour",
+                                                         NULL,
+                                                         "Colour",
+                                                         G_VARIANT_TYPE ("ad"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_SIGMA_COLOUR,
+                                   g_param_spec_variant ("sigma-colour",
+                                                         NULL,
+                                                         "Colour standard deviation",
+                                                         G_VARIANT_TYPE ("ad"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_THIRDPAR,
+                                   g_param_spec_variant ("thirdpar",
+                                                         NULL,
+                                                         "Thirdpar",
+                                                         G_VARIANT_TYPE ("ad"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_SIGMA_THIRDPAR,
+                                   g_param_spec_variant ("sigma-thirdpar",
+                                                         NULL,
+                                                         "Thirdpar standard deviation",
+                                                         G_VARIANT_TYPE ("ad"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_DIAG_MAG_WIDTH,
+                                   g_param_spec_variant ("diag-mag-width",
+                                                         NULL,
+                                                         "Diagonal of the magnitude-width covariance",
+                                                         G_VARIANT_TYPE ("ad"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_DIAG_MAG_COLOUR,
+                                   g_param_spec_variant ("diag-mag-colour",
+                                                         NULL,
+                                                         "Diagonal of the magnitude-colour covariance",
+                                                         G_VARIANT_TYPE ("ad"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_DIAG_WIDTH_COLOUR,
+                                   g_param_spec_variant ("diag-width-colour",
+                                                         NULL,
+                                                         "Diagonal of the width-colour covariance",
+                                                         G_VARIANT_TYPE ("ad"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_ABSMAG_SET,
+                                   g_param_spec_variant ("absmag-set",
+                                                         NULL,
+                                                         "Absolute magnitude set",
+                                                         G_VARIANT_TYPE ("au"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_VAR_MAG,
+                                   g_param_spec_variant ("var-mag",
+                                                         NULL,
+                                                         "Magnitude variance",
+                                                         G_VARIANT_TYPE ("aad"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_VAR_WIDTH,
+                                   g_param_spec_variant ("var-width",
+                                                         NULL,
+                                                         "Width variance",
+                                                         G_VARIANT_TYPE ("aad"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_VAR_COLOUR,
+                                   g_param_spec_variant ("var-colour",
+                                                         NULL,
+                                                         "Colour variance",
+                                                         G_VARIANT_TYPE ("aad"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_VAR_MAG_WIDTH,
+                                   g_param_spec_variant ("var-mag-width",
+                                                         NULL,
+                                                         "Magnitude-width covariance",
+                                                         G_VARIANT_TYPE ("aad"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_VAR_MAG_COLOUR,
+                                   g_param_spec_variant ("var-mag-colour",
+                                                         NULL,
+                                                         "Magnitude-colour covariance",
+                                                         G_VARIANT_TYPE ("aad"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_VAR_WIDTH_COLOUR,
+                                   g_param_spec_variant ("var-width-colour",
+                                                         NULL,
+                                                         "Width-colour covariance",
+                                                         G_VARIANT_TYPE ("aad"), NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
 
   data_class->prepare    = &_nc_data_snia_cov_prepare;
   gauss_class->mean_func = &_nc_data_snia_cov_mean_func;
   gauss_class->cov_func  = &_nc_data_snia_cov_func;
-  
+  gauss_class->set_size  = &_nc_data_snia_cov_set_size;  
 }
 
 static void
 _nc_data_snia_cov_prepare (NcmData *data, NcmMSet *mset)
 {
   NcSNIADistCov *dcov = NC_SNIA_DIST_COV (ncm_mset_peek (mset, nc_snia_dist_cov_id ()));
+  NCM_UNUSED (data);
   nc_snia_dist_cov_prepare_if_needed (dcov, mset);
 }
 
@@ -263,103 +547,98 @@ nc_data_snia_cov_new (gboolean use_det)
 NcmData *
 nc_data_snia_cov_new_full (gchar *filename, gboolean use_det)
 {
-  return g_object_new (NC_TYPE_DATA_SNIA_COV,
-                       "use-det", use_det,
-                       "data-filename", filename,
-                       NULL);
+  NcmData *data = g_object_new (NC_TYPE_DATA_SNIA_COV,
+                                "use-det", use_det,
+                                NULL);
+  nc_data_snia_cov_load (NC_DATA_SNIA_COV (data), filename);
+  return data;
 }
 
-/**
- * nc_data_snia_cov_set_size:
- * @snia_cov: FIXME
- * @mu_len: FIXME
- * 
- * FIXME
- * 
- */
-void 
-nc_data_snia_cov_set_size (NcDataSNIACov *snia_cov, guint mu_len)
+static void 
+_nc_data_snia_cov_set_size (NcmDataGaussCov *gauss, guint mu_len)
 {
-  NcmDataGaussCov *gauss = NCM_DATA_GAUSS_COV (snia_cov);
-  
-  if (mu_len == 0 || mu_len != snia_cov->mu_len)
+  /* Chain up : start */
+  NCM_DATA_GAUSS_COV_CLASS (nc_data_snia_cov_parent_class)->set_size (gauss, mu_len);  
   {
-    ncm_data_gauss_cov_set_size (gauss, 0);
-    ncm_vector_clear (&snia_cov->z_cmb);
-    ncm_vector_clear (&snia_cov->z_he);
 
-    ncm_vector_clear (&snia_cov->mag);
-    ncm_vector_clear (&snia_cov->width);
-    ncm_vector_clear (&snia_cov->colour);
-    ncm_vector_clear (&snia_cov->thirdpar);
+    NcDataSNIACov *snia_cov = NC_DATA_SNIA_COV (gauss);
 
-    ncm_vector_clear (&snia_cov->sigma_z);
-    ncm_vector_clear (&snia_cov->sigma_mag);
-    ncm_vector_clear (&snia_cov->sigma_width);
-    ncm_vector_clear (&snia_cov->sigma_colour);
-    ncm_vector_clear (&snia_cov->sigma_thirdpar);
-
-    ncm_vector_clear (&snia_cov->diag_mag_width);
-    ncm_vector_clear (&snia_cov->diag_mag_colour);
-    ncm_vector_clear (&snia_cov->diag_width_colour);
-    
-    ncm_matrix_clear (&snia_cov->var_mag);
-    ncm_matrix_clear (&snia_cov->var_width);
-    ncm_matrix_clear (&snia_cov->var_colour);
-
-    ncm_matrix_clear (&snia_cov->var_mag_width);
-    ncm_matrix_clear (&snia_cov->var_mag_colour);
-    ncm_matrix_clear (&snia_cov->var_width_colour);
-
-    if (snia_cov->dataset != NULL)
+    if (mu_len == 0 || mu_len != snia_cov->mu_len)
     {
-      g_array_unref (snia_cov->dataset);
-      snia_cov->dataset     = NULL;
-      snia_cov->dataset_len = 0;
+      ncm_vector_clear (&snia_cov->z_cmb);
+      ncm_vector_clear (&snia_cov->z_he);
+
+      ncm_vector_clear (&snia_cov->mag);
+      ncm_vector_clear (&snia_cov->width);
+      ncm_vector_clear (&snia_cov->colour);
+      ncm_vector_clear (&snia_cov->thirdpar);
+
+      ncm_vector_clear (&snia_cov->sigma_z);
+      ncm_vector_clear (&snia_cov->sigma_mag);
+      ncm_vector_clear (&snia_cov->sigma_width);
+      ncm_vector_clear (&snia_cov->sigma_colour);
+      ncm_vector_clear (&snia_cov->sigma_thirdpar);
+
+      ncm_vector_clear (&snia_cov->diag_mag_width);
+      ncm_vector_clear (&snia_cov->diag_mag_colour);
+      ncm_vector_clear (&snia_cov->diag_width_colour);
+
+      ncm_matrix_clear (&snia_cov->var_mag);
+      ncm_matrix_clear (&snia_cov->var_width);
+      ncm_matrix_clear (&snia_cov->var_colour);
+
+      ncm_matrix_clear (&snia_cov->var_mag_width);
+      ncm_matrix_clear (&snia_cov->var_mag_colour);
+      ncm_matrix_clear (&snia_cov->var_width_colour);
+
+      if (snia_cov->dataset != NULL)
+      {
+        g_array_unref (snia_cov->dataset);
+        snia_cov->dataset     = NULL;
+        snia_cov->dataset_len = 0;
+      }
+      NCM_DATA (snia_cov)->init = FALSE;
     }
-    NCM_DATA (snia_cov)->init = FALSE;
-  }
 
-  if (mu_len > 0 && mu_len != snia_cov->mu_len)
-  {
-    ncm_data_gauss_cov_set_size (gauss, mu_len);
-    
-    snia_cov->mu_len           = mu_len;
+    if (mu_len > 0 && mu_len != snia_cov->mu_len)
+    {
+      snia_cov->mu_len           = mu_len;
 
-    snia_cov->z_cmb            = ncm_vector_new (mu_len);
-    snia_cov->z_he             = ncm_vector_new (mu_len);
+      snia_cov->z_cmb            = ncm_vector_new (mu_len);
+      snia_cov->z_he             = ncm_vector_new (mu_len);
 
-    g_assert_cmpuint (ncm_vector_len (gauss->y), ==, mu_len);
-    
-    snia_cov->mag              = ncm_vector_ref (gauss->y);
-    snia_cov->width            = ncm_vector_new (mu_len);
-    snia_cov->colour           = ncm_vector_new (mu_len);
-    snia_cov->thirdpar         = ncm_vector_new (mu_len);
+      g_assert_cmpuint (ncm_vector_len (gauss->y), ==, mu_len);
 
-    snia_cov->sigma_z          = ncm_vector_new (mu_len);
-    snia_cov->sigma_mag        = ncm_vector_new (mu_len);
-    snia_cov->sigma_width      = ncm_vector_new (mu_len);
-    snia_cov->sigma_colour     = ncm_vector_new (mu_len);
-    snia_cov->sigma_thirdpar   = ncm_vector_new (mu_len);
+      snia_cov->mag              = ncm_vector_ref (gauss->y);
+      snia_cov->width            = ncm_vector_new (mu_len);
+      snia_cov->colour           = ncm_vector_new (mu_len);
+      snia_cov->thirdpar         = ncm_vector_new (mu_len);
 
-    snia_cov->diag_mag_width    = ncm_vector_new (mu_len);
-    snia_cov->diag_mag_colour   = ncm_vector_new (mu_len);
-    snia_cov->diag_width_colour = ncm_vector_new (mu_len);
+      snia_cov->sigma_z          = ncm_vector_new (mu_len);
+      snia_cov->sigma_mag        = ncm_vector_new (mu_len);
+      snia_cov->sigma_width      = ncm_vector_new (mu_len);
+      snia_cov->sigma_colour     = ncm_vector_new (mu_len);
+      snia_cov->sigma_thirdpar   = ncm_vector_new (mu_len);
 
-    snia_cov->var_mag          = ncm_matrix_new (mu_len, mu_len);
-    snia_cov->var_width        = ncm_matrix_new (mu_len, mu_len);
-    snia_cov->var_colour       = ncm_matrix_new (mu_len, mu_len);
+      snia_cov->diag_mag_width    = ncm_vector_new (mu_len);
+      snia_cov->diag_mag_colour   = ncm_vector_new (mu_len);
+      snia_cov->diag_width_colour = ncm_vector_new (mu_len);
 
-    snia_cov->var_mag_width    = ncm_matrix_new (mu_len, mu_len);
-    snia_cov->var_mag_colour   = ncm_matrix_new (mu_len, mu_len);
-    snia_cov->var_width_colour = ncm_matrix_new (mu_len, mu_len);
+      snia_cov->var_mag          = ncm_matrix_new (mu_len, mu_len);
+      snia_cov->var_width        = ncm_matrix_new (mu_len, mu_len);
+      snia_cov->var_colour       = ncm_matrix_new (mu_len, mu_len);
 
-    snia_cov->dataset          = g_array_sized_new (FALSE, FALSE, sizeof (guint32), mu_len);
-    snia_cov->dataset_len      = 0;
+      snia_cov->var_mag_width    = ncm_matrix_new (mu_len, mu_len);
+      snia_cov->var_mag_colour   = ncm_matrix_new (mu_len, mu_len);
+      snia_cov->var_width_colour = ncm_matrix_new (mu_len, mu_len);
 
-    g_array_set_size (snia_cov->dataset, mu_len);
+      snia_cov->dataset          = g_array_sized_new (FALSE, FALSE, sizeof (guint32), mu_len);
+      snia_cov->dataset_len      = 0;
 
-    NCM_DATA (snia_cov)->init = FALSE;
+      g_array_set_size (snia_cov->dataset, mu_len);
+
+      NCM_DATA (snia_cov)->init = FALSE;
+    }
   }
 }
 
@@ -382,18 +661,6 @@ nc_data_snia_cov_load_txt (NcDataSNIACov *snia_cov, const gchar *filename)
   gchar *datafile = NULL;
   guint64 mu_len;
 
-  if (filename == NULL)
-    g_error ("nc_data_snia_cov_load: null filename");
-  else if (snia_cov->filename == NULL)
-  {
-    snia_cov->filename = g_strdup (filename);
-  }
-  else if (snia_cov->filename != filename && g_strcmp0 (filename, snia_cov->filename) != 0)
-  {
-    g_free (snia_cov->filename);
-    snia_cov->filename = g_strdup (filename);
-  }
-
   if (!g_key_file_load_from_file (snia_keyfile, filename, G_KEY_FILE_NONE, &error))
     g_error ("nc_data_snia_cov_load: invalid configuration: %s\n  %s\n", 
              filename, 
@@ -420,7 +687,7 @@ nc_data_snia_cov_load_txt (NcDataSNIACov *snia_cov, const gchar *filename)
                                   NC_DATA_SNIA_COV_DATA_LEN_KEY,
                                   &error);
 
-  nc_data_snia_cov_set_size (snia_cov, mu_len);
+  ncm_data_gauss_cov_set_size (NCM_DATA_GAUSS_COV (snia_cov), mu_len);
   
   _nc_data_snia_cov_load_snia_data (snia_cov, datafile);
 
@@ -514,7 +781,7 @@ nc_data_snia_cov_load_txt (NcDataSNIACov *snia_cov, const gchar *filename)
     _nc_data_snia_cov_load_matrix (datafile, snia_cov->var_width_colour);
   }
 
-  ncm_data_set_init (NCM_DATA (snia_cov));
+  ncm_data_set_init (NCM_DATA (snia_cov), TRUE);
  
   g_key_file_free (snia_keyfile);
 }
@@ -631,7 +898,7 @@ _nc_data_snia_cov_load_matrix (const gchar *filename, NcmMatrix *data)
   guint itens_len = 0;
   guint pad_start = 0;
   guint pad_end = 0;
-  gint64 matrix_len = 0;
+  guint64 matrix_len = 0;
   guint i, j;
   if (!g_file_get_contents (filename, &file, &length, &error))
     g_error ("_nc_data_snia_cov_load_matrix: cannot open file %s: %s", 
@@ -652,9 +919,9 @@ _nc_data_snia_cov_load_matrix (const gchar *filename, NcmMatrix *data)
   pad_start++;
   itens_len--;
   
-  if (matrix_len != NCM_MATRIX_NROWS (data))
-    g_error ("_nc_data_snia_cov_load_matrix: expected a %zux%zu matrix but got %"G_GINT64_MODIFIER"dx%"G_GINT64_MODIFIER"d", 
-             NCM_MATRIX_NROWS (data), NCM_MATRIX_NROWS (data), matrix_len, matrix_len);
+  if (matrix_len != ncm_matrix_nrows (data))
+    g_error ("_nc_data_snia_cov_load_matrix: expected a %ux%u matrix but got %"G_GINT64_MODIFIER"dx%"G_GINT64_MODIFIER"d", 
+             ncm_matrix_nrows (data), ncm_matrix_nrows (data), matrix_len, matrix_len);
   
   if (matrix_len * matrix_len != itens_len)
     g_error ("_nc_data_snia_cov_load_matrix: matrix header say %"G_GINT64_MODIFIER"d length but got %u\n", 
@@ -703,15 +970,6 @@ nc_data_snia_cov_load (NcDataSNIACov *snia_cov, const gchar *filename)
 
   if (filename == NULL)
     g_error ("nc_data_snia_cov_load: null filename");
-  else if (snia_cov->filename == NULL)
-  {
-    snia_cov->filename = g_strdup (filename);
-  }
-  else if (snia_cov->filename != filename && g_strcmp0 (filename, snia_cov->filename) != 0)
-  {
-    g_free (snia_cov->filename);
-    snia_cov->filename = g_strdup (filename);
-  }
 
   if (fits_open_file (&fptr, filename, READONLY, &status))
     NC_FITS_ERROR (status);
@@ -726,7 +984,7 @@ nc_data_snia_cov_load (NcDataSNIACov *snia_cov, const gchar *filename)
     glong nrows;
     if (fits_get_num_rows (fptr, &nrows, &status))
       NC_FITS_ERROR (status);
-    nc_data_snia_cov_set_size (snia_cov, nrows);
+    ncm_data_gauss_cov_set_size (NCM_DATA_GAUSS_COV (snia_cov), nrows);
   }
 
   {
@@ -763,7 +1021,7 @@ nc_data_snia_cov_load (NcDataSNIACov *snia_cov, const gchar *filename)
                           NULL, &status))
     NC_FITS_ERROR (status);
   {
-    gint i;
+    guint i;
     for (i = 0; i < snia_cov->mu_len; i++)
     {
       snia_cov->dataset_len = GSL_MAX (g_array_index (snia_cov->dataset, guint32, i) + 1, snia_cov->dataset_len);
@@ -789,7 +1047,7 @@ nc_data_snia_cov_load (NcDataSNIACov *snia_cov, const gchar *filename)
     }
   }
 
-  ncm_data_set_init (NCM_DATA (snia_cov));
+  ncm_data_set_init (NCM_DATA (snia_cov), TRUE);
 }
 
 /**
